@@ -1,4 +1,4 @@
-/* $OpenBSD: clientloop.c,v 1.415 2025/09/25 06:23:19 jsg Exp $ */
+/* $OpenBSD: clientloop.c,v 1.418 2025/11/27 02:18:48 dtucker Exp $ */
 /*
  * Author: Tatu Ylonen <ylo@cs.hut.fi>
  * Copyright (c) 1995 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
@@ -832,7 +832,7 @@ client_repledge(void)
 	/* Might be able to tighten pledge now that session is established */
 	if (options.control_master || options.control_path != NULL ||
 	    options.forward_x11 || options.fork_after_authentication ||
-	    can_update_hostkeys() ||
+	    options.pkcs11_provider != NULL || can_update_hostkeys() ||
 	    (session_ident != -1 && !session_setup_complete)) {
 		/* Can't tighten */
 		return;
@@ -1015,6 +1015,7 @@ static struct escape_help_text esc_txt[] = {
 	SUPPRESS_MUXCLIENT},
     {"B",  "send a BREAK to the remote system", SUPPRESS_NEVER},
     {"C",  "open a command line", SUPPRESS_MUXCLIENT|SUPPRESS_NOCMDLINE},
+    {"I",  "show connection information", SUPPRESS_NEVER},
     {"R",  "request rekey", SUPPRESS_NEVER},
     {"V/v",  "decrease/increase verbosity (LogLevel)", SUPPRESS_MUXCLIENT},
     {"^Z", "suspend ssh", SUPPRESS_MUXCLIENT},
@@ -1056,6 +1057,29 @@ print_escape_help(struct sshbuf *b, int escape_char, int mux_client,
 		fatal_fr(r, "sshbuf_putf");
 }
 
+static void
+process_escapes_I(struct sshbuf *berr, struct ssh *ssh, int escape_char) {
+	struct sshbuf *buf = NULL;
+	int r;
+
+	buf = sshbuf_new();
+	if (buf == NULL) return;
+
+	r = sshbuf_putf(buf, "%cI\r\n", escape_char);
+	if (r != 0)
+		fatal_fr(r, "sshbuf_putf");
+
+	r = sshbuf_put_conninfo(buf, ssh);
+	if (r != 0)
+		fatal_fr(r, "sshbuf_put_conninfo");
+
+	r = sshbuf_put(berr, sshbuf_ptr(buf), sshbuf_len(buf));
+	if (r != 0)
+		fatal_fr(r, "sshbuf_put");
+
+	sshbuf_free(buf);
+}
+
 /*
  * Process the characters one by one.
  */
@@ -1137,6 +1161,10 @@ process_escapes(struct ssh *ssh, Channel *c,
 					fatal_fr(r, "send packet");
 				continue;
 
+			case 'I':
+				process_escapes_I(berr, ssh, efc->escape_char);
+				continue;
+
 			case 'R':
 				if (ssh_compat_fellows(ssh, SSH_BUG_NOREKEY))
 					logit("Server does not "
@@ -1350,7 +1378,9 @@ client_loop(struct ssh *ssh, int have_pty, int escape_char_arg,
 	debug("Entering interactive session.");
 	session_ident = ssh2_chan_id;
 
-	if (options.control_master &&
+	if (options.pkcs11_provider != NULL)
+		debug("pledge: disabled (PKCS11Provider active)");
+	else if (options.control_master &&
 	    !option_clear_or_none(options.control_path)) {
 		debug("pledge: id");
 		if (pledge("stdio rpath wpath cpath unix inet dns recvfd sendfd proc exec id tty",
