@@ -98,6 +98,9 @@ enum
 # define	IPTOS_DSCP_CS6		0xc0
 # define	IPTOS_DSCP_CS7		0xe0
 #endif /* IPTOS_DSCP_CS0 */
+#ifndef IPTOS_DSCP_VA
+# define	IPTOS_DSCP_VA		0xb0
+#endif /* IPTOS_DSCP_VA */
 #ifndef IPTOS_DSCP_EF
 # define	IPTOS_DSCP_EF		0xb8
 #endif /* IPTOS_DSCP_EF */
@@ -945,24 +948,25 @@ struct winsize {
 #endif
 
 
-/* LDAP use conditions */
+/* LDAP lookup conditions */
 #undef USE_LDAP_STORE
-#if defined(USE_OPENSSL_STORE2) && \
-    defined(LDAP_ENABLED) && defined(USE_OPENSSL_ENGINE)
-   /* OpenSSL >= 1.1.1 */
-#  define USE_LDAP_STORE		1
-#endif
-
 #undef USE_X509_LOOKUP_STORE
-#if defined(USE_OPENSSL_STORE2) && defined(HAVE_X509_LOOKUP_STORE)
-   /* OpenSSL >= 3.0 */
-#  define USE_X509_LOOKUP_STORE		1
-#endif
-
 #undef USE_X509_LOOKUP_MYSTORE
-#if defined(USE_OPENSSL_STORE2) && !defined(HAVE_X509_LOOKUP_STORE)
-   /* OpenSSL 1.1.1* only */
+
+#ifdef USE_OPENSSL_STORE2		/* OpenSSL >= 1.1.1 */
+# ifdef LDAP_ENABLED
+#  ifdef USE_OPENSSL_ENGINE
+    /* LDAP engine module with X.509 find */
+#   define USE_LDAP_STORE		1
+#  else
+    /* TODO: LDAP provider */
+#  endif
+# endif
+# ifdef HAVE_X509_LOOKUP_STORE		/* OpenSSL >= 3.0 */
+#  define USE_X509_LOOKUP_STORE		1
+# else
 #  define USE_X509_LOOKUP_MYSTORE	1
+# endif
 #endif
 
 
@@ -1059,7 +1063,27 @@ struct winsize {
 #endif
 
 
-#ifdef HAVE_EVP_KEYMGMT_GET0_PROVIDER	/* OpenSSL >= 3.0.0 */
+#ifndef ENABLE_ETM_ALGORITHMS
+/* OpenSSH EtM algorithms are subject to prefix truncation i.e., "Terrapin
+ * Attack: Breaking SSH Channel Integrity By Sequence Number Manipulation".
+ *
+ * Note that you could enable EtM if build is with configuration option
+ * --enable-etm-mode, but with configuration preprocessor flags to disable
+ * its use by default.
+ * In latter case add -DWITHOUT_ETM_FUNCTIONALITY to flags.
+ *
+ * Disabling "Encrypt-then-MAC"(EtM) functionality also disables "strict key
+ * exchange" as it is useless.
+ *
+ * Note that if these algorithms are listed protocol allows to select them
+ * if remote peer does not support "strict key exchange" i.e., session is
+ * vulnerable to "Terrapin Attack".
+ */
+# define WITHOUT_ETM_FUNCTIONALITY
+#endif
+
+
+#ifdef HAVE_EVP_KEYMGMT_FETCH	/* OpenSSL >= 3.0.0 */
 # define ENABLE_KEY_PROVIDERS
 #endif
 
