@@ -1,4 +1,4 @@
-/* $OpenBSD: hostfile.c,v 1.96 2025/04/30 05:23:15 djm Exp $ */
+/* $OpenBSD: hostfile.c,v 1.100 2025/11/25 00:57:04 djm Exp $ */
 /*
  * Author: Tatu Ylonen <ylo@cs.hut.fi>
  * Copyright (c) 1995 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
@@ -357,6 +357,9 @@ hostkey_match(const struct sshkey *key, const struct sshkey *found) {
 	return(sshkey_equal(key, found));
 }
 
+extern const struct sshkey_impl * /* TODO */
+sshkey_impl_from_type_nid(int type, int nid);
+
 /*
  * Match keys against a specified key, or look one up by key type.
  *
@@ -373,8 +376,9 @@ hostkey_match(const struct sshkey *key, const struct sshkey *found) {
  * Finally, check any found key is not revoked.
  */
 static HostStatus
-check_hostkeys_by_key_or_types(struct hostkeys *hostkeys,
-    struct sshkey *k, int keytype, int subtype, const struct hostkey_entry **found)
+check_hostkeys_by_key_or_impl(struct hostkeys *hostkeys,
+    struct sshkey *k, const struct sshkey_impl *impl,
+    const struct hostkey_entry **found)
 {
 	u_int i;
 	HostStatus end_return = HOST_NEW;
@@ -384,16 +388,13 @@ check_hostkeys_by_key_or_types(struct hostkeys *hostkeys,
 	if (found != NULL)
 		*found = NULL;
 	if (k != NULL)
-		subtype = k->ecdsa_nid;
+		impl = sshkey_impl_from_type_nid(k->type, k->ecdsa_nid);
 
 	for (i = 0; i < hostkeys->num_entries; i++) {
 		if (hostkeys->entries[i].marker != want_marker)
 			continue;
 		if (k == NULL) {
-			if (hostkeys->entries[i].key->type != keytype)
-				continue;
-			/* NOTE ecdsa_nid is -1 for non ECC keys */
-			if (subtype != -1 && subtype != hostkeys->entries[i].key->ecdsa_nid)
+			if (!sshkey_match_impl(hostkeys->entries[i].key, impl))
 				continue;
 			end_return = HOST_FOUND;
 			if (found != NULL)
@@ -437,14 +438,23 @@ check_key_in_hostkeys(struct hostkeys *hostkeys, struct sshkey *key,
 {
 	if (key == NULL)
 		fatal("no key to look up");
-	return check_hostkeys_by_key_or_types(hostkeys, key, 0, -1, found);
+	return check_hostkeys_by_key_or_impl(hostkeys, key, NULL, found);
 }
 
+int
+lookup_key_in_hostkeys_by_impl(struct hostkeys *hostkeys, const struct sshkey_impl *impl,
+    const struct hostkey_entry **found)
+{
+	return (check_hostkeys_by_key_or_impl(hostkeys, NULL, impl,
+	    found) == HOST_FOUND);
+}
 int
 lookup_key_in_hostkeys_by_types(struct hostkeys *hostkeys, int keytype, int subtype,
     const struct hostkey_entry **found)
 {
-	return (check_hostkeys_by_key_or_types(hostkeys, NULL, keytype, subtype,
+	const struct sshkey_impl *impl; /* TODO */
+	impl = sshkey_impl_from_type_nid(keytype, subtype);
+	return (check_hostkeys_by_key_or_impl(hostkeys, NULL, impl,
 	    found) == HOST_FOUND);
 }
 
@@ -484,13 +494,8 @@ write_host_entry(FILE *f, const char *host, const char *ip,
 		fprintf(f, "%s ", lhost);
 	}
 	free(lhost);
-	if (sshkey_is_x509(key)) {
-		/* key_write prints x509 certificate in blob format :-( */
-		r = Xkey_write_subject(NULL, key, f)
-		    ? SSH_ERR_SUCCESS
-		    : SSH_ERR_INTERNAL_ERROR;
-	} else
-		r = sshkey_write(key, f);
+
+	r = Xkey_printf(NULL, key, f);
 	if (r == 0)
 		success = 1;
 	else
@@ -635,7 +640,7 @@ hostfile_replace_entries(const char *filename, const char *host, const char *ip,
 {
 	int r, fd, oerrno = 0;
 	struct host_delete_ctx ctx;
-	char *fp, *temp = NULL, *back = NULL;
+	char *temp = NULL, *back = NULL;
 	const char *what;
 	mode_t omask;
 	size_t i;
@@ -689,14 +694,6 @@ hostfile_replace_entries(const char *filename, const char *host, const char *ip,
 	for (i = 0; i < nkeys; i++) {
 		if (keys[i] == NULL || (want & ctx.match_keys[i]) == want)
 			continue;
-		if (sshkey_is_x509(keys[i]))
-			fp = x509key_subject(keys[i]);
-		else
-			fp = sshkey_fingerprint(keys[i], hash_alg, SSH_FP_DEFAULT);
-		if (fp == NULL) {
-			r = SSH_ERR_ALLOC_FAIL;
-			goto fail;
-		}
 		/* write host/ip */
 		what = "";
 		if (ctx.match_keys[i] == 0) {
@@ -721,10 +718,18 @@ hostfile_replace_entries(const char *filename, const char *host, const char *ip,
 				goto fail;
 			}
 		}
+	{	char *fp = sshkey_is_x509(keys[i])
+		    ? x509key_subject(keys[i])
+		    : sshkey_fingerprint(keys[i], hash_alg, SSH_FP_DEFAULT);
+		if (fp == NULL) {
+			r = SSH_ERR_ALLOC_FAIL;
+			goto fail;
+		}
 		verbose("%s for %s%s%s to %s: %s %s", what,
 		    host, ip == NULL ? "" : ",", ip == NULL ? "" : ip, filename,
 		    sshkey_ssh_name(keys[i]), fp);
 		free(fp);
+	}
 		ctx.modified = 1;
 	}
 	fclose(ctx.out);
