@@ -1,4 +1,4 @@
-/* $OpenBSD: misc.c,v 1.208 2025/09/25 06:33:19 djm Exp $ */
+/* $OpenBSD: misc.c,v 1.211 2025/12/05 17:48:47 phessler Exp $ */
 /*
  * Copyright (c) 2000 Markus Friedl.  All rights reserved.
  * Copyright (c) 2005,2006 Damien Miller.  All rights reserved.
@@ -614,6 +614,107 @@ a2tun(const char *s, int *remote)
 #define DAYS		(HOURS * 24)
 #define WEEKS		(DAYS * 7)
 
+/*
+ * Convert an interval/duration time string into seconds, which may include
+ * fractional seconds.
+ *
+ * The format is a sequence of:
+ *      time[qualifier]
+ *
+ * This supports fractional values for the seconds value only. All other
+ * values must be integers.
+ *
+ * Valid time qualifiers are:
+ *      <none>  seconds
+ *      s|S     seconds
+ *      m|M     minutes
+ *      h|H     hours
+ *      d|D     days
+ *      w|W     weeks
+ *
+ * Examples:
+ *      90m      90 minutes
+ *      1h30m    90 minutes
+ *      1.5s     1.5 seconds
+ *      2d       2 days
+ *      1w       1 week
+ *
+ * Returns -1.0 if the time string is not in the specified format.
+ */
+double
+convtime_double(const char *s)
+{
+	double val, total_sec = 0.0, multiplier;
+	const char *p, *start_p;
+	char *endp;
+	int seen_seconds = 0;
+
+	if (s == NULL || *s == '\0')
+		return -1.0;
+
+	for (p = s; *p != '\0';) {
+		if (!isdigit((unsigned char)*p) && *p != '.')
+			return -1.0;
+
+		errno = 0;
+		val = strtod(p, &endp);
+		if (errno != 0 || p == endp)
+			return -1.0;
+		/* Allow only decimal forms */
+		if (p + strspn(p, "0123456789.") != endp)
+			return -1.0;
+		start_p = p;
+		p = endp;
+
+		switch (*p) {
+		case '\0':
+			/* FALLTHROUGH */
+		case 's':
+		case 'S':
+			if (seen_seconds++)
+				return -1.0;
+			multiplier = SECONDS;
+			break;
+		case 'm':
+		case 'M':
+			multiplier = MINUTES;
+			break;
+		case 'h':
+		case 'H':
+			multiplier = HOURS;
+			break;
+		case 'd':
+		case 'D':
+			multiplier = DAYS;
+			break;
+		case 'w':
+		case 'W':
+			multiplier = WEEKS;
+			break;
+		default:
+			return -1.0;
+		}
+
+		/* Special handling if this was a decimal */
+		if (memchr(start_p, '.', endp - start_p) != NULL) {
+			/* Decimal point present */
+			if (multiplier > 1.0)
+				return -1.0; /* No fractionals for non-seconds */
+		#if 0	/* NOTE: 1. is valid floating point number! */
+			/* For seconds, ensure digits follow */
+			if (!isdigit((unsigned char)*(endp - 1)))
+				return -1.0;
+		#endif
+		}
+
+		total_sec += val * multiplier;
+
+		if (*p != '\0')
+			p++;
+	}
+	return total_sec;
+}
+
 /*
  * Convert a time string into seconds; format is
  * a sequence of:
@@ -1916,6 +2017,7 @@ static const struct {
 	{ "cs7", IPTOS_DSCP_CS7 },
 	{ "ef", IPTOS_DSCP_EF },
 	{ "le", IPTOS_DSCP_LE },
+	{ "va",	IPTOS_DSCP_VA },
 	{ NULL, -1 }
 };
 
