@@ -1,4 +1,4 @@
-/* $OpenBSD: mux.c,v 1.107 2025/09/30 00:03:09 djm Exp $ */
+/* $OpenBSD: mux.c,v 1.109 2025/12/22 01:17:31 djm Exp $ */
 /*
  * Copyright (c) 2002-2008 Damien Miller <djm@openbsd.org>
  * Copyright (c) 2018-2025 Roumen Petrov.  All rights reserved.
@@ -116,6 +116,7 @@ struct mux_master_state {
 #define MUX_C_NEW_STDIO_FWD	0x10000008
 #define MUX_C_STOP_LISTENING	0x10000009
 #define MUX_C_PROXY		0x1000000f
+#define MUX_C_EXT_INFO		0x20000001
 #define MUX_S_OK		0x80000001
 #define MUX_S_PERMISSION_DENIED	0x80000002
 #define MUX_S_FAILURE		0x80000003
@@ -125,12 +126,23 @@ struct mux_master_state {
 #define MUX_S_REMOTE_PORT	0x80000007
 #define MUX_S_TTY_ALLOC_FAIL	0x80000008
 #define MUX_S_PROXY		0x8000000f
+#define MUX_S_EXT_INFO		0x90000001
 
 /* type codes for MUX_C_OPEN_FWD and MUX_C_CLOSE_FWD */
 #define MUX_FWD_LOCAL   1
 #define MUX_FWD_REMOTE  2
 #define MUX_FWD_DYNAMIC 3
 
+#define MUX_EXT_INFO		0x00000001
+
+/* Bitmask of supported extensions */
+static u_int extensions = 0;
+
+/* Supported extensions names */
+static const char MUX_EXT_INFO_NAME[] = "info";
+static const char MUX_EXT_INFO_CONNECTION[] = "connection";
+static const char MUX_EXT_INFO_CHANNELS[] = "channels";
+
 static void mux_session_confirm(struct ssh *, int, int, void *);
 static void mux_stdio_confirm(struct ssh *, int, int, void *);
 
@@ -152,6 +164,8 @@ static int mux_master_process_stop_listening(struct ssh *, u_int,
 	    Channel *, struct sshbuf *, struct sshbuf *);
 static int mux_master_process_proxy(struct ssh *, u_int,
 	    Channel *, struct sshbuf *, struct sshbuf *);
+static int mux_master_process_ext_info(struct ssh *, u_int,
+	    Channel *, struct sshbuf *, struct sshbuf *);
 
 static const struct {
 	u_int type;
@@ -167,6 +181,7 @@ static const struct {
 	{ MUX_C_NEW_STDIO_FWD, mux_master_process_stdio_fwd },
 	{ MUX_C_STOP_LISTENING, mux_master_process_stop_listening },
 	{ MUX_C_PROXY, mux_master_process_proxy },
+	{ MUX_C_EXT_INFO, mux_master_process_ext_info },
 	{ 0, NULL }
 };
 
@@ -290,8 +305,13 @@ mux_master_process_hello(struct ssh *ssh, u_int rid,
 			free(name);
 			return -1;
 		}
-		debug2_f("unrecognised extension \"%s\" length %zu",
-		    name, value_len);
+		if (strcmp(name, MUX_EXT_INFO_NAME) == 0) {
+			debug_f("received 'info' extension");
+			extensions |= MUX_EXT_INFO;
+		} else {
+			debug2_f("unrecognised extension \"%s\" length %zu",
+			    name, value_len);
+		}
 		free(name);
 	}
 	state->hello_rcvd = 1;
@@ -496,6 +516,54 @@ mux_master_process_alive_check(struct ssh *ssh, u_int rid,
 	return 0;
 }
 
+/* The "info" extension. */
+static int
+mux_master_process_ext_info(struct ssh *ssh, u_int rid,
+    Channel *c, struct sshbuf *m, struct sshbuf *reply)
+{
+	char *name = NULL;
+	struct sshbuf *buf = NULL;
+	u_int32_t status = 0;
+	int r;
+
+	debug2_f("channel %d: info request", c->self);
+
+	r = sshbuf_get_cstring(m, &name, NULL);
+	if (r != 0)
+		fatal_fr(r, "parse");
+
+	buf = sshbuf_new();
+	if (buf == NULL)
+		fatal_fr(r, "sshbuf_new");
+
+	if (strcmp(name, MUX_EXT_INFO_CONNECTION) == 0) {
+		r = sshbuf_put_conninfo(buf, ssh);
+		if (r != 0)
+			fatal_fr(r, "sshbuf_put_conninfo");
+		status = 1;
+	} else if (strcmp(name, MUX_EXT_INFO_CHANNELS) == 0) {
+		r = sshbuf_put_channels(buf, ssh);
+		if (r != 0)
+			fatal_fr(r, "sshbuf_put_channels");
+		status = 1;
+	} else {
+		r = sshbuf_putf(buf, "info request type not supported");
+		if (r != 0)
+			fatal_fr(r, "sshbuf_putf");
+		status = 0;
+	}
+
+	/* prepare reply */
+	if ((r = sshbuf_put_u32(reply, MUX_S_EXT_INFO)) != 0 ||
+	    (r = sshbuf_put_u32(reply, rid)) != 0 ||
+	    (r = sshbuf_put_u32(reply, status)) != 0 ||
+	    (r = sshbuf_put_cstring(reply, sshbuf_ptr(buf))) != 0)
+		fatal_fr(r, "reply");
+
+	free(name);
+	return 0;
+}
+
 static int
 mux_master_process_terminate(struct ssh *ssh, u_int rid,
     Channel *c, struct sshbuf *m, struct sshbuf *reply)
@@ -1166,7 +1234,10 @@ mux_master_read_cb(struct ssh *ssh, Channel *c)
 		if ((r = sshbuf_put_u32(out, MUX_MSG_HELLO)) != 0 ||
 		    (r = sshbuf_put_u32(out, SSHMUX_VER)) != 0)
 			fatal_fr(r, "reply");
-		/* no extensions */
+		/* "info" extension */
+		if ((r = sshbuf_put_cstring(out, MUX_EXT_INFO_NAME)) != 0 ||
+		    (r = sshbuf_put_cstring(out, "0")) != 0)
+			fatal_fr(r, "put info extension");
 		if ((r = sshbuf_put_stringb(c->output, out)) != 0)
 			fatal_fr(r, "enqueue");
 		debug3_f("channel %d: hello sent", c->self);
@@ -1638,7 +1709,13 @@ mux_client_hello_exchange(int fd, int timeout_ms)
 			error_fr(r, "parse extension");
 			goto out;
 		}
-		debug2_f("unrecognised master extension \"%s\"", name);
+		/* Process extensions. */
+		if (strcmp(name, MUX_EXT_INFO_NAME) == 0) {
+			debug_f("received 'info' extension");
+			extensions |= MUX_EXT_INFO;
+		} else {
+			debug2_f("unrecognised master extension \"%s\"", name);
+		}
 		free(name);
 	}
 	/* success */
@@ -1699,6 +1776,74 @@ mux_client_request_alive(int fd)
 	return pid;
 }
 
+static char *
+mux_client_request_info(int fd, const char *name)
+{
+	struct sshbuf *m;
+	char *e, *msg;
+	u_int type, rid, status;
+	int r;
+
+	debug3_f("entering");
+
+	if ((m = sshbuf_new()) == NULL)
+		fatal_f("sshbuf_new");
+	if ((r = sshbuf_put_u32(m, MUX_C_EXT_INFO)) != 0 ||
+	    (r = sshbuf_put_u32(m, muxclient_request_id)) != 0 ||
+	    (r = sshbuf_put_cstring(m, name)) != 0)
+		fatal_fr(r, "assemble");
+
+	if (mux_client_write_packet(fd, m) != 0)
+		fatal_f("write packet: %s", strerror(errno));
+
+	sshbuf_reset(m);
+
+	/* Read their reply */
+	if (mux_client_read_packet(fd, m) != 0) {
+		sshbuf_free(m);
+		return 0;
+	}
+
+	if ((r = sshbuf_get_u32(m, &type)) != 0)
+		fatal_fr(r, "parse type");
+	if (type != MUX_S_EXT_INFO) {
+		if ((r = sshbuf_get_cstring(m, &e, NULL)) != 0)
+			fatal_fr(r, "parse error message");
+		fatal_f("master returned error: %s", e);
+	}
+
+	if ((r = sshbuf_get_u32(m, &rid)) != 0)
+		fatal_fr(r, "parse remote ID");
+	if (rid != muxclient_request_id)
+		fatal_f("out of sequence reply: my id %u theirs %u",
+		    muxclient_request_id, rid);
+	if ((r = sshbuf_get_u32(m, &status)) != 0 ||
+	    (r = sshbuf_get_cstring(m, &msg, NULL)) != 0)
+		fatal_fr(r, "parse connection info");
+	sshbuf_free(m);
+
+	muxclient_request_id++;
+
+	return msg;
+}
+
+static void
+mux_client_print_ext_info(int fd, const char *name)
+{
+	char *msg = NULL;
+
+	if (!(extensions & MUX_EXT_INFO))
+		fatal("mux server does not support '%s' extension",
+		    MUX_EXT_INFO_NAME);
+
+	msg = mux_client_request_info(fd, name);
+	if (msg == NULL)
+		fatal_f("'%s' info request failed", name);
+
+	printf("%s", msg);
+	free(msg);
+}
+
 static void
 mux_client_request_terminate(int fd)
 {
@@ -2009,7 +2154,7 @@ mux_client_request_session(int fd)
 		enter_raw_mode(options.request_tty == REQUEST_TTY_FORCE);
 
 	/*
-	 * Stick around until the controler closes the client_fd.
+	 * Stick around until the controller closes the client_fd.
 	 * Before it does, it is expected to write an exit message.
 	 * This process must read the value and wait for the closure of
 	 * the client_fd; if this one closes early, the multiplex master will
@@ -2339,6 +2484,12 @@ muxclient(const char *path)
 			fatal_f("master alive check failed");
 		fprintf(stderr, "Master running (pid=%u)\r\n", pid);
 		exit(0);
+	case SSHMUX_COMMAND_CONNINFO:
+		mux_client_print_ext_info(sock, MUX_EXT_INFO_CONNECTION);
+		exit(0);
+	case SSHMUX_COMMAND_CHANINFO:
+		mux_client_print_ext_info(sock, MUX_EXT_INFO_CHANNELS);
+		exit(0);
 	case SSHMUX_COMMAND_TERMINATE:
 		mux_client_request_terminate(sock);
 		if (options.log_level != SYSLOG_LEVEL_QUIET)
