@@ -62,7 +62,6 @@
 #include "glob.h"
 
 #include <sys/types.h>
-#include <sys/stat.h>
 
 #include <dirent.h>
 #include <ctype.h>
@@ -70,9 +69,6 @@
 #include <limits.h>
 #include <pwd.h>
 #include <stdlib.h>
-#ifdef HAVE_STDINT_H
-#include <stdint.h>
-#endif
 #include <string.h>
 #include <unistd.h>
 
