@@ -1,4 +1,4 @@
-/* $OpenBSD: packet.c,v 1.323 2025/09/25 06:33:19 djm Exp $ */
+/* $OpenBSD: packet.c,v 1.327 2025/12/05 06:16:27 dtucker Exp $ */
 /*
  * Author: Tatu Ylonen <ylo@cs.hut.fi>
  * Copyright (c) 1995 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
@@ -2862,3 +2862,200 @@ sshpkt_add_padding(struct ssh *ssh, u_char pad)
 	ssh->state->extra_pad = pad;
 	return 0;
 }
+
+
+#define PUT_CONNINFO(block) { r = block; if (r != 0) return r; }
+
+static int
+sshbuf_put_conninfo_head(struct sshbuf *buf) {
+	char thishost[NI_MAXHOST] = "unknown";
+	(void)gethostname(thishost, sizeof(thishost));
+	return sshbuf_putf(buf,
+	    "Connection information for %s pid %lld\r\n",
+	    thishost, (long long)getpid());
+}
+
+static int
+sshbuf_put_conninfo_tcp(struct sshbuf *buf, struct ssh *ssh) {
+	if (ssh_local_port(ssh) == 65535) return 0;
+	if (strcmp(ssh_local_ipaddr(ssh), "UNKNOWN") == 0) return 0;
+	return sshbuf_putf(buf,
+	    "  tcp\t\t: %s:%d -> %s:%d\r\n",
+	    ssh_local_ipaddr(ssh), ssh_local_port(ssh),
+	    ssh_remote_ipaddr(ssh), ssh_remote_port(ssh));
+}
+
+static int
+sshbuf_put_conninfo_kex(struct sshbuf *buf, struct kex *kex) {
+	if (kex->impl == NULL) return 0;
+	return sshbuf_putf(buf,
+	    "  kex alg.\t: %s\r\n  host-key alg.\t: %s\r\n",
+	    kex->impl->name, kex->hostkey_alg);
+}
+
+static int
+sshbuf_put_conninfo_alg(struct sshbuf *buf, const char *type,
+    const char *in, const char *out) {
+	const char *fmt;
+	if (in == NULL) in = "<implicit>";
+	if (out == NULL) out = "<implicit>";
+	fmt = (strcmp(in, out) == 0)
+	    ? "  %s\t: %s\r\n"
+	    : "  %s\t: %s in, %s out\r\n";
+	return sshbuf_putf(buf, fmt, type, in, out);
+}
+
+static int
+sshbuf_put_conninfo_rekey(struct sshbuf *buf, struct session_state *state) {
+	int r;
+
+	PUT_CONNINFO(sshbuf_putf(buf, "  rekey\t\t: "))
+	PUT_CONNINFO(sshbuf_putf(buf, "limit "))
+
+	/* volume based rekeying */
+	if (state->rekey_limit == 0)
+		r = sshbuf_putf(buf, "none");
+	else {
+		if (state->max_blocks_in == state->max_blocks_out)
+			r = sshbuf_putf(buf,
+			    "blocks %llu",
+			    (unsigned long long)state->max_blocks_in);
+		else
+			r = sshbuf_putf(buf,
+			    "blocks %llu in %llu out",
+			    (unsigned long long)state->max_blocks_in,
+			    (unsigned long long)state->max_blocks_out);
+	}
+	if (r != 0) return r;
+
+	PUT_CONNINFO(sshbuf_putf(buf, ", interval "))
+
+	/* time based rekeying. */
+	if (state->rekey_interval == 0)
+		r = sshbuf_putf(buf, "none");
+	else {
+		char *rekey_interval;
+		char rekey_next[64];
+
+		rekey_interval = fmttime(state->rekey_interval);
+
+	{	uint64_t rekey_time;
+		rekey_time = (u_int64_t)time(NULL) - monotime() +
+		    state->rekey_time + state->rekey_interval;
+		format_absolute_time(rekey_time, rekey_next, sizeof(rekey_next));
+	}
+
+		r = sshbuf_putf(buf, "%s, next %s",
+		    rekey_interval, rekey_next);
+
+		free(rekey_interval);
+	}
+	if (r != 0) return r;
+
+	return sshbuf_putf(buf, "\r\n");
+}
+
+static int
+sshbuf_put_conninfo_trafic_state(struct sshbuf *buf, struct packet_state *ps) {
+	char bytes[FMT_SCALED_STRSIZE];
+
+	if (ps->bytes > LLONG_MAX || fmt_scaled(ps->bytes, bytes) != 0)
+		strlcpy(bytes, "OVERFLOW", sizeof(bytes));
+
+	return sshbuf_putf(buf,
+	    "%lu packets %llu blocks %sB",
+	    (unsigned long)ps->packets,
+	    (unsigned long long)ps->blocks,
+	    bytes);
+}
+
+static int
+sshbuf_put_conninfo_trafic(struct sshbuf *buf, struct session_state *state) {
+	int r;
+
+	PUT_CONNINFO(sshbuf_putf(buf, "  traffic\t:\r\n    "))
+	PUT_CONNINFO(sshbuf_put_conninfo_trafic_state(buf, &state->p_read))
+	PUT_CONNINFO(sshbuf_putf(buf, " in, "))
+	PUT_CONNINFO(sshbuf_put_conninfo_trafic_state(buf, &state->p_send))
+	return sshbuf_putf(buf, " out\r\n");
+}
+
+#ifdef WITH_ZLIB
+static int
+sshbuf_put_z_stream_status(struct sshbuf *buf, int started, z_stream *stream) {
+	unsigned long long raw = 0, cmp = 0;
+	char raw_f[FMT_SCALED_STRSIZE] = "";
+	char cmp_f[FMT_SCALED_STRSIZE] = "";
+
+	if (started) {
+		raw = stream->total_out;
+		cmp = stream->total_in;
+	}
+
+	if (fmt_scaled(raw, raw_f) != 0)
+	    strlcpy(raw_f, "OVERFLOW", sizeof(raw_f));
+
+	if (fmt_scaled(cmp, cmp_f) != 0)
+	    strlcpy(cmp_f, "OVERFLOW", sizeof(cmp_f));
+
+	return sshbuf_putf(buf,
+	    "%s/%s (*%.3f)",
+	    cmp_f, raw_f, raw == 0 ? 0.0 : (double)cmp / raw);
+}
+#endif	/* WITH_ZLIB */
+
+static int
+sshbuf_put_comp_status(struct sshbuf *buf, struct session_state *state) {
+#ifdef WITH_ZLIB
+	int r;
+
+	if (!state->compression_buffer) return 0;
+
+	PUT_CONNINFO(sshbuf_putf(buf, "    compressed "))
+	PUT_CONNINFO(sshbuf_put_z_stream_status(buf,
+	    state->compression_in_started,
+	    &state->compression_in_stream))
+	PUT_CONNINFO(sshbuf_putf(buf, " in, "))
+	PUT_CONNINFO(sshbuf_put_z_stream_status(buf,
+	    state->compression_out_started,
+	    &state->compression_out_stream))
+	return sshbuf_putf(buf, " out\r\n");
+#else
+	return 0;
+#endif	/* WITH_ZLIB */
+}
+
+int
+sshbuf_put_conninfo(struct sshbuf *buf, struct ssh *ssh) {
+	struct session_state *state;
+	struct newkeys *in;
+	struct newkeys *out;
+	int r;
+
+	state = ssh->state;
+	in = state->newkeys[MODE_IN];
+	out = state->newkeys[MODE_OUT];
+
+	PUT_CONNINFO(sshbuf_put_conninfo_head(buf))
+	PUT_CONNINFO(sshbuf_put_conninfo_tcp(buf, ssh))
+	PUT_CONNINFO(sshbuf_put_conninfo_kex(buf, ssh->kex))
+	PUT_CONNINFO(sshbuf_put_conninfo_alg(buf, "cipher",
+	    in->enc.name, out->enc.name))
+	PUT_CONNINFO(sshbuf_put_conninfo_alg(buf, "mac\t",
+	    in->mac.name, out->mac.name))
+	PUT_CONNINFO(sshbuf_put_conninfo_rekey(buf, state))
+	PUT_CONNINFO(sshbuf_put_conninfo_trafic(buf, state))
+	PUT_CONNINFO(sshbuf_put_conninfo_alg(buf, "compression",
+	    in->comp.name, out->comp.name))
+	return sshbuf_put_comp_status(buf, state);
+}
+
+int
+sshbuf_put_channels(struct sshbuf *buf, struct ssh *ssh) {
+	const char *s;
+
+	s = channel_open_message(ssh);
+	if (s  == NULL)
+		fatal_f("channel_open_message");
+	return sshbuf_put(buf, s, strlen(s));
+}
