@@ -47,32 +47,33 @@ printf '' > $OBJ/authorized_keys_$USER
 
 for t in $SSH_KEYTYPES ; do
 	# generate user key for agent
-	rm -f $OBJ/$t-agent $OBJ/$t-agent.pub*
-	${SSHKEYGEN} -q -N '' -t $t -f $OBJ/$t-agent ||\
-		 fatal "ssh-keygen for $t-agent failed"
+	AK=$OBJ/agent.$t
+	rm -f $AK*
+	${SSHKEYGEN} -q -N '' -t $t -f $AK ||\
+		 fatal "ssh-keygen for $t failed"
 	# Make a certificate for each too.
 	${SSHKEYGEN} -qs $OBJ/user_ca_key -I "$t cert" \
-		-n estragon $OBJ/$t-agent.pub || fatal "ca sign failed"
+		-n estragon $AK.pub || fatal "ca sign failed"
 
 	# add to authorized keys
-	cat $OBJ/$t-agent.pub >> $OBJ/authorized_keys_$USER
+	cat $AK.pub >> $OBJ/authorized_keys_$USER
 	# add private key to agent
-	${SSHADD} $OBJ/$t-agent > /dev/null 2>&1
+	${SSHADD} $AK > /dev/null 2>&1
 	r=$?
 	if [ $r -ne 0 ]; then
 		fail "ssh-add failed exit code $r"
 	fi
 	# add private key to second agent
-	SSH_AUTH_SOCK=$FW_SSH_AUTH_SOCK ${SSHADD} $OBJ/$t-agent > /dev/null 2>&1
+	SSH_AUTH_SOCK=$FW_SSH_AUTH_SOCK ${SSHADD} $AK > /dev/null 2>&1
 	r=$?
 	if [ $r -ne 0 ]; then
 		fail "ssh-add failed exit code $r"
 	fi
 	# Move private key to ensure that we aren't accidentally using it.
 	# Keep the corresponding public keys/certs around for later use.
-	mv -f $OBJ/$t-agent $OBJ/$t-agent-private
-	cp -f $OBJ/$t-agent.pub $OBJ/$t-agent-private.pub
-	cp -f $OBJ/$t-agent-cert.pub $OBJ/$t-agent-private-cert.pub
+	mv -f $AK $AK-private
+	cp -f $AK.pub $AK-private.pub
+	cp -f $AK-cert.pub $AK-private-cert.pub
 done
 
 # Remove explicit identity directives from ssh_proxy
@@ -100,7 +101,8 @@ fi
 
 for t in $SSH_KEYTYPES ; do
 	trace "connect via agent using $t key"
-	${SSH} -F $OBJ/ssh_proxy -i $OBJ/$t-agent.pub -oIdentitiesOnly=yes \
+	AK=$OBJ/agent.$t
+	${SSH} -F $OBJ/ssh_proxy -i $AK.pub -oIdentitiesOnly=yes \
 		somehost exit 52
 	r=$?
 	if [ $r -ne 52 ]; then
@@ -154,8 +156,9 @@ fi
 	> $OBJ/authorized_keys_$USER
 for t in $SSH_KEYTYPES ; do
 	trace "connect via agent using $t key"
-	${SSH} -F $OBJ/ssh_proxy -i $OBJ/$t-agent.pub \
-		-oCertificateFile=$OBJ/$t-agent-cert.pub \
+	AK=$OBJ/agent.$t
+	${SSH} -F $OBJ/ssh_proxy -i $AK.pub \
+		-oCertificateFile=$AK-cert.pub \
 		-oIdentitiesOnly=yes somehost exit 52
 	r=$?
 	if [ $r -ne 52 ]; then
@@ -180,7 +183,8 @@ fi
 trace "readd keys"
 # re-add keys/certs to agent
 for t in $SSH_KEYTYPES ; do
-	${SSHADD} $OBJ/$t-agent-private >/dev/null 2>&1 || \
+	AK=$OBJ/agent.$t
+	${SSHADD} $AK-private >/dev/null 2>&1 || \
 		fail "ssh-add failed exit code $?"
 done
 # make sure they are there
@@ -204,7 +208,8 @@ if [ $r -ne 1 ]; then
 fi
 # re-add keys/certs to agent
 for t in $SSH_KEYTYPES ; do
-	${SSHADD} $OBJ/$t-agent-private >/dev/null 2>&1 || \
+	AK=$OBJ/agent.$t
+	${SSHADD} $AK-private >/dev/null 2>&1 || \
 		fail "ssh-add failed exit code $?"
 done
 # make sure they are there
@@ -228,29 +233,30 @@ check_key_present() {
 }
 
 # delete the ed25519 key
+AK=$OBJ/agent.ssh-ed25519
 trace "delete single key by file"
-${SSHADD} -qdk $OBJ/ssh-ed25519-agent || fail "ssh-add -d ed25519 failed"
+${SSHADD} -qdk $AK || fail "ssh-add -d ed25519 failed"
 check_key_absent ssh-ed25519
 check_key_present ssh-ed25519-cert-v01@openssh.com
 # Put key/cert back.
-${SSHADD} $OBJ/ssh-ed25519-agent-private >/dev/null 2>&1 || \
+${SSHADD} $AK-private >/dev/null 2>&1 || \
 	fail "ssh-add failed exit code $?"
 check_key_present ssh-ed25519
 # Delete both key and certificate.
 trace "delete key/cert by file"
-${SSHADD} -qd $OBJ/ssh-ed25519-agent || fail "ssh-add -d ed25519 failed"
+${SSHADD} -qd $AK || fail "ssh-add -d ed25519 failed"
 check_key_absent ssh-ed25519
 check_key_absent ssh-ed25519-cert-v01@openssh.com
 # Put key/cert back.
-${SSHADD} $OBJ/ssh-ed25519-agent-private >/dev/null 2>&1 || \
+${SSHADD} $AK-private >/dev/null 2>&1 || \
 	fail "ssh-add failed exit code $?"
 check_key_present ssh-ed25519
 # Delete certificate via stdin
-${SSHADD} -qd - < $OBJ/ssh-ed25519-agent-cert.pub || fail "ssh-add -d - failed"
+${SSHADD} -qd - < $AK-cert.pub || fail "ssh-add -d - failed"
 check_key_present ssh-ed25519
 check_key_absent ssh-ed25519-cert-v01@openssh.com
 # Delete key via stdin
-${SSHADD} -qd - < $OBJ/ssh-ed25519-agent.pub || fail "ssh-add -d - failed"
+${SSHADD} -qd - < $AK.pub || fail "ssh-add -d - failed"
 check_key_absent ssh-ed25519
 check_key_absent ssh-ed25519-cert-v01@openssh.com
 
