@@ -15,6 +15,9 @@ rm -f $OBJ/.dropbear/dbclient.log
 for c in $ciphers ; do
   for m in $macs; do
     for i in $OBJ/.dropbear/id_* ; do
+      case $i in
+      *.pub) continue;;
+      esac
       kt=`echo $i | sed -e 's/.*id_//'`
       verbose "$tid: cipher $c mac $m kt $kt"
       rm -f $COPY
