@@ -7,8 +7,13 @@ $REGRESS_INTEROP_DROPBEAR || { RESULT=1; skip "dropbear interop tests are not en
 
 cp $OBJ/sshd_proxy $OBJ/sshd_proxy_bak
 
-kex="curve25519-sha256 curve25519-sha256@libssh.org
-    diffie-hellman-group14-sha256 diffie-hellman-group14-sha1"
+# Dropbear v2025.88
+kex=
+kex="$kex sntrup761x25519-sha512" # requires specific PKIX-SSH build
+kex="$kex mlkem768x25519-sha256"
+kex="$kex curve25519-sha256"
+kex="$kex ecdh-sha2-nistp521 ecdh-sha2-nistp384 ecdh-sha2-nistp256"
+kex="$kex diffie-hellman-group14-sha256"
 
 rm -f $OBJ/.dropbear/dbclient.log
 for k in $kex; do
