@@ -39,6 +39,10 @@ for fmt in RFC4716 PKCS8 PEM OpenSSH ; do
 		*openssh.com)
 			# *@openssh.com keys are stored only in custom format
 			$customfmt || continue ;;
+		ssh-rsa|ssh-dss|ecdsa-sha2-nistp*)
+			# cryptographic library supports traditional PEM for
+			# keytypes implemented in OpenSSL < 1.1.1.
+			;;
 		ssh-ed25519)
 			# traditional PEM is not supported by cryptographic
 			# library for ed25519 keys
@@ -48,6 +52,9 @@ for fmt in RFC4716 PKCS8 PEM OpenSSH ; do
 				$customfmt || continue
 			fi
 			;;
+		*)	# no PEM format for new keytypes
+			test "$fmt" = "PEM" && continue
+			;;
 		esac
 		comment="foo bar"
 		$SSHKEYGEN -m $fmt -N '' -C "$comment" \
