@@ -6,21 +6,20 @@ tid="known hosts command "
 # cross-project configuration
 if test "$sshd_type" != "pkix" ; then
 mv $OBJ/sshd_proxy $OBJ/sshd_proxy.orig
-(
-	grep -v "HostkeyAlgorithms"  $OBJ/sshd_proxy.orig
-	echo "HostkeyAlgorithms *,ssh-dss*"
+( grep -vi "HostkeyAlgorithms"  $OBJ/sshd_proxy.orig
+  echo "HostkeyAlgorithms *,ssh-dss*"
 ) > $OBJ/sshd_proxy
 fi
 
 
 rm -f $OBJ/knownhosts_command $OBJ/ssh_proxy_khc
-cp $OBJ/ssh_proxy $OBJ/ssh_proxy_orig
 
-( grep -vi GlobalKnownHostsFile $OBJ/ssh_proxy_orig | \
-    grep -vi UserKnownHostsFile;
-  echo "GlobalKnownHostsFile none" ;
-  echo "UserKnownHostsFile none" ;
-  echo "KnownHostsCommand $OBJ/knownhosts_command '%t' '%K' '%u'" ;
+mv $OBJ/ssh_proxy $OBJ/ssh_proxy.orig
+( grep -vi GlobalKnownHostsFile $OBJ/ssh_proxy.orig | \
+  grep -vi UserKnownHostsFile;
+  echo "GlobalKnownHostsFile none"
+  echo "UserKnownHostsFile none"
+  echo "KnownHostsCommand $OBJ/knownhosts_command '%t' '%K' '%u'"
 ) > $OBJ/ssh_proxy
 
 > $OBJ/knownhosts_command
