@@ -1,4 +1,4 @@
-#	$OpenBSD: test-exec.sh,v 1.127 2025/03/28 05:41:15 dtucker Exp $
+#	$OpenBSD: test-exec.sh,v 1.136 2025/12/05 06:55:22 dtucker Exp $
 #	Placed in the Public Domain.
 
 #SUDO=sudo
@@ -86,20 +86,6 @@ CYGWIN*)
 esac
 
 
-# wrappers to programs found by configure script
-egrep() {
-  $EGREP ${1+"$@"}
-}
-
-fgrep() {
-  $FGREP ${1+"$@"}
-}
-
-awk() {
-  $AWK ${1+"$@"}
-}
-
-
 SRC=`dirname ${SCRIPT}`
 
 # defaults
@@ -580,6 +566,7 @@ rm -f $OBJ/known_hosts $OBJ/authorized_keys_$USER
 
 SSH_KEYTYPES=`$SSH -Q key-plain | grep -v "^x509v3-"`
 SSH_HOSTKEY_TYPES=`$SSH -Q key-plain | grep -v "^x509v3-"`
+SSH_KEX_ALGS=`$SSH -Q kex`
 
 case $SCRIPT in
 */fips-*)
@@ -593,39 +580,42 @@ esac
 
 for t in ${SSH_KEYTYPES}; do
 	# generate user key
+	ID=$OBJ/id.$t
 	if test -n "$OPENSSL_FIPS" ; then
-		rm -f $OBJ/$t
+		rm -f $ID
 	fi
-	if [ ! -f $OBJ/$t ] || [ ${SSHKEYGEN_BIN} -nt $OBJ/$t ]; then
+	if test ! -f $ID -o $SSHKEYGEN_BIN -nt $ID ; then
 		trace "generating key type $t"
-		rm -f $OBJ/$t
+		rm -f $ID
 		keytype_compat $t
-		$SSHKEYGEN -q -N '' -t $t $keytype_val -f $OBJ/$t ||\
+		$SSHKEYGEN -q -N '' -t $t $keytype_val -f $ID ||\
 			fail "ssh-keygen for $t failed"
 	else
 		trace "using cached key type $t"
 	fi
 
 	# setup authorized keys
-	cat $OBJ/$t.pub >> $OBJ/authorized_keys_$USER
-	echo IdentityFile $OBJ/$t >> $OBJ/ssh_config
+	cat $ID.pub >> $OBJ/authorized_keys_$USER
+	echo IdentityFile $ID >> $OBJ/ssh_config
 done
 
 for t in ${SSH_HOSTKEY_TYPES}; do
 	# known hosts file for client
+	ID=$OBJ/id.$t
 	(
 		printf 'localhost-with-alias,127.0.0.1,::1 '
-		cat $OBJ/$t.pub
+		cat $ID.pub
 	) >> $OBJ/known_hosts
 
-	if test ! -f $OBJ/host.$t || test $OBJ/$t -nt $OBJ/host.$t ; then
+	HK=$OBJ/host.$t
+	if test ! -f $HK -o $ID -nt $HK ; then
 		# use key as host key, too
-		cp $OBJ/$t $OBJ/host.$t
+		cp $ID $HK
 	fi
-	echo HostKey $OBJ/host.$t >> $OBJ/sshd_config
+	echo HostKey $HK >> $OBJ/sshd_config
 
 	# don't use SUDO for proxy connect
-	echo HostKey $OBJ/$t >> $OBJ/sshd_proxy
+	echo HostKey $HK >> $OBJ/sshd_proxy
 done
 chmod 644 $OBJ/authorized_keys_$USER
 
@@ -662,17 +652,17 @@ if $REGRESS_INTEROP_CONCH ; then
 	SSH_ID_CONCH_RSA=false
 	if $SSH_ID_CONCH_RSA ; then
 	# ensure that ssh key is PEM format
-	cp $OBJ/ssh-rsa $OBJ/ssh-rsa_pem
-	$SSHKEYGEN -p -N '' -m PEM -f $OBJ/ssh-rsa_pem >/dev/null
-	$SSHKEYGEN -y -f $OBJ/ssh-rsa_pem > $OBJ/ssh-rsa_pem.pub
+	SSH_ID_CONCH=$OBJ/conch.rsa_pem
+	cp $OBJ/id.ssh-rsa $SSH_ID_CONCH
+	$SSHKEYGEN -p -N '' -m PEM -f $SSH_ID_CONCH >/dev/null
 	else
 	# ensure that ssh ed25519 key is in custom format
 	# NOTE conch does not work with ed25519 in PKCS#8 format!
-	SSH_ID_CONCH=$OBJ/ssh-ed25519-conch
-	cp $OBJ/ssh-ed25519 $SSH_ID_CONCH
+	SSH_ID_CONCH=$OBJ/conch.ssh-ed25519
+	cp $OBJ/id.ssh-ed25519 $SSH_ID_CONCH
 	$SSHKEYGEN -p -N '' -m OpenSSH -f $SSH_ID_CONCH >/dev/null
-	$SSHKEYGEN -y -f $SSH_ID_CONCH > $SSH_ID_CONCH.pub
 	fi
+	$SSHKEYGEN -y -f $SSH_ID_CONCH > $SSH_ID_CONCH.pub
 fi
 
 puttysetup() {
@@ -695,16 +685,16 @@ puttysetup() {
 	fi
 
 	# ensure that ssh key is PEM format
-	cp $OBJ/ssh-rsa $OBJ/ssh-rsa_pem
-	$SSHKEYGEN -p -N '' -m PEM -f $OBJ/ssh-rsa_pem >/dev/null
+	cp $OBJ/id.ssh-rsa $OBJ/putty.rsa_pem
+	$SSHKEYGEN -p -N '' -m PEM -f $OBJ/putty.rsa_pem >/dev/null
 
 	# Add a PuTTY key to authorized_keys
 	$PUTTYGEN -O public-openssh ${OBJ}/putty.rsa2 \
 	    >> $OBJ/authorized_keys_$USER
 
-	${SRC}/ssh2putty.sh 127.0.0.1 $PORT $OBJ/ssh-rsa_pem > \
+	${SRC}/ssh2putty.sh 127.0.0.1 $PORT $OBJ/putty.rsa_pem > \
 	    $PUTTYDIR/sshhostkeys
-	${SRC}/ssh2putty.sh 127.0.0.1 22 $OBJ/ssh-rsa_pem >> \
+	${SRC}/ssh2putty.sh 127.0.0.1 22 $OBJ/putty.rsa_pem >> \
 	    $PUTTYDIR/sshhostkeys
 
 	# Setup proxied session
@@ -727,7 +717,7 @@ if $REGRESS_INTEROP_DROPBEAR ; then
 		ssh-rsa) i=rsa;;
 		ecdsa-sha2-nistp256) i=ecdsa;;
 		ssh-ed25519) i=ed25519;;
-		ssh-dss) i=dss;;
+		#ssh-dss) i=dss;; #-Dropbear v2025.88
 		*) continue;;
 		esac
 		# Note dropbear fail to convert from PKIX-SSH key formats.
@@ -739,6 +729,11 @@ if $REGRESS_INTEROP_DROPBEAR ; then
 			    $OBJ/.dropbear/id_$i \
 			    $OBJ/.dropbear/ossh.id_$i
 			) > /dev/null 2>&1
+			ret=$?
+			if test $ret -ne 0 ; then
+				echo "skip unsupported type $i" >&2
+				continue
+			fi
 		fi
 		printf "%s dropbear_$i\n" \
 		    "`$SSHKEYGEN -y -f $OBJ/.dropbear/ossh.id_$i`" \
