@@ -5,8 +5,7 @@
  * Placed in the public domain
  */
 
-#define OPENSSL_SUPPRESS_DEPRECATED	/* TODO implement OpenSSL 3.1 API */
-
+#define SSHKEY_INTERNAL 1	/* access internals for testing */
 #include "../test_helper/test_helper.h"
 
 #include <fcntl.h>
@@ -28,43 +27,6 @@
 #include "common.h"
 
 
-/* TODO: rewrite direct RSA tests */
-#ifndef HAVE_RSA_GET0_KEY
-/* opaque RSA key structure */
-static inline void
-RSA_get0_key(const RSA *rsa, const BIGNUM **n, const BIGNUM **e, const BIGNUM **d) {
-	if (n != NULL) *n = rsa->n;
-	if (e != NULL) *e = rsa->e;
-	if (d != NULL) *d = rsa->d;
-}
-
-static inline void
-RSA_get0_factors(const RSA *rsa, const BIGNUM **p, const BIGNUM **q) {
-	if (p != NULL) *p = rsa->p;
-	if (q != NULL) *q = rsa->q;
-}
-#endif /*ndef HAVE_RSA_GET0_KEY*/
-
-#ifdef WITH_DSA
-/* TODO: rewrite direct DSA tests */
-#ifndef HAVE_DSA_GET0_KEY
-/* opaque DSA key structure */
-static inline void
-DSA_get0_key(const DSA *dsa, const BIGNUM **pub_key, const BIGNUM **priv_key) {
-	if (pub_key  != NULL) *pub_key  = dsa->pub_key;
-	if (priv_key != NULL) *priv_key = dsa->priv_key;
-}
-
-static inline void
-DSA_get0_pqg(const DSA *dsa, const BIGNUM **p, const BIGNUM **q, const BIGNUM **g) {
-	if (p != NULL) *p = dsa->p;
-	if (q != NULL) *q = dsa->q;
-	if (g != NULL) *g = dsa->g;
-}
-#endif /*ndef HAVE_DSA_GET0_KEY*/
-#endif /*def WITH_DSA*/
-
-
 void sshkey_file_tests(void);
 
 void
@@ -90,19 +52,18 @@ sshkey_file_tests(void)
 	a = load_bignum("rsa_1.param.n");
 	b = load_bignum("rsa_1.param.p");
 	c = load_bignum("rsa_1.param.q");
-{
-	const BIGNUM *n = NULL, *p = NULL, *q = NULL;
-	RSA *rsa = EVP_PKEY_get1_RSA(k1->pk);
+{	BIGNUM *n = NULL, *p = NULL, *q = NULL;
 
-	ASSERT_PTR_NE(rsa, NULL);
-	RSA_get0_key(rsa, &n, NULL, NULL);
-	RSA_get0_factors(rsa, &p, &q);
+	ssh_EVP_PKEY_RSA_get1_key(k1->pk, &n, NULL, NULL);
+	ssh_EVP_PKEY_RSA_get1_factors(k1->pk, &p, &q);
 
 	ASSERT_BIGNUM_EQ(n, a);
 	ASSERT_BIGNUM_EQ(p, b);
 	ASSERT_BIGNUM_EQ(q, c);
 
-	RSA_free(rsa);
+	BN_free(n); /* not secret */
+	BN_clear_free(p);
+	BN_clear_free(q);
 }
 	BN_free(a);
 	BN_free(b);
@@ -217,19 +178,18 @@ sshkey_file_tests(void)
 	a = load_bignum("dsa_1.param.g");
 	b = load_bignum("dsa_1.param.priv");
 	c = load_bignum("dsa_1.param.pub");
-{
-	const BIGNUM *g = NULL, *pub_key = NULL, *priv_key = NULL;
-	DSA *dsa = EVP_PKEY_get1_DSA(k1->pk);
+{	BIGNUM *g = NULL, *pub_key = NULL, *priv_key = NULL;
 
-	ASSERT_PTR_NE(dsa, NULL);
-	DSA_get0_pqg(dsa, NULL, NULL, &g);
-	DSA_get0_key(dsa, &pub_key, &priv_key);
+	ssh_EVP_PKEY_DSA_get1_pqg(k1->pk, NULL, NULL, &g);
+	ssh_EVP_PKEY_DSA_get1_key(k1->pk, &pub_key, &priv_key);
 
 	ASSERT_BIGNUM_EQ(g, a);
 	ASSERT_BIGNUM_EQ(priv_key, b);
 	ASSERT_BIGNUM_EQ(pub_key, c);
 
-	DSA_free(dsa);
+	BN_free(g); /* not secret */
+	BN_free(pub_key); /* not secret */
+	BN_clear_free(priv_key);
 }
 	BN_free(a);
 	BN_free(b);
@@ -328,26 +288,30 @@ sshkey_file_tests(void)
 	sshbuf_free(buf);
 	a = load_bignum("ecdsa_1.param.priv");
 	b = load_bignum("ecdsa_1.param.pub");
-{
-	const EC_GROUP *ec_group;
-	const EC_POINT *ec_pub;
-	const BIGNUM *ec_priv;
-	EC_KEY *ec = EVP_PKEY_get1_EC_KEY(k1->pk);
+{	EC_GROUP *ec_group = NULL;
+	EC_POINT *ec_pub = NULL;
+	BIGNUM *ec_priv = NULL;
 
-	ASSERT_PTR_NE(ec, NULL);
-	ec_group = EC_KEY_get0_group(ec);
-	ec_pub = EC_KEY_get0_public_key(ec);
-	ec_priv = EC_KEY_get0_private_key(ec);
+	ssh_EVP_PKEY_EC_KEY_get1_public_key(k1->pk, &ec_group, &ec_pub);
+	ssh_EVP_PKEY_EC_KEY_get1_private_key(k1->pk, &ec_priv);
 
-	c = EC_POINT_point2bn(ec_group, ec_pub, POINT_CONVERSION_UNCOMPRESSED, NULL, NULL);
-	ASSERT_PTR_NE(c, NULL);
 	ASSERT_BIGNUM_EQ(ec_priv, a);
-	ASSERT_BIGNUM_EQ(b, c);
-	EC_KEY_free(ec);
+
+{	char *hex1, *hex2;
+	hex1 = BN_bn2hex(b);
+	hex2 = EC_POINT_point2hex(ec_group, ec_pub, POINT_CONVERSION_UNCOMPRESSED, NULL);
+	ASSERT_PTR_NE(hex1, NULL);
+	ASSERT_PTR_NE(hex2, NULL);
+	ASSERT_INT_EQ(strcmp(hex1, hex2), 0);
+	OPENSSL_free(hex1);
+	OPENSSL_free(hex2);
+}
+	EC_GROUP_free(ec_group);
+	EC_POINT_free(ec_pub);
+	BN_free(ec_priv);
 }
 	BN_free(a);
 	BN_free(b);
-	BN_free(c);
 	TEST_DONE();
 
 	TEST_START("parse ECDSA from private w/ passphrase");
