@@ -5,9 +5,7 @@
  * Placed in the public domain
  */
 
-#define SSHBUF_INTERNAL 1	/* access internals for testing */
-#define OPENSSL_SUPPRESS_DEPRECATED	/* TODO implement OpenSSL 3.1 API */
-
+#define SSHKEY_INTERNAL 1	/* access internals for testing */
 #include "../test_helper/test_helper.h"
 
 #include <stdio.h>
@@ -26,43 +24,6 @@
 #include "ssh2.h"
 
 
-/* TODO: rewrite or remove direct RSA tests */
-#ifndef HAVE_RSA_GET0_KEY
-/* opaque RSA key structure */
-static inline void
-RSA_get0_key(const RSA *rsa, const BIGNUM **n, const BIGNUM **e, const BIGNUM **d) {
-	if (n != NULL) *n = rsa->n;
-	if (e != NULL) *e = rsa->e;
-	if (d != NULL) *d = rsa->d;
-}
-
-static inline void
-RSA_get0_factors(const RSA *rsa, const BIGNUM **p, const BIGNUM **q) {
-	if (p != NULL) *p = rsa->p;
-	if (q != NULL) *q = rsa->q;
-}
-#endif /*ndef HAVE_RSA_GET0_KEY*/
-
-#ifdef WITH_DSA
-/* TODO: rewrite or remove direct DSA tests */
-#ifndef HAVE_DSA_GET0_KEY
-/* opaque DSA key structure */
-static inline void
-DSA_get0_key(const DSA *dsa, const BIGNUM **pub_key, const BIGNUM **priv_key) {
-	if (pub_key  != NULL) *pub_key  = dsa->pub_key;
-	if (priv_key != NULL) *priv_key = dsa->priv_key;
-}
-
-static inline void
-DSA_get0_pqg(const DSA *dsa, const BIGNUM **p, const BIGNUM **q, const BIGNUM **g) {
-	if (p != NULL) *p = dsa->p;
-	if (q != NULL) *q = dsa->q;
-	if (g != NULL) *g = dsa->g;
-}
-#endif /*ndef HAVE_DSA_GET0_KEY*/
-#endif /*def WITH_DSA*/
-
-
 void sshkey_tests(void);
 void sshkey_benchmarks(void);
 
@@ -287,7 +248,7 @@ sshkey_tests(void)
 {
 	struct sshkey *k1, *k2, *kf = NULL;
 #ifdef WITH_OPENSSL
-	struct sshkey *kr = NULL, *kd = NULL, *ke= NULL;
+	struct sshkey *kr = NULL, *kd = NULL, *ke = NULL;
 #endif /* WITH_OPENSSL */
 	struct sshbuf *b = NULL;
 
@@ -389,20 +350,19 @@ sshkey_tests(void)
 	ASSERT_INT_EQ(sshkey_generate(KEY_RSA, 1024, &kr), 0);
 	ASSERT_PTR_NE(kr, NULL);
 	ASSERT_PTR_NE(kr->pk, NULL);
-{
-	const BIGNUM *n = NULL, *e = NULL, *p = NULL;
-	RSA *rsa = EVP_PKEY_get1_RSA(kr->pk);
+{	BIGNUM *n = NULL, *e = NULL, *p = NULL;
 
-	ASSERT_PTR_NE(rsa, NULL);
-	RSA_get0_key(rsa, &n, &e, NULL);
-	RSA_get0_factors(rsa, &p, NULL);
+	ssh_EVP_PKEY_RSA_get1_key(kr->pk, &n, &e, NULL);
+	ssh_EVP_PKEY_RSA_get1_factors(kr->pk, &p, NULL);
 
 	ASSERT_PTR_NE(n, NULL);
 	ASSERT_PTR_NE(e, NULL);
 	ASSERT_PTR_NE(p, NULL);
 	ASSERT_INT_EQ(BN_num_bits(n), 1024);
 
-	RSA_free(rsa);
+	BN_free(n); /* not secret */
+	BN_free(e); /* not secret */
+	BN_clear_free(p);
 }
 	TEST_DONE();
 
@@ -411,18 +371,16 @@ sshkey_tests(void)
 	ASSERT_INT_EQ(sshkey_generate(KEY_DSA, 1024, &kd), 0);
 	ASSERT_PTR_NE(kd, NULL);
 	ASSERT_PTR_NE(kd->pk, NULL);
-{
-	const BIGNUM *g = NULL, *priv_key = NULL;
-	DSA *dsa = EVP_PKEY_get1_DSA(kd->pk);
+{	BIGNUM *g = NULL, *priv_key = NULL;
 
-	ASSERT_PTR_NE(dsa, NULL);
-	DSA_get0_pqg(dsa, NULL, NULL, &g);
-	DSA_get0_key(dsa, NULL, &priv_key);
+	ssh_EVP_PKEY_DSA_get1_pqg(kd->pk, NULL, NULL, &g);
+	ssh_EVP_PKEY_DSA_get1_key(kd->pk, NULL, &priv_key);
 
 	ASSERT_PTR_NE(g, NULL);
 	ASSERT_PTR_NE(priv_key, NULL);
 
-	DSA_free(dsa);
+	BN_free(g);
+	BN_clear_free(priv_key);
 }
 	TEST_DONE();
 #endif
@@ -432,19 +390,20 @@ sshkey_tests(void)
 	ASSERT_INT_EQ(sshkey_generate(KEY_ECDSA, 256, &ke), 0);
 	ASSERT_PTR_NE(ke, NULL);
 	ASSERT_PTR_NE(ke->pk, NULL);
-{
-	const EC_POINT *ec_pub;
-	const BIGNUM *ec_priv;
-	EC_KEY *ec = EVP_PKEY_get1_EC_KEY(ke->pk);
+{	EC_GROUP *ec_group = NULL;
+	EC_POINT *ec_pub = NULL;
+	BIGNUM *ec_priv = NULL;
 
-	ASSERT_PTR_NE(ec, NULL);
-	ec_pub = EC_KEY_get0_public_key(ec);
-	ec_priv = EC_KEY_get0_private_key(ec);
+	ssh_EVP_PKEY_EC_KEY_get1_public_key(ke->pk, &ec_group, &ec_pub);
+	ssh_EVP_PKEY_EC_KEY_get1_private_key(ke->pk, &ec_priv);
 
+	ASSERT_PTR_NE(ec_group, NULL);
 	ASSERT_PTR_NE(ec_pub, NULL);
 	ASSERT_PTR_NE(ec_priv, NULL);
 
-	EC_KEY_free(ec);
+	EC_GROUP_free(ec_group);
+	EC_POINT_free(ec_pub);
+	BN_free(ec_priv);
 }
 	TEST_DONE();
 #endif /* OPENSSL_HAS_ECC */
@@ -465,19 +424,18 @@ sshkey_tests(void)
 	ASSERT_PTR_NE(kr, k1);
 	ASSERT_INT_EQ(k1->type, KEY_RSA);
 	ASSERT_PTR_NE(k1->pk, NULL);
-{
-	const BIGNUM *n = NULL, *e = NULL, *p = NULL;
-	RSA *rsa = EVP_PKEY_get1_RSA(k1->pk);
+{	BIGNUM *n = NULL, *e = NULL, *p = NULL;
 
-	ASSERT_PTR_NE(rsa, NULL);
-	RSA_get0_key(rsa, &n, &e, NULL);
-	RSA_get0_factors(rsa, &p, NULL);
+	ssh_EVP_PKEY_RSA_get1_key(k1->pk, &n, &e, NULL);
+	ssh_EVP_PKEY_RSA_get1_factors(k1->pk, &p, NULL);
 
 	ASSERT_PTR_NE(n, NULL);
 	ASSERT_PTR_NE(e, NULL);
 	ASSERT_PTR_EQ(p, NULL);
 
-	RSA_free(rsa);
+	BN_free(n); /* not secret */
+	BN_free(e); /* not secret */
+	BN_clear_free(p);
 }
 	TEST_DONE();
 
@@ -494,18 +452,16 @@ sshkey_tests(void)
 	ASSERT_PTR_NE(kd, k1);
 	ASSERT_INT_EQ(k1->type, KEY_DSA);
 	ASSERT_PTR_NE(k1->pk, NULL);
-{
-	const BIGNUM *g = NULL, *priv_key = NULL;
-	DSA *dsa = EVP_PKEY_get1_DSA(k1->pk);
+{	BIGNUM *g = NULL, *priv_key = NULL;
 
-	ASSERT_PTR_NE(dsa, NULL);
-	DSA_get0_pqg(dsa, NULL, NULL, &g);
-	DSA_get0_key(dsa, NULL, &priv_key);
+	ssh_EVP_PKEY_DSA_get1_pqg(k1->pk, NULL, NULL, &g);
+	ssh_EVP_PKEY_DSA_get1_key(k1->pk, NULL, &priv_key);
 
 	ASSERT_PTR_NE(g, NULL);
 	ASSERT_PTR_EQ(priv_key, NULL);
 
-	DSA_free(dsa);
+	BN_free(g);
+	BN_clear_free(priv_key);
 }
 	TEST_DONE();
 
@@ -522,21 +478,22 @@ sshkey_tests(void)
 	ASSERT_PTR_NE(k1, NULL);
 	ASSERT_PTR_NE(ke, k1);
 	ASSERT_INT_EQ(k1->type, KEY_ECDSA);
-	ASSERT_PTR_NE(k1->pk, NULL);
-{
-	const EC_POINT *ec_pub;
-	const BIGNUM *ec_priv;
-	EC_KEY *ec = EVP_PKEY_get1_EC_KEY(k1->pk);
-
-	ASSERT_PTR_NE(ec, NULL);
-	ec_pub = EC_KEY_get0_public_key(ec);
-	ec_priv = EC_KEY_get0_private_key(ec);
-
 	ASSERT_INT_EQ(k1->ecdsa_nid, ke->ecdsa_nid);
+	ASSERT_PTR_NE(k1->pk, NULL);
+{	EC_GROUP *ec_group = NULL;
+	EC_POINT *ec_pub = NULL;
+	BIGNUM *ec_priv = NULL;
+
+	ssh_EVP_PKEY_EC_KEY_get1_public_key(k1->pk, &ec_group, &ec_pub);
+	ssh_EVP_PKEY_EC_KEY_get1_private_key(k1->pk, &ec_priv);
+
+	ASSERT_PTR_NE(ec_group, NULL);
 	ASSERT_PTR_NE(ec_pub, NULL);
 	ASSERT_PTR_EQ(ec_priv, NULL);
 
-	EC_KEY_free(ec);
+	EC_GROUP_free(ec_group);
+	EC_POINT_free(ec_pub);
+	BN_free(ec_priv);
 }
 	TEST_DONE();
 
@@ -751,8 +708,8 @@ sshkey_tests(void)
 	sshbuf_free(b);
 }
 
-void
-sshkey_benchmarks(void)
+static void
+sshkey_generate_benchmarks(void)
 {
 	struct sshkey *k = NULL;
 
@@ -774,6 +731,15 @@ sshkey_benchmarks(void)
 	TEST_DONE();
 	BENCH_FINISH("keys");
 
+	BENCH_START("generate RSA-3072");
+	TEST_START("generate KEY_RSA");
+	ASSERT_INT_EQ(sshkey_generate(KEY_RSA, 3072, &k), 0);
+	ASSERT_PTR_NE(k, NULL);
+	sshkey_free(k);
+	k = NULL;
+	TEST_DONE();
+	BENCH_FINISH("keys");
+
 #ifdef OPENSSL_HAS_ECC
 	BENCH_START("generate ECDSA-256");
 	TEST_START("generate KEY_ECDSA");
@@ -815,7 +781,11 @@ sshkey_benchmarks(void)
 	k = NULL;
 	TEST_DONE();
 	BENCH_FINISH("keys");
+}
 
+static void
+sshkey_signature_benchmarks(void)
+{
 	/* sign */
 	signature_benchmark("RSA-1024/SHA1", KEY_RSA, 1024, "ssh-rsa", 0);
 #ifdef HAVE_EVP_SHA256
@@ -826,6 +796,11 @@ sshkey_benchmarks(void)
 #ifdef HAVE_EVP_SHA256
 	signature_benchmark("RSA-2048/SHA256", KEY_RSA, 2048, "rsa-sha2-256", 0);
 	signature_benchmark("RSA-2048/SHA512", KEY_RSA, 2048, "rsa-sha2-512", 0);
+#endif
+	signature_benchmark("RSA-3072/SHA1", KEY_RSA, 3072, "ssh-rsa", 0);
+#ifdef HAVE_EVP_SHA256
+	signature_benchmark("RSA-3072/SHA256", KEY_RSA, 3072, "rsa-sha2-256", 0);
+	signature_benchmark("RSA-3072/SHA512", KEY_RSA, 3072, "rsa-sha2-512", 0);
 #endif
 #ifdef OPENSSL_HAS_ECC
 	signature_benchmark("ECDSA-256", KEY_ECDSA, 256, NULL, 0);
@@ -846,6 +821,11 @@ sshkey_benchmarks(void)
 #ifdef HAVE_EVP_SHA256
 	signature_benchmark("RSA-2048/SHA256", KEY_RSA, 2048, "rsa-sha2-256", 1);
 	signature_benchmark("RSA-2048/SHA512", KEY_RSA, 2048, "rsa-sha2-512", 1);
+#endif
+	signature_benchmark("RSA-3072/SHA1", KEY_RSA, 3072, "ssh-rsa", 1);
+#ifdef HAVE_EVP_SHA256
+	signature_benchmark("RSA-3072/SHA256", KEY_RSA, 3072, "rsa-sha2-256", 1);
+	signature_benchmark("RSA-3072/SHA512", KEY_RSA, 3072, "rsa-sha2-512", 1);
 #endif
 #ifdef OPENSSL_HAS_ECC
 	signature_benchmark("ECDSA-256", KEY_ECDSA, 256, NULL, 1);
@@ -856,3 +836,10 @@ sshkey_benchmarks(void)
 #endif
 	signature_benchmark("ED25519", KEY_ED25519, 0, NULL, 1);
 }
+
+void
+sshkey_benchmarks(void)
+{
+	sshkey_generate_benchmarks();
+	sshkey_signature_benchmarks();
+}
