@@ -1,4 +1,4 @@
-/*	$OpenBSD: test_helper.c,v 1.14 2025/04/15 04:00:42 djm Exp $	*/
+/*	$OpenBSD: test_helper.c,v 1.15 2025/12/05 11:13:35 djm Exp $	*/
 /*
  * Copyright (c) 2011 Damien Miller <djm@mindrot.org>
  *
@@ -583,6 +583,51 @@ assert_u64(const char *file, int line, const char *a1, const char *a2,
 	test_die();
 }
 
+#ifndef __DBL_EPSILON__
+# define __DBL_EPSILON__ 1.e-9
+#endif
+
+void
+assert_double(const char *file, int line, const char *a1, const char *a2,
+    double aa1, double aa2, enum test_predicate pred)
+{
+	const double epsilon = __DBL_EPSILON__;
+
+	switch (pred) {
+	case TEST_EQ:
+		if (fabs(aa1 - aa2) < epsilon)
+			return;
+		break;
+	case TEST_NE:
+		if (fabs(aa1 - aa2) >= epsilon)
+			return;
+		break;
+	case TEST_LT:
+		if (aa1 < aa2)
+			return;
+		break;
+	case TEST_LE:
+		if (aa1 <= aa2)
+			return;
+		break;
+	case TEST_GT:
+		if (aa1 > aa2)
+			return;
+		break;
+	case TEST_GE:
+		if (aa1 >= aa2)
+			return;
+		break;
+	default:
+		abort();
+	}
+
+	test_header(file, line, a1, a2, "DOUBLE", pred);
+	fprintf(stderr, "%12s = %f\n", a1, aa1);
+	fprintf(stderr, "%12s = %f\n", a2, aa2);
+	test_die();
+}
+
 void
 assert_ptr(const char *file, int line, const char *a1, const char *a2,
     const void *aa1, const void *aa2, enum test_predicate pred)
@@ -626,17 +671,17 @@ bench_done(void)
 }
 
 void
-bench_case_start()
+bench_case_start(void)
 {
-	clock_gettime(CLOCK_REALTIME, &bench_start_time);
+	monotime_ts(&bench_start_time);
 }
 
 void
-bench_case_finish()
+bench_case_finish(void)
 {
 	struct timespec ts;
 
-	clock_gettime(CLOCK_REALTIME, &bench_finish_time);
+	monotime_ts(&bench_finish_time);
 	timespecsub(&bench_finish_time, &bench_start_time, &ts);
 	if (bench_nruns >= bench_nalloc) {
 		if (bench_nalloc >= INT_MAX / 2) {
@@ -701,7 +746,7 @@ bench_finish(const char *file, int line, const char *unit)
 		    mean_spr * 1000, med_spr * 1000, unit, std_dev * 1000,
 		    mean_rps, med_rps, unit);
 	else
-		printf("%-40s %0.2f %s/s\n", bench_name, med_rps, unit);
+		printf("%-40s %11.2f %s/s\n", bench_name, med_rps, unit);
 
 	bench_name = NULL;
 	bench_nruns = 0;
