@@ -1,4 +1,4 @@
-/* $OpenBSD: session.c,v 1.344 2025/09/25 02:15:39 jsg Exp $ */
+/* $OpenBSD: session.c,v 1.345 2025/11/17 12:59:29 jca Exp $ */
 /*
  * Copyright (c) 1995 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
  *                    All rights reserved
@@ -1062,6 +1062,15 @@ do_setup_env(struct ssh *ssh, Session *s, const char *shell)
 	if (getenv("TZ"))
 		child_set_env(&env, &envsize, "TZ", getenv("TZ"));
 
+	/*
+	 * Next variable is set by pam_systemd(8) or if LOGIN_SETXDGENV
+	 * is part of setusercontext(LOGIN_SETALL).
+	 */
+	{
+		if ((cp = getenv("XDG_RUNTIME_DIR")) != NULL)
+			child_set_env(&env, &envsize, "XDG_RUNTIME_DIR", cp);
+	}
+
 	/*
 	 * Since we clear KRB5CCNAME at startup, if it's set now then it
 	 * must have been set by a native authentication method (eg AIX or
