@@ -1,4 +1,4 @@
-/* $OpenBSD: sftp-server.c,v 1.149 2025/09/02 09:26:21 djm Exp $ */
+/* $OpenBSD: sftp-server.c,v 1.150 2025/12/08 00:41:46 djm Exp $ */
 /*
  * Copyright (c) 2000-2004 Markus Friedl.  All rights reserved.
  * Copyright (c) 2021-2025 Roumen Petrov.  All rights reserved.
@@ -52,7 +52,7 @@
 /*
  * Broken realpath: return path as is if does not exist.
  * Goal is to use it (temporary?) in sftp related code that relies
- * on broken OpenBSD implemenation.
+ * on broken OpenBSD implementation.
  * NOTE: there is no such requirement in filexfer-03 draft.
  */
 static char*
@@ -1681,7 +1681,7 @@ process_extended_get_users_groups_by_id(u_int32_t id)
 		debug3_f("gid %u => \"%s\"", n, name);
 		if ((r = sshbuf_put_cstring(groupnames, name)) != 0)
 			fatal_fr(r, "assemble gid reply");
-		nusers++;
+		ngroups++;
 	}
 	verbose("users-groups-by-id: %u users, %u groups", nusers, ngroups);
 
