@@ -1,4 +1,4 @@
-/* $OpenBSD: ssh-add.c,v 1.175 2025/08/29 03:50:38 djm Exp $ */
+/* $OpenBSD: ssh-add.c,v 1.184 2025/11/24 23:43:10 djm Exp $ */
 /*
  * Author: Tatu Ylonen <ylo@cs.hut.fi>
  * Copyright (c) 1995 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
@@ -477,9 +477,28 @@ test_key(int agent_fd, const char *filename)
 	return ret;
 }
 
+static int
+print_agent_identity(struct sshbuf *b, struct ssh_identitylist *idlist,
+    size_t k, int print_dn) {
+	int r;
+
+	r = print_dn
+	    ? Xkey_printb(NULL, idlist->keys[k], b)
+	    : sshkey_format_text(idlist->keys[k], b);
+	if (r != 0) return r;
+
+	if (!print_dn) {
+		r = sshbuf_putf(b, " %s", idlist->comments[k]);
+		if (r != 0) return r;
+	}
+
+	return sshbuf_putf(b, "\n");
+}
+
 static int
 list_identities(int agent_fd, int do_fp, int key_only)
 {
+	struct sshbuf *b;
 	char *fp;
 	int r;
 	struct ssh_identitylist *idlist;
@@ -493,6 +512,10 @@ list_identities(int agent_fd, int do_fp, int key_only)
 			printf("The agent has no identities.\n");
 		return -1;
 	}
+
+	b = sshbuf_new();
+	if (b == NULL) goto out;
+
 	for (i = 0; i < idlist->nkeys; i++) {
 		if (do_fp) {
 			fp = sshkey_fingerprint(idlist->keys[i],
@@ -503,24 +526,21 @@ list_identities(int agent_fd, int do_fp, int key_only)
 			free(fp);
 		} else {
 			int print_dn = !key_only && sshkey_is_x509(idlist->keys[i]);
-			if (print_dn) {
-				if(!Xkey_write_subject(NULL,
-				    idlist->keys[i], stdout)) {
-					fprintf(stderr,
-					    "Xkey_write_subject failed\n");
-					continue;
-				}
-			} else
-			if ((r = sshkey_write(idlist->keys[i], stdout)) != 0) {
-				fprintf(stderr, "sshkey_write: %s\n",
+
+			sshbuf_reset(b);
+
+			r = print_agent_identity(b, idlist, i, print_dn);
+			if (r != 0 ) {
+				fprintf(stderr, "print agent identity failed: %s\n",
 				    ssh_err(r));
 				continue;
 			}
-			if (!print_dn)
-				fprintf(stdout, " %s", idlist->comments[i]);
-			fprintf(stdout, "\n");
+
+			fwrite(sshbuf_ptr(b), sshbuf_len(b), 1, stdout);
 		}
 	}
+out:
+	sshbuf_free(b);
 	ssh_free_identitylist(idlist);
 	return 0;
 }
