@@ -1,4 +1,4 @@
-/* $OpenBSD: ssh-agent.c,v 1.313 2025/08/29 03:50:38 djm Exp $ */
+/* $OpenBSD: ssh-agent.c,v 1.314 2025/11/07 04:33:52 djm Exp $ */
 /*
  * Author: Tatu Ylonen <ylo@cs.hut.fi>
  * Copyright (c) 1995 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
@@ -102,8 +102,8 @@ typedef struct socket_entry {
 	struct sshbuf *request;
 } SocketEntry;
 
-u_int sockets_alloc = 0;
-SocketEntry *sockets = NULL;
+static u_int sockets_alloc = 0;
+static SocketEntry *sockets = NULL;
 
 typedef struct identity {
 	TAILQ_ENTRY(identity) next;
@@ -120,23 +120,23 @@ struct idtable {
 };
 
 /* private key table */
-struct idtable *idtab;
+static struct idtable *idtab = NULL;
 
-int max_fd = 0;
+static int max_fd = 0;
 
 /* pid of shell == parent of agent */
-pid_t parent_pid = -1;
-time_t parent_alive_interval = 0;
+static pid_t parent_pid = -1;
+static time_t parent_alive_interval = 0;
 
 static volatile sig_atomic_t signalled_exit = 0;
 static volatile sig_atomic_t signalled_keydrop = 0;
 
 /* pid of process for which cleanup_socket is applicable */
-pid_t cleanup_pid = 0;
+static pid_t cleanup_pid = 0;
 
 /* pathname and directory for AUTH_SOCKET */
-char socket_name[PATH_MAX];
-char socket_dir[PATH_MAX];
+static char *socket_name = NULL;
+static char socket_dir[PATH_MAX];
 
 /* Pattern-list of allowed PKCS#11 paths */
 static char *allowed_providers = NULL;
@@ -145,9 +145,9 @@ static char *allowed_providers = NULL;
 #define LOCK_SIZE	32
 #define LOCK_SALT_SIZE	16
 #define LOCK_ROUNDS	1
-int locked = 0;
-u_char lock_pwhash[LOCK_SIZE];
-u_char lock_salt[LOCK_SALT_SIZE];
+static int locked = 0;
+static u_char lock_pwhash[LOCK_SIZE];
+static u_char lock_salt[LOCK_SALT_SIZE];
 
 extern char *__progname;
 
@@ -266,7 +266,7 @@ process_request_identities(SocketEntry *e)
 	    (r = sshbuf_put_u32(msg, idtab->nentries)) != 0)
 		fatal_fr(r, "compose");
 	TAILQ_FOREACH(id, &idtab->idlist, next) {
-		if ((r = Akey_puts_opts(id->key, msg, SSHKEY_SERIALIZE_DEFAULT)) != 0 ||
+		if ((r = Akey_puts(id->key, msg)) != 0 ||
 		    (r = sshbuf_put_cstring(msg, id->comment)) != 0) {
 			error_fr(r, "compose key/comment");
 			continue;
@@ -1149,8 +1149,11 @@ cleanup_socket(void)
 	if (cleanup_pid != 0 && getpid() != cleanup_pid)
 		return;
 	debug_f("cleanup");
-	if (socket_name[0])
+	if (socket_name != NULL) {
 		unlink(socket_name);
+		free(socket_name);
+		socket_name = NULL;
+	}
 	if (socket_dir[0])
 		rmdir(socket_dir);
 }
@@ -1202,6 +1205,16 @@ usage(void)
 	exit(1);
 }
 
+static void
+print_auth_sock(const char *format)
+{
+	char *cp;
+
+	cp = argv_assemble(1, &socket_name);
+	printf(format, SSH_AUTHSOCKET_ENV_NAME, cp, SSH_AUTHSOCKET_ENV_NAME);
+	free(cp);
+}
+
 int
 main(int ac, char **av)
 {
@@ -1210,11 +1223,11 @@ main(int ac, char **av)
 	LogLevel log_level = SYSLOG_LEVEL_INFO;
 	int log_stderr = 0;
 	int sock = -1, ch, result, saved_errno;
+	pid_t pid;
 	char *shell, *format, *pidstr, *agentsocket = NULL;
+	char pidstrbuf[1 + 3 * sizeof pid];
 	extern int optind;
 	extern char *optarg;
-	pid_t pid;
-	char pidstrbuf[1 + 3 * sizeof pid];
 	size_t len;
 	mode_t prev_mask;
 	struct pollfd *pfd = NULL;
@@ -1222,7 +1235,6 @@ main(int ac, char **av)
 	u_int maxfds;
 	sigset_t nsigset;
 
-	socket_name[0] = '\0';
 	socket_dir[0] = '\0';
 
 	ssh_malloc_init();	/* must be called before any mallocs */
@@ -1388,20 +1400,16 @@ main(int ac, char **av)
 				perror("mkdtemp: private socket dir");
 				exit(1);
 			}
-		{	int r = snprintf(socket_name, sizeof socket_name,
-			    "%s/agent.%ld", socket_dir, (long)parent_pid);
-			if (r < 0 || r > (int)sizeof(socket_name)) {
+			if (xasprintf(&socket_name, "%s/agent.%ld",
+			    socket_dir, (long)parent_pid) >= PATH_MAX) {
 				fprintf(stderr, "too long socket path\n");
 				exit(1);
 			}
-		}
+
 		} else {
 			/* Try to use specified agent socket */
-			if (strlcpy(socket_name, agentsocket, sizeof socket_name)
-			    > sizeof(socket_name)) {
-				fprintf(stderr, "too long agent socket path\n");
-				exit(1);
-			}
+			socket_dir[0] = '\0';
+			socket_name = xstrdup(agentsocket);
 		}
 	}
 
@@ -1420,17 +1428,15 @@ main(int ac, char **av)
 		umask(prev_mask);
 	}
 
+	format = c_flag ? "setenv %s %s;\n" : "%s=%s; export %s;\n";
 	/*
 	 * Fork, and have the parent execute the command, if any, or present
 	 * the socket data.  The child continues as the authentication agent.
 	 */
 	if (D_flag || d_flag) {
 		log_init(__progname, log_level, log_facility, log_stderr);
-		if (socket_name[0] != '\0') {
-			format = c_flag ?
-			    "setenv %s %s;\n" : "%s=%s; export %s;\n";
-			printf(format, SSH_AUTHSOCKET_ENV_NAME, socket_name,
-			    SSH_AUTHSOCKET_ENV_NAME);
+		if (socket_name != NULL) {
+			print_auth_sock(format);
 			printf("echo Agent pid %ld;\n", (long)parent_pid);
 			fflush(stdout);
 		}
@@ -1445,12 +1451,11 @@ main(int ac, char **av)
 		close(sock);
 		snprintf(pidstrbuf, sizeof pidstrbuf, "%ld", (long)pid);
 		if (ac == 0) {
-			format = c_flag ? "setenv %s %s;\n" : "%s=%s; export %s;\n";
-			printf(format, SSH_AUTHSOCKET_ENV_NAME, socket_name,
-			    SSH_AUTHSOCKET_ENV_NAME);
+			print_auth_sock(format);
 			printf(format, SSH_AGENTPID_ENV_NAME, pidstrbuf,
 			    SSH_AGENTPID_ENV_NAME);
 			printf("echo Agent pid %ld;\n", (long)pid);
+			fflush(stdout);
 			exit(0);
 		}
 		if (setenv(SSH_AUTHSOCKET_ENV_NAME, socket_name, 1) == -1 ||
