@@ -49,6 +49,9 @@
 
 #include "evp-compat.h"
 #include <openssl/bn.h>
+#ifdef ENABLE_KEY_PROVIDERS
+# include <openssl/core_names.h>
+#endif
 
 #include <stdarg.h>
 #include <string.h>
@@ -71,6 +74,8 @@
 #endif
 
 
+#define SSH_DSA_BITS			1024
+
 #define INTBLOB_LEN	20
 #define SIGBLOB_LEN	(2*INTBLOB_LEN)
 
@@ -166,6 +171,70 @@ DSA_set0_pqg(DSA *dsa, BIGNUM *p, BIGNUM *q, BIGNUM *g) {
 }
 #endif /* ndef HAVE_DSA_GET0_KEY */
 
+
+void
+ssh_EVP_PKEY_DSA_get1_key(EVP_PKEY *pkey,
+    BIGNUM **pubp, BIGNUM **privp
+) {
+#ifdef ENABLE_KEY_PROVIDERS
+	if (EVP_PKEY_get0_provider(pkey) != NULL) {
+		if (pubp != NULL)
+			(void)EVP_PKEY_get_bn_param(pkey,
+			    OSSL_PKEY_PARAM_PUB_KEY, pubp);
+		if (privp != NULL)
+			(void)EVP_PKEY_get_bn_param(pkey,
+			    OSSL_PKEY_PARAM_PRIV_KEY, privp);
+		return;
+	}
+#endif
+{	const BIGNUM *pub_key, *priv_key;
+	DSA *dsa = EVP_PKEY_get1_DSA(pkey);
+
+	if (dsa == NULL) return;
+
+	DSA_get0_key(dsa, &pub_key, &priv_key);
+
+	if (pubp != NULL) *pubp = BN_dup(pub_key);
+	if (privp != NULL) *privp = BN_dup(priv_key);
+
+	DSA_free(dsa);
+}
+}
+
+void
+ssh_EVP_PKEY_DSA_get1_pqg(EVP_PKEY *pkey,
+    BIGNUM **pp, BIGNUM **qp, BIGNUM **gp
+) {
+#ifdef ENABLE_KEY_PROVIDERS
+	if (EVP_PKEY_get0_provider(pkey) != NULL) {
+		if (pp != NULL)
+			(void)EVP_PKEY_get_bn_param(pkey,
+			    OSSL_PKEY_PARAM_FFC_P, pp);
+		if (qp != NULL)
+			(void)EVP_PKEY_get_bn_param(pkey,
+			    OSSL_PKEY_PARAM_FFC_Q, qp);
+		if (gp != NULL)
+			(void)EVP_PKEY_get_bn_param(pkey,
+			    OSSL_PKEY_PARAM_FFC_G, gp);
+		return;
+	}
+#endif
+{	const BIGNUM *p = NULL, *q = NULL, *g = NULL;
+	DSA *dsa = EVP_PKEY_get1_DSA(pkey);
+
+	if (dsa == NULL) return;
+
+	DSA_get0_pqg(dsa, &p, &q, &g);
+
+	if (pp != NULL) *pp = BN_dup(p);
+	if (qp != NULL) *qp = BN_dup(q);
+	if (gp != NULL) *gp = BN_dup(g);
+
+	DSA_free(dsa);
+}
+}
+
+
 #ifndef HAVE_EVP_PKEY_CMP	/* OpenSSL < 0.9.8 */
 extern int /* see sshkey-crypto.c */
 ssh_EVP_PKEY_cmp_dsa(const EVP_PKEY *ka, const EVP_PKEY *kb);
@@ -307,31 +376,6 @@ ssh_EVP_PKEY_complete_pub_dsa(EVP_PKEY *pk) {
 }
 
 
-extern int /* see sshkey-crypto.c */
-sshkey_from_pkey_dsa(EVP_PKEY *pk, struct sshkey **keyp);
-
-int
-sshkey_from_pkey_dsa(EVP_PKEY *pk, struct sshkey **keyp) {
-	int r;
-	struct sshkey* key;
-
-	r = ssh_EVP_PKEY_complete_pub_dsa(pk);
-	if (r != 0) return r;
-
-	key = sshkey_new(KEY_UNSPEC);
-	if (key == NULL)
-		return SSH_ERR_ALLOC_FAIL;
-
-	/* success */
-	key->type = KEY_DSA;
-	key->pk = pk;
-
-	SSHKEY_DUMP(key);
-	*keyp = key;
-	return 0;
-}
-
-
 static int
 sshbuf_read_pub_dsa(struct sshbuf *buf, struct sshkey *key) {
 	int r;
@@ -430,7 +474,7 @@ sshbuf_write_priv_dsa(struct sshbuf *buf, const struct sshkey *key) {
 }
 
 
-extern int /* method used localy only in ssh-keygen.c */
+extern int /* method used locally only in ssh-keygen.c */
 sshbuf_read_custom_dsa(struct sshbuf *buf, struct sshkey *key);
 
 int
@@ -475,6 +519,38 @@ err:
 #endif /* def WITH_OPENSSL_4_0_API */
 
 
+/* pkey implementation */
+
+static int
+ssh_dss_sshkey_from_pkey(EVP_PKEY *pk, struct sshkey **keyp) {
+	int r;
+	struct sshkey* key;
+
+	r = ssh_EVP_PKEY_complete_pub_dsa(pk);
+	if (r != 0) return r;
+
+	key = sshkey_new(KEY_UNSPEC);
+	if (key == NULL)
+		return SSH_ERR_ALLOC_FAIL;
+
+	/* success */
+	key->type = KEY_DSA;
+	key->pk = pk;
+
+	SSHKEY_DUMP(key);
+	*keyp = key;
+	return 0;
+}
+
+static const struct ssh_pkey_impl_funcs
+ssh_pkey_dsa_funcs = {
+	ssh_dss_sshkey_from_pkey,
+	ssh_pkey_validate_public_dsa
+};
+
+#define SSH_PKEY_DSA_FUNCS	&ssh_pkey_dsa_funcs
+
+
 /* key implementation */
 
 static u_int
@@ -496,21 +572,20 @@ ssh_dss_equal(const struct sshkey *a, const struct sshkey *b)
 }
 
 static int
-ssh_dss_serialize_public(const struct sshkey *key, struct sshbuf *buf,
-    enum sshkey_serialize_rep opts)
+ssh_dss_serialize_public(const struct sshkey_impl *impl,
+    const struct sshkey *key, struct sshbuf *buf)
 {
-	UNUSED(opts);
+	UNUSED(impl);
 	return sshbuf_write_pub_dsa(buf, key);
 }
 
 static int
-ssh_dss_serialize_private(const struct sshkey *key, struct sshbuf *buf,
-    enum sshkey_serialize_rep opts)
+ssh_dss_serialize_private(const struct sshkey_impl *impl,
+    const struct sshkey *key, struct sshbuf *buf)
 {
 	int r;
 
-	UNUSED(opts);
-	if (!sshkey_is_cert(key)) {
+	if (!impl->cert) {
 		if ((r = sshbuf_write_pub_dsa(buf, key)) != 0)
 			return r;
 	}
@@ -664,6 +739,20 @@ ssh_dss_generate(struct sshkey *key, int bits) {
 	return r;
 }
 
+static u_int
+ssh_dss_default_bits(const char *name) {
+	UNUSED(name);
+	return SSH_DSA_BITS;
+}
+
+static void
+ssh_dss_valid_bits(int bits) {
+	if (sshdsa_verify_length(bits) == 0)
+		return;
+
+	fatal("Invalid DSA key length: must be %d bits", SSH_DSA_BITS);
+}
+
 static void
 ssh_dss_move_public(struct sshkey *from, struct sshkey *to) {
 	sshkey_move_pk(from, to);
@@ -713,21 +802,20 @@ err:
 }
 
 static int
-ssh_dss_deserialize_public(const char *pkalg, struct sshbuf *buf,
-    struct sshkey *key)
+ssh_dss_deserialize_public(const struct sshkey_impl *impl,
+    struct sshbuf *buf, struct sshkey *key)
 {
-	UNUSED(pkalg);
+	UNUSED(impl);
 	return sshbuf_read_pub_dsa(buf, key);
 }
 
 static int
-ssh_dss_deserialize_private(const char *pkalg, struct sshbuf *buf,
-    struct sshkey *key)
+ssh_dss_deserialize_private(const struct sshkey_impl *impl,
+    struct sshbuf *buf, struct sshkey *key)
 {
 	int r;
 
-	UNUSED(pkalg);
-	if (!sshkey_is_cert(key)) {
+	if (!impl->cert) {
 		if ((r = sshbuf_read_pub_dsa(buf, key))!= 0)
 			return r;
 	}
@@ -834,6 +922,8 @@ static const struct sshkey_impl_funcs sshkey_dss_funcs = {
 	/* .serialize_private = */	ssh_dss_serialize_private,
 	/* .deserialize_private = */	ssh_dss_deserialize_private,
 	/* .generate = */	ssh_dss_generate,
+	/* .default_bits = */	ssh_dss_default_bits,
+	/* .valid_bits = */	ssh_dss_valid_bits,
 	/* .move_public = */	ssh_dss_move_public,
 	/* .copy_public = */	ssh_dss_copy_public,
 	/* .sign = */		ssh_dss_sign,
@@ -845,20 +935,21 @@ static int sshkey_dss_enabled(void) { return 1; }
 
 #else
 
-#include "sshkey.h"
+#include "sshxkey.h"
 
 #define KEY_DSA KEY_UNSPEC
 #define KEY_DSA_CERT KEY_UNSPEC
 
 static int sshkey_dss_enabled(void) { return 0; }
 #define SSHKEY_DSS_FUNCS	NULL
+#define SSH_PKEY_DSA_FUNCS	NULL
 
 #endif /* WITH_DSA */
 
-static char *pkey_dsa_typename[] = { "DSA", NULL };
+static const char *pkey_dsa_typename[] = { "DSA", NULL };
 
 const struct sshkey_impl sshkey_dss_impl = {
-	/* .id = */		{ KEY_DSA, 0,
+	/* .id = */		{ KEY_DSA, -1,
 				  EVP_PKEY_DSA, pkey_dsa_typename },
 	/* .name = */		"ssh-dss",
 	/* .shortname = */	"DSA",
@@ -871,7 +962,7 @@ const struct sshkey_impl sshkey_dss_impl = {
 };
 
 const struct sshkey_impl sshkey_dsa_cert_impl = {
-	/* .id = */		{ KEY_DSA_CERT, 0,
+	/* .id = */		{ KEY_DSA_CERT, -1,
 				  EVP_PKEY_DSA, pkey_dsa_typename },
 	/* .name = */		"ssh-dss-cert-v01@openssh.com",
 	/* .shortname = */	"DSA-CERT",
@@ -882,3 +973,9 @@ const struct sshkey_impl sshkey_dsa_cert_impl = {
 	/* .enabled = */	sshkey_dss_enabled,
 	/* .funcs = */		SSHKEY_DSS_FUNCS
 };
+
+const struct ssh_pkey_impl ssh_pkey_dsa_impl = {
+	pkey_dsa_typename,
+	EVP_PKEY_DSA,
+	SSH_PKEY_DSA_FUNCS
+};
