@@ -1,4 +1,4 @@
-/* $OpenBSD: ssh-ecdsa.c,v 1.27 2024/08/15 00:51:51 djm Exp $ */
+/* $OpenBSD: ssh-ecdsa.c,v 1.28 2025/07/24 05:44:55 djm Exp $ */
 /*
  * Copyright (c) 2000 Markus Friedl.  All rights reserved.
  * Copyright (c) 2010 Damien Miller.  All rights reserved.
@@ -57,7 +57,11 @@
 #include "log.h"
 #include "xmalloc.h"
 
+#if 0
+# define SKIP_AFFINE_COORDINATES_CHECK
+#endif
 
+#ifdef SKIP_AFFINE_COORDINATES_CHECK
 #ifndef HAVE_EC_POINT_GET_AFFINE_COORDINATES		/* OpenSSL < 1.1.1 */
 /* Functions are available even in 0.9.7* but EC is not activated
  * as NIST curves are not supported yet.
@@ -70,6 +74,7 @@ EC_POINT_get_affine_coordinates(
 	return EC_POINT_get_affine_coordinates_GFp(group, p, x, y, ctx);
 }
 #endif /*ndef HAVE_EC_POINT_GET_AFFINE_COORDINATES*/
+#endif /*def SKIP_AFFINE_COORDINATES_CHECK*/
 
 #ifdef ENABLE_KEY_PROVIDERS
 static size_t
@@ -106,6 +111,30 @@ err:
 }
 #endif /*def ENABLE_KEY_PROVIDERS*/
 
+void
+ssh_EVP_PKEY_EC_KEY_get1_private_key(EVP_PKEY *pkey,
+    BIGNUM **privp
+) {
+#ifdef ENABLE_KEY_PROVIDERS
+	if (EVP_PKEY_get0_provider(pkey) != NULL) {
+		if (privp != NULL)
+			(void)EVP_PKEY_get_bn_param(pkey,
+			    OSSL_PKEY_PARAM_PRIV_KEY, privp);
+		return;
+	}
+#endif
+{	const BIGNUM *priv_key;
+	EC_KEY *ec = EVP_PKEY_get1_EC_KEY(pkey);
+
+	if (ec == NULL) return;
+
+	priv_key = EC_KEY_get0_private_key(ec);
+	if (privp != NULL) *privp = BN_dup(priv_key);
+
+	EC_KEY_free(ec);
+}
+}
+
 
 static int
 nid_list[] = {
@@ -147,6 +176,64 @@ ssh_ecdsa_dgst(const struct sshkey *key) {
 }
 
 
+static inline u_int
+sshkey_curve_nid_to_bits(int nid)
+{
+	switch (nid) {
+	case NID_X9_62_prime256v1: return 256;
+	case NID_secp384r1: return 384;
+#ifdef OPENSSL_HAS_NISTP521
+	case NID_secp521r1: return 521;
+#endif /* OPENSSL_HAS_NISTP521 */
+	}
+	return 0;
+}
+
+static inline int
+sshkey_curve_bits_to_nid(int bits)
+{
+	switch (bits) {
+	case 256: return NID_X9_62_prime256v1;
+	case 384: return NID_secp384r1;
+#ifdef OPENSSL_HAS_NISTP521
+	case 521: return NID_secp521r1;
+#endif /* OPENSSL_HAS_NISTP521 */
+	}
+	return NID_undef;
+}
+
+
+static inline int
+sshkey_curve_name_to_nid(const char *name)
+{
+	if (strcmp(name, "nistp256") == 0)
+		return NID_X9_62_prime256v1;
+	if (strcmp(name, "nistp384") == 0)
+		return NID_secp384r1;
+# ifdef OPENSSL_HAS_NISTP521
+	if (strcmp(name, "nistp521") == 0)
+		return NID_secp521r1;
+# endif /* OPENSSL_HAS_NISTP521 */
+	return NID_undef;
+}
+
+static inline const char *
+sshkey_curve_nid_to_name(int nid)
+{
+	switch (nid) {
+	case NID_X9_62_prime256v1:
+		return "nistp256";
+	case NID_secp384r1:
+		return "nistp384";
+# ifdef OPENSSL_HAS_NISTP521
+	case NID_secp521r1:
+		return "nistp521";
+# endif /* OPENSSL_HAS_NISTP521 */
+	}
+	return NULL;
+}
+
+
 #ifdef WITH_OPENSSL_4_0_API
 /* TODO: new methods compatible with OpenSSL 4.0 API.
  * Remark: OpenSSL 3* is too buggy - almost each release fail
@@ -205,7 +292,7 @@ ssh_EC_KEY_new_by_curve_name(int nid) {
 #if defined(OPENSSL_VERSION_NUMBER) && (OPENSSL_VERSION_NUMBER < 0x10100000L) || \
     defined(LIBRESSL_VERSION_NUMBER)
 	/* Note since 1.1.0 OpenSSL uses named curve parameter encoding by default.
-	 * It seems to me default is changed in upcomming 3.0 but key is marked
+	 * It seems to me default is changed in upcoming 3.0 but key is marked
 	 * properly when created by nid.
 	 */
 	EC_KEY_set_asn1_flag(ec, OPENSSL_EC_NAMED_CURVE);
@@ -247,13 +334,12 @@ err:
 }
 
 static int
-sshbuf_read_ec_curve(struct sshbuf *buf, const char *pkalg, struct sshkey *key) {
-	int r, nid;
+sshbuf_read_ec_curve(struct sshbuf *buf,
+    const struct sshkey_impl *impl, struct sshkey *key)
+{
+	int r, nid = impl->id.subtype;
 
-	nid = sshkey_ecdsa_nid_from_name(pkalg);
-	debug3_f("pkalg/nid: %s/%d", pkalg, nid);
-	if (nid == -1)
-		return SSH_ERR_INVALID_ARGUMENT;
+	debug3_f("impl->name/nid: %s/%d", impl->name, nid);
 
 {	char *curve;
 
@@ -281,7 +367,10 @@ static int
 ssh_EC_GROUP_check_public(const EC_GROUP *group, const EC_POINT *public)
 {
 	EC_POINT *nq = NULL;
-	BIGNUM *order = NULL, *x = NULL, *y = NULL, *tmp = NULL;
+	BIGNUM *order = NULL;
+#ifdef SKIP_AFFINE_COORDINATES_CHECK
+	BIGNUM *x = NULL, *y = NULL, *tmp = NULL;
+#endif
 	int ret = SSH_ERR_KEY_INVALID_EC_VALUE;
 
 	/*
@@ -296,14 +385,21 @@ ssh_EC_GROUP_check_public(const EC_GROUP *group, const EC_POINT *public)
 	if (EC_POINT_is_at_infinity(group, public))
 		goto out;
 
+#ifdef SKIP_AFFINE_COORDINATES_CHECK
 	if ((x = BN_new()) == NULL ||
 	    (y = BN_new()) == NULL ||
-	    (order = BN_new()) == NULL ||
 	    (tmp = BN_new()) == NULL) {
 		ret = SSH_ERR_ALLOC_FAIL;
 		goto out;
 	}
+#endif
 
+	if ((order = BN_new()) == NULL) {
+		ret = SSH_ERR_ALLOC_FAIL;
+		goto out;
+	}
+
+#ifdef SKIP_AFFINE_COORDINATES_CHECK
 	/* log2(x) > log2(order)/2, log2(y) > log2(order)/2 */
 	if (EC_GROUP_get_order(group, order, NULL) != 1 ||
 	    EC_POINT_get_affine_coordinates(group, public,
@@ -314,8 +410,31 @@ ssh_EC_GROUP_check_public(const EC_GROUP *group, const EC_POINT *public)
 	if (BN_num_bits(x) <= BN_num_bits(order) / 2 ||
 	    BN_num_bits(y) <= BN_num_bits(order) / 2)
 		goto out;
+#endif /*def SKIP_AFFINE_COORDINATES_CHECK*/
 
 	/* nQ == infinity (n == order of subgroup) */
+#ifdef SKIP_AFFINE_COORDINATES_CHECK
+	/* As per NIST SP 800-56A, 5.6.2.3 check may be skipped for curves
+	 * with cofactor 1.
+	 */
+{	BIGNUM *cofactor;
+	if ((cofactor = BN_new()) == NULL) {
+		ret = SSH_ERR_ALLOC_FAIL;
+		goto out;
+	}
+	if (EC_GROUP_get_cofactor(group, cofactor, NULL) != 1) {
+		BN_clear_free(cofactor);
+		goto out;
+	}
+	if (BN_is_one(cofactor)) {
+		BN_clear_free(cofactor);
+		ret = 0;
+		goto out;
+	}
+	BN_clear_free(cofactor);
+}
+#endif
+
 	if ((nq = EC_POINT_new(group)) == NULL) {
 		ret = SSH_ERR_ALLOC_FAIL;
 		goto out;
@@ -327,6 +446,7 @@ ssh_EC_GROUP_check_public(const EC_GROUP *group, const EC_POINT *public)
 	if (EC_POINT_is_at_infinity(group, nq) != 1)
 		goto out;
 
+#ifdef SKIP_AFFINE_COORDINATES_CHECK
 	/* x < order - 1, y < order - 1 */
 	if (!BN_sub(tmp, order, BN_value_one())) {
 		ret = SSH_ERR_LIBCRYPTO_ERROR;
@@ -334,15 +454,18 @@ ssh_EC_GROUP_check_public(const EC_GROUP *group, const EC_POINT *public)
 	}
 	if (BN_cmp(x, tmp) >= 0 || BN_cmp(y, tmp) >= 0)
 		goto out;
+#endif /*def SKIP_AFFINE_COORDINATES_CHECK*/
 
 	/* other checks ? */
 
 	ret = 0;
  out:
+#ifdef SKIP_AFFINE_COORDINATES_CHECK
 	BN_clear_free(x);
 	BN_clear_free(y);
-	BN_clear_free(order);
 	BN_clear_free(tmp);
+#endif /*def SKIP_AFFINE_COORDINATES_CHECK*/
+	BN_clear_free(order);
 	EC_POINT_free(nq);
 	return ret;
 }
@@ -420,6 +543,94 @@ ssh_EVP_PKEY_prov_get_EC_GROUP(EVP_PKEY *pkey) {
 }
 #endif /*def ENABLE_KEY_PROVIDERS*/
 
+#ifdef ENABLE_KEY_PROVIDERS
+static int
+ssh_EVP_PKEY_get1_public_key_ecprov(EVP_PKEY *pk,
+    EC_GROUP **groupp, EC_POINT **pubp
+) {
+	EC_GROUP *group = NULL;
+	EC_POINT *pub_key = NULL;
+	int r;
+
+	if (groupp != NULL) *groupp = NULL;
+	if (pubp != NULL) *pubp = NULL;
+
+	group = ssh_EVP_PKEY_prov_get_EC_GROUP(pk);
+	if (group == NULL)
+		return SSH_ERR_ALLOC_FAIL;
+
+	pub_key = EC_POINT_new(group);
+	if (pub_key == NULL) {
+		r = SSH_ERR_ALLOC_FAIL;
+		goto err;
+	}
+
+{	unsigned char *data;
+	size_t len;
+
+	len = ssh_EVP_PKEY_get1_encoded_public_key(pk, &data);
+	if (len == 0) {
+		r = SSH_ERR_KEY_INVALID_EC_VALUE;
+		goto err;
+	}
+
+	/* Only handle uncompressed points */
+	if (*data != POINT_CONVERSION_UNCOMPRESSED) {
+		r = SSH_ERR_INVALID_FORMAT;
+		goto data_err;
+	}
+
+	if (EC_POINT_oct2point(group, pub_key, data, len, NULL) != 1) {
+		r = SSH_ERR_INVALID_FORMAT;
+		goto data_err;
+	}
+
+	if (groupp != NULL) {
+		*groupp = group;
+		group = NULL;
+	}
+	if (pubp != NULL) {
+		*pubp = pub_key;
+		pub_key = NULL;
+	}
+
+	r = 0;
+
+data_err:
+	OPENSSL_free(data);
+}
+
+err:
+	EC_GROUP_free(group);
+	EC_POINT_free(pub_key);
+	return r;
+}
+#endif /*def ENABLE_KEY_PROVIDERS*/
+
+void
+ssh_EVP_PKEY_EC_KEY_get1_public_key(EVP_PKEY *pkey,
+    EC_GROUP **groupp, EC_POINT **pubp
+) {
+#ifdef ENABLE_KEY_PROVIDERS
+	if (EVP_PKEY_get0_provider(pkey) != NULL) {
+		(void)ssh_EVP_PKEY_get1_public_key_ecprov(pkey, groupp, pubp);
+		return;
+	}
+#endif
+{	const EC_GROUP *group;
+	const EC_POINT *pub_key;
+	EC_KEY *ec = EVP_PKEY_get1_EC_KEY(pkey);
+	if (ec == NULL) return;
+
+	group = EC_KEY_get0_group(ec);
+	pub_key = EC_KEY_get0_public_key(ec);
+
+	if (groupp != NULL) *groupp = EC_GROUP_dup(group);
+	if (pubp != NULL) *pubp = EC_POINT_dup(pub_key, group);
+	EC_KEY_free(ec);
+}
+}
+
 #ifdef ENABLE_KEY_PROVIDERS
 static int
 ssh_EVP_PKEY_check_public_ecprov(EVP_PKEY *pkey, const EC_POINT *public) {
@@ -468,48 +679,17 @@ ssh_EVP_PKEY_check_public_ec(EVP_PKEY *pk, const EC_POINT *public) {
 static int
 ssh_EVP_PKEY_validate_public_ecprov(EVP_PKEY *pk) {
 	EC_GROUP *group = NULL;
-	EC_POINT *public = NULL;
+	EC_POINT *pub_key = NULL;
 	int r;
 
-	group = ssh_EVP_PKEY_prov_get_EC_GROUP(pk);
-	if (group == NULL)
-		return SSH_ERR_ALLOC_FAIL;
+	r = ssh_EVP_PKEY_get1_public_key_ecprov(pk, &group, &pub_key);
+	if (r != 0) goto err;
 
-	public = EC_POINT_new(group);
-	if (public == NULL) {
-		r = SSH_ERR_ALLOC_FAIL;
-		goto err;
-	}
-
-{	unsigned char *data;
-	size_t len;
-
-	len = ssh_EVP_PKEY_get1_encoded_public_key(pk, &data);
-	if (len == 0) {
-		r = SSH_ERR_KEY_INVALID_EC_VALUE;
-		goto err;
-	}
-
-	/* Only handle uncompressed points */
-	if (*data != POINT_CONVERSION_UNCOMPRESSED) {
-		r = SSH_ERR_INVALID_FORMAT;
-		goto data_err;
-	}
-
-	if (EC_POINT_oct2point(group, public, data, len, NULL) != 1) {
-		r = SSH_ERR_INVALID_FORMAT;
-		goto data_err;
-	}
-
-	r = ssh_EVP_PKEY_check_public_ecprov(pk, public);
-
-data_err:
-	OPENSSL_free(data);
-}
+	r = ssh_EVP_PKEY_check_public_ecprov(pk, pub_key);
 
 err:
 	EC_GROUP_free(group);
-	EC_POINT_free(public);
+	EC_POINT_free(pub_key);
 	return r;
 }
 #endif /*def ENABLE_KEY_PROVIDERS*/
@@ -519,25 +699,6 @@ ssh_EC_KEY_validate_public(const EC_KEY *ec) {
 	return ssh_EC_KEY_check_public(ec, EC_KEY_get0_public_key(ec));
 }
 
-int
-ssh_pkey_validate_public_ecdsa(EVP_PKEY *pk) {
-#ifdef ENABLE_KEY_PROVIDERS
-	if (EVP_PKEY_get0_provider(pk) != NULL)
-		return ssh_EVP_PKEY_validate_public_ecprov(pk);
-#endif
-{	EC_KEY *ec;
-	int r;
-
-	ec = EVP_PKEY_get1_EC_KEY(pk);
-	if (ec == NULL) return SSH_ERR_INVALID_ARGUMENT;
-
-	r = ssh_EC_KEY_validate_public(ec);
-
-	EC_KEY_free(ec);
-	return r;
-}
-}
-
 
 #ifdef ENABLE_KEY_PROVIDERS
 static int
@@ -715,43 +876,6 @@ ssh_pkey_validate_private_ecdsa(EVP_PKEY *pk, int skip_priv) {
 }
 
 
-extern int /* see sshkey-crypto.c */
-sshkey_from_pkey_ecdsa(EVP_PKEY *pk, struct sshkey **keyp);
-
-int
-sshkey_from_pkey_ecdsa(EVP_PKEY *pk, struct sshkey **keyp) {
-	int r;
-	struct sshkey* key;
-
-	r = ssh_EVP_PKEY_complete_pub_ecdsa(pk);
-	if (r != 0) return r;
-
-	key = sshkey_new(KEY_UNSPEC);
-	if (key == NULL)
-		return SSH_ERR_ALLOC_FAIL;
-
-	key->type = KEY_ECDSA;
-	key->pk = pk;
-
-	r = sshkey_set_nid(key->pk, &key->ecdsa_nid);
-	if (r != 0) goto err;
-
-	/* private part is not required */
-	r = ssh_pkey_validate_private_ecdsa(key->pk, 1);
-	if (r != 0) goto err;
-
-	/* success */
-	SSHKEY_DUMP(key);
-	*keyp = key;
-	return 0;
-
-err:
-	key->pk = NULL; /* transfer failed */
-	sshkey_free(key);
-	return r;
-}
-
-
 static int
 sshbuf_read_pub_ecdsa(struct sshbuf *buf, struct sshkey *key) {
 	int r;
@@ -899,6 +1023,69 @@ sshbuf_write_priv_ecdsa(struct sshbuf *buf, const struct sshkey *key) {
 #endif /* def WITH_OPENSSL_4_0_API */
 
 
+/* pkey implementation */
+
+static int
+ssh_ecdsa_sshkey_from_pkey(EVP_PKEY *pk, struct sshkey **keyp) {
+	int r;
+	struct sshkey* key;
+
+	r = ssh_EVP_PKEY_complete_pub_ecdsa(pk);
+	if (r != 0) return r;
+
+	key = sshkey_new(KEY_UNSPEC);
+	if (key == NULL)
+		return SSH_ERR_ALLOC_FAIL;
+
+	key->type = KEY_ECDSA;
+	key->pk = pk;
+
+	r = sshkey_set_nid(key->pk, &key->ecdsa_nid);
+	if (r != 0) goto err;
+
+	/* private part is not required */
+	r = ssh_pkey_validate_private_ecdsa(key->pk, 1);
+	if (r != 0) goto err;
+
+	/* success */
+	SSHKEY_DUMP(key);
+	*keyp = key;
+	return 0;
+
+err:
+	key->pk = NULL; /* transfer failed */
+	sshkey_free(key);
+	return r;
+}
+
+static int
+ssh_ecdsa_validate_public(EVP_PKEY *pk) {
+#ifdef ENABLE_KEY_PROVIDERS
+	if (EVP_PKEY_get0_provider(pk) != NULL)
+		return ssh_EVP_PKEY_validate_public_ecprov(pk);
+#endif
+{	EC_KEY *ec;
+	int r;
+
+	ec = EVP_PKEY_get1_EC_KEY(pk);
+	if (ec == NULL) return SSH_ERR_INVALID_ARGUMENT;
+
+	r = ssh_EC_KEY_validate_public(ec);
+
+	EC_KEY_free(ec);
+	return r;
+}
+}
+
+static const struct ssh_pkey_impl_funcs
+ssh_pkey_ec_funcs = {
+	ssh_ecdsa_sshkey_from_pkey,
+	ssh_ecdsa_validate_public
+};
+
+#define SSH_PKEY_EC_FUNCS	&ssh_pkey_ec_funcs
+
+
 /* key implementation */
 
 static u_int
@@ -920,25 +1107,24 @@ ssh_ecdsa_equal(const struct sshkey *a, const struct sshkey *b)
 }
 
 static int
-ssh_ecdsa_serialize_public(const struct sshkey *key, struct sshbuf *buf,
-    enum sshkey_serialize_rep opts)
+ssh_ecdsa_serialize_public(const struct sshkey_impl *impl,
+    const struct sshkey *key, struct sshbuf *buf)
 {
 	int r;
 
-	UNUSED(opts);
+	UNUSED(impl);
 	if ((r = sshbuf_write_ec_curve(buf, key)) != 0)
 		return r;
 	return sshbuf_write_pub_ecdsa(buf, key);
 }
 
 static int
-ssh_ecdsa_serialize_private(const struct sshkey *key, struct sshbuf *buf,
-    enum sshkey_serialize_rep opts)
+ssh_ecdsa_serialize_private(const struct sshkey_impl *impl,
+    const struct sshkey *key, struct sshbuf *buf)
 {
 	int r;
 
-	UNUSED(opts);
-	if (!sshkey_is_cert(key)) {
+	if (!impl->cert) {
 		if ((r = sshbuf_write_ec_curve(buf, key)) != 0 ||
 		    (r = sshbuf_write_pub_ecdsa(buf, key)) != 0)
 			return r;
@@ -1028,8 +1214,8 @@ ssh_ecdsa_generate(struct sshkey *key, int bits) {
 	EVP_PKEY *pk;
 	int r = 0, nid;
 
-	nid = sshkey_ecdsa_bits_to_nid(bits);
-	if (nid == -1) return SSH_ERR_KEY_LENGTH;
+	nid = sshkey_curve_bits_to_nid(bits);
+	if (nid == NID_undef) return SSH_ERR_KEY_LENGTH;
 
 #ifdef USE_EVP_PKEY_KEYGEN
 	r = ssh_pkey_keygen_ec(nid, &pk);
@@ -1044,6 +1230,35 @@ ssh_ecdsa_generate(struct sshkey *key, int bits) {
 	return r;
 }
 
+static u_int
+ssh_ecdsa_default_bits(const char *name) {
+	const struct sshkey_impl *impl;
+	int nid;
+
+	impl = (name != NULL)
+		? sshkey_impl_from_name(name)
+		: NULL;
+	/* NOTE: sinc with keygen manual page */
+	nid = (impl != NULL)
+		? impl->id.subtype
+		: NID_X9_62_prime256v1;
+	return sshkey_curve_nid_to_bits(nid);
+}
+
+static void
+ssh_ecdsa_valid_bits(int bits) {
+	if (sshkey_curve_bits_to_nid(bits) != NID_undef)
+		return;
+
+#ifdef OPENSSL_HAS_NISTP521
+	fatal("Invalid ECDSA key length: valid lengths are "
+	    "256, 384 or 521 bits");
+#else
+	fatal("Invalid ECDSA key length: valid lengths are "
+	    "256 or 384 bits");
+#endif
+}
+
 static void
 ssh_ecdsa_move_public(struct sshkey *from, struct sshkey *to) {
 	sshkey_move_pk(from, to);
@@ -1083,24 +1298,24 @@ err:
 }
 
 static int
-ssh_ecdsa_deserialize_public(const char *pkalg, struct sshbuf *buf,
-    struct sshkey *key)
+ssh_ecdsa_deserialize_public(const struct sshkey_impl *impl,
+    struct sshbuf *buf, struct sshkey *key)
 {
 	int r;
 
-	if ((r = sshbuf_read_ec_curve(buf, pkalg, key)) != 0)
+	if ((r = sshbuf_read_ec_curve(buf, impl, key)) != 0)
 		return r;
 	return sshbuf_read_pub_ecdsa(buf, key);
 }
 
 static int
-ssh_ecdsa_deserialize_private(const char *pkalg, struct sshbuf *buf,
-    struct sshkey *key)
+ssh_ecdsa_deserialize_private(const struct sshkey_impl *impl,
+    struct sshbuf *buf, struct sshkey *key)
 {
 	int r;
 
-	if (!sshkey_is_cert(key)) {
-		if ((r = sshbuf_read_ec_curve(buf, pkalg, key)) != 0 ||
+	if (!impl->cert) {
+		if ((r = sshbuf_read_ec_curve(buf, impl, key)) != 0 ||
 		    (r = sshbuf_read_pub_ecdsa(buf, key)) != 0)
 			return r;
 	}
@@ -1127,7 +1342,7 @@ ssh_ecdsa_sign(const ssh_sign_ctx *ctx, u_char **sigp, size_t *lenp,
 	dgst = ssh_ecdsa_dgst(key);
 	if (dgst == NULL) return SSH_ERR_INTERNAL_ERROR;
 
-	ret = ssh_pkey_validate_public_ecdsa(key->pk);
+	ret = ssh_ecdsa_validate_public(key->pk);
 	if (ret != 0) return ret;
 
 	if (ssh_pkey_sign(dgst, key->pk, sigblob, &siglen, data, datalen) <= 0) {
@@ -1163,7 +1378,7 @@ ssh_ecdsa_verify(const ssh_verify_ctx *ctx,
 	dgst = ssh_ecdsa_dgst(key);
 	if (dgst == NULL) return SSH_ERR_INTERNAL_ERROR;
 
-	ret = ssh_pkey_validate_public_ecdsa(key->pk);
+	ret = ssh_ecdsa_validate_public(key->pk);
 	if (ret != 0) return ret;
 
 	/* fetch signature */
@@ -1202,6 +1417,8 @@ static const struct sshkey_impl_funcs sshkey_ecdsa_funcs = {
 	/* .serialize_private = */	ssh_ecdsa_serialize_private,
 	/* .deserialize_private = */	ssh_ecdsa_deserialize_private,
 	/* .generate = */	ssh_ecdsa_generate,
+	/* .default_bits = */	ssh_ecdsa_default_bits,
+	/* .valid_bits = */	ssh_ecdsa_valid_bits,
 	/* .move_public = */	ssh_ecdsa_move_public,
 	/* .copy_public = */	ssh_ecdsa_copy_public,
 	/* .sign = */		ssh_ecdsa_sign,
@@ -1220,7 +1437,7 @@ static int sshkey_ecdsa_nistp521_enabled(void) {
 
 #else
 
-#include "sshkey.h"
+#include "sshxkey.h"
 
 #undef NID_X9_62_prime256v1
 #define NID_X9_62_prime256v1	NID_undef
@@ -1232,10 +1449,11 @@ static int sshkey_ecdsa_nistp521_enabled(void) {
 static int sshkey_ecdsa_enabled(void) { return 0; }
 static int sshkey_ecdsa_nistp521_enabled(void) { return 0; }
 #define SSHKEY_ECDSA_FUNCS	NULL
+#define SSH_PKEY_EC_FUNCS	NULL
 
 #endif /* WITH_OPENSSL && OPENSSL_HAS_ECC */
 
-static char *pkey_ec_typename[] = { "EC", NULL };
+static const char *pkey_ec_typename[] = { "EC", NULL };
 
 const struct sshkey_impl sshkey_ecdsa_nistp256_impl = {
 	/* .id = */		{ KEY_ECDSA, NID_X9_62_prime256v1,
@@ -1314,3 +1532,9 @@ const struct sshkey_impl sshkey_ecdsa_nistp521_cert_impl = {
 	/* .enabled = */	sshkey_ecdsa_nistp521_enabled,
 	/* .funcs = */		SSHKEY_ECDSA_FUNCS
 };
+
+const struct ssh_pkey_impl ssh_pkey_ec_impl = {
+	pkey_ec_typename,
+	EVP_PKEY_EC,
+	SSH_PKEY_EC_FUNCS
+};
