@@ -1,4 +1,4 @@
-/* $OpenBSD: ssh-keygen.c,v 1.486 2025/10/09 23:26:47 djm Exp $ */
+/* $OpenBSD: ssh-keygen.c,v 1.487 2025/11/13 10:35:14 dtucker Exp $ */
 /*
  * Author: Tatu Ylonen <ylo@cs.hut.fi>
  * Copyright (c) 1994 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
@@ -99,19 +99,32 @@ default_key_type_name(void) {
 }
 
 /*
- * Default number of bits in the RSA, DSA and ECDSA keys.  These value can be
- * overridden on the command line.
+ * Default number of bits in the keys.
+ * These value can be overridden on the command line.
  *
- * These values, with the exception of DSA, provide security equivalent to at
- * least 128 bits of security according to NIST Special Publication 800-57:
- * Recommendation for Key Management Part 1 rev 4 section 5.6.1.
+ * With the exception of DSA:
+ * These values provide security equivalent to at least 128 bits of security
+ * according to NIST Special Publication 800-57: Recommendation for Key
+ * Management Part 1 rev 4 section 5.6.1.
+ *
+ * DSA:
  * For DSA it (and FIPS-186-4 section 4.2) specifies that the only size for
  * which a 160bit hash is acceptable is 1kbit, and since ssh-dss specifies only
  * SHA1 we limit the DSA key size 1k bits.
  */
-#define DEFAULT_BITS		3072
-#define DEFAULT_BITS_DSA	1024
-#define DEFAULT_BITS_ECDSA	256
+
+/* NOTE "generate" is declared with integer parameter for bits
+ * All these functions raise fatal exception if bits are not acceptable.
+ */
+static void
+type_bits_valid(const struct sshkey_impl *impl, const char *name, u_int *bitsp)
+{
+	if (*bitsp == 0 && impl->funcs->default_bits != NULL)
+		*bitsp = impl->funcs->default_bits(name);
+
+	if (impl->funcs->valid_bits != NULL)
+		impl->funcs->valid_bits(*bitsp);
+}
 
 static int quiet = 0;
 
@@ -222,64 +235,6 @@ fingerprint_one_key(const struct sshkey *public, const char *comment) {
 	fingerprint_format(public, comment, print_fingerprint_one_key);
 }
 
-
-static void
-type_bits_valid(int type, const char *name, u_int32_t *bitsp)
-{
-	if (type == KEY_UNSPEC)
-		fatal("unknown key type %s", key_type_name);
-	if (*bitsp == 0) {
-#ifdef WITH_OPENSSL
-		int nid;
-
-		switch(type) {
-	#ifdef WITH_DSA
-		case KEY_DSA:
-			*bitsp = DEFAULT_BITS_DSA;
-			break;
-	#endif
-		case KEY_ECDSA:
-			if (name != NULL &&
-			    (nid = sshkey_ecdsa_nid_from_name(name)) > 0)
-				*bitsp = sshkey_curve_nid_to_bits(nid);
-			if (*bitsp == 0)
-				*bitsp = DEFAULT_BITS_ECDSA;
-			break;
-		case KEY_RSA:
-			*bitsp = DEFAULT_BITS;
-			break;
-		}
-#endif
-	}
-#ifdef WITH_OPENSSL
-	switch (type) {
-#ifdef WITH_DSA
-	case KEY_DSA:
-		if (*bitsp != SSH_DSA_BITS)
-			fatal("Invalid DSA key length: must be %d bits", SSH_DSA_BITS);
-		break;
-#endif
-	case KEY_RSA:
-		if (*bitsp < SSH_RSA_MINIMUM_MODULUS_SIZE)
-			fatal("Invalid RSA key length: minimum is %d bits",
-			    SSH_RSA_MINIMUM_MODULUS_SIZE);
-		else if (*bitsp > OPENSSL_RSA_MAX_MODULUS_BITS)
-			fatal("Invalid RSA key length: maximum is %d bits",
-			    OPENSSL_RSA_MAX_MODULUS_BITS);
-		break;
-	case KEY_ECDSA:
-		if (sshkey_ecdsa_bits_to_nid(*bitsp) == -1)
-#ifdef OPENSSL_HAS_NISTP521
-			fatal("Invalid ECDSA key length: valid lengths are "
-			    "256, 384 or 521 bits");
-#else
-			fatal("Invalid ECDSA key length: valid lengths are "
-			    "256 or 384 bits");
-#endif
-	}
-#endif
-}
-
 /*
  * Checks whether a file exists and, if so, asks the user whether they wish
  * to overwrite it.
@@ -305,42 +260,46 @@ confirm_overwrite(const char *filename)
 	return yesno[0] == 'y' || yesno[0] == 'Y';
 }
 
+static const char*
+get_key_type_path(const char *name) {
+	const struct sshkey_impl *impl;
+	int k;
+	struct {
+		char *name;
+		char *path;
+	} key_types[] = {
+		{ "ED25519", _PATH_SSH_CLIENT_ID_ED25519 },
+		{ "RSA", _PATH_SSH_CLIENT_ID_RSA },
+		{ "ECDSA", _PATH_SSH_CLIENT_ID_ECDSA },
+		{ "DSA", _PATH_SSH_CLIENT_ID_DSA },
+		{ NULL, NULL }
+	};
+
+	impl = sshkey_impl_from_shortname(name);
+	if (impl == NULL)
+		fatal("unsupported key type '%s'", name);
+
+	for (k = 0; key_types[k].name != NULL; k++) {
+		if (strcmp(key_types[k].name, impl->shortname) == 0)
+			return key_types[k].path;
+	}
+
+	return NULL;
+}
+
 static void
 ask_filename(const struct passwd *pw, const char *prompt)
 {
 	char buf[1024];
-	char *name = NULL;
+	const char *name;
 
 	if (key_type_name == NULL)
 		key_type_name = default_key_type_name();
 
-	/* NOTE: keep block to minimise code differences */
-	{
-		switch (sshkey_type_from_shortname(key_type_name)) {
-#ifdef WITH_DSA
-		case KEY_DSA_CERT:
-		case KEY_DSA:
-			name = _PATH_SSH_CLIENT_ID_DSA;
-			break;
-#endif
-#ifdef OPENSSL_HAS_ECC
-		case KEY_ECDSA_CERT:
-		case KEY_ECDSA:
-			name = _PATH_SSH_CLIENT_ID_ECDSA;
-			break;
-#endif
-		case KEY_RSA_CERT:
-		case KEY_RSA:
-			name = _PATH_SSH_CLIENT_ID_RSA;
-			break;
-		case KEY_ED25519:
-		case KEY_ED25519_CERT:
-			name = _PATH_SSH_CLIENT_ID_ED25519;
-			break;
-		default:
-			fatal("bad key type");
-		}
-	}
+	name = get_key_type_path(key_type_name);
+	if (name == NULL)
+		fatal_r(SSH_ERR_INTERNAL_ERROR, "unsupported key type %s", key_type_name);
+
 	snprintf(identity_file, sizeof(identity_file),
 	    "%s/%s", pw->pw_dir, name);
 	printf("%s (%s): ", prompt, identity_file);
@@ -454,7 +413,7 @@ do_convert_to_ssh2(const struct passwd *pw, struct sshkey *k)
 }
 
 
-/* defined in sshkey-crypto.c but used only localy here */
+/* defined in sshkey-crypto.c but used only locally here */
 extern int
 sshkey_public_to_fp(struct sshkey *key, FILE *fp, int format);
 
@@ -488,7 +447,7 @@ do_convert_to(const struct passwd *pw)
 }
 
 
-/* defined in sshkey-crypto.c but used only localy here */
+/* defined in sshkey-crypto.c but used only locally here */
 extern int
 sshbuf_read_custom_rsa(struct sshbuf *buf, struct sshkey *key);
 extern int
@@ -666,7 +625,7 @@ do_convert_from_ssh2(struct sshkey **k, int *private)
 	fclose(fp);
 }
 
-/* defined in sshkey-crypto.c but used only localy here */
+/* defined in sshkey-crypto.c but used only locally here */
 extern int
 sshkey_public_from_fp(FILE *fp, int format, struct sshkey **key);
 
@@ -983,26 +942,22 @@ do_gen_all_hostkeys(const struct passwd *pw)
 {
 	struct {
 		char *key_type;
-		char *key_type_display;
 		char *path;
 	} key_types[] = {
-#ifdef WITH_OPENSSL
-		{ "rsa", "RSA" ,_PATH_HOST_RSA_KEY_FILE },
-#ifdef OPENSSL_HAS_ECC
-		{ "ecdsa", "ECDSA",_PATH_HOST_ECDSA_KEY_FILE },
-#endif /* OPENSSL_HAS_ECC */
-#endif /* WITH_OPENSSL */
-		{ "ed25519", "ED25519",_PATH_HOST_ED25519_KEY_FILE },
-		{ NULL, NULL, NULL }
+		{ "rsa", _PATH_HOST_RSA_KEY_FILE },
+		{ "ecdsa", _PATH_HOST_ECDSA_KEY_FILE },
+		{ "ed25519", _PATH_HOST_ED25519_KEY_FILE },
+		{ NULL, NULL }
 	};
 
 	int first = 0;
 	struct stat st;
 	struct sshkey *private, *public;
 	char comment[1024], *prv_tmp, *pub_tmp, *prv_file, *pub_file;
-	int i, type, fd, r;
+	int i, fd, r;
 
 	for (i = 0; key_types[i].key_type; i++) {
+		const struct sshkey_impl *impl;
 		const char *key_path = key_types[i].path;
 	#if defined(__ANDROID__)
 		char r_key_path[PATH_MAX];
@@ -1038,18 +993,22 @@ do_gen_all_hostkeys(const struct passwd *pw)
 			first = 1;
 			printf("%s: generating new host keys: ", __progname);
 		}
-		printf("%s ", key_types[i].key_type_display);
+		impl = sshkey_impl_from_shortname(key_types[i].key_type);
+		if (impl == NULL) continue; /* not enabled */
+
+		printf("%s ", impl->shortname);
 		fflush(stdout);
-		type = sshkey_type_from_shortname(key_types[i].key_type);
+
 		if ((fd = mkstemp(prv_tmp)) == -1) {
 			error("Could not save your private key in %s: %s",
 			    prv_tmp, strerror(errno));
 			goto failnext;
 		}
 		(void)close(fd); /* just using mkstemp() to reserve a name */
-	{	u_int32_t bits = 0;
-		type_bits_valid(type, NULL, &bits);
-		if ((r = sshkey_generate(type, bits, &private)) != 0) {
+
+	{	u_int bits = 0;
+		type_bits_valid(impl, NULL, &bits);
+		if ((r = sshkey_generate(impl->id.type, bits, &private)) != 0) {
 			error_r(r, "sshkey_generate failed");
 			goto failnext;
 		}
@@ -1711,10 +1670,12 @@ do_ca_sign(const struct passwd *pw, const char *ca_key_path, int prefer_agent,
 	}
 	free(tmp);
 
-	if (key_type_name != NULL &&
-	    sshkey_type_from_shortname(key_type_name) != ca->type)  {
-		fatal("CA key type %s doesn't match specified %s",
-		    sshkey_ssh_name(ca), key_type_name);
+	if (key_type_name != NULL) {
+		const struct sshkey_impl *impl;
+		impl = sshkey_impl_from_shortname(key_type_name);
+		if (impl->id.type != ca->type)
+			fatal("CA key type %s doesn't match specified %s",
+			    sshkey_ssh_name(ca), key_type_name);
 	}
 
 	finalise_cert_exts();
@@ -2430,8 +2391,15 @@ do_moduli_gen(const char *out_file, char **opts, size_t nopts)
 	}
 	setvbuf(out, NULL, _IOLBF, 0);
 
-	if (moduli_bits == 0)
-		moduli_bits = DEFAULT_BITS;
+	if (moduli_bits == 0) {
+		const struct sshkey_impl *impl;
+		impl = sshkey_impl_from_shortname("rsa");
+		if (impl == NULL)
+			fatal_fr(SSH_ERR_INTERNAL_ERROR, "rsa?");
+		if (impl->funcs->default_bits == NULL)
+			fatal_fr(SSH_ERR_INTERNAL_ERROR, "rsa-bits?");
+		moduli_bits = impl->funcs->default_bits(NULL);
+	}
 	if (gen_candidates(out, moduli_bits, start) != 0)
 		fatal("modulus candidate generation failed");
 #else /*ndef ENABLE_KEX_DH*/
@@ -2604,7 +2572,7 @@ main(int argc, char **argv)
 	struct sshkey *private = NULL, *public = NULL;
 	const struct passwd *pw;
 	int exit_status = 0;
-	int r, opt, type;
+	int r, opt;
 	int change_passphrase = 0, change_comment = 0, show_cert = 0;
 	int find_host = 0, delete_host = 0, hash_hosts = 0;
 	int gen_all_hostkeys = 0, gen_krl = 0, update_krl = 0, check_krl = 0;
@@ -2614,7 +2582,7 @@ main(int argc, char **argv)
 	unsigned long long cert_serial = 0;
 	char *identity_comment = NULL, *ca_key_path = NULL, **opts = NULL;
 	size_t nopts = 0;
-	u_int32_t bits = 0;
+	u_int bits = 0;
 	const char *errstr;
 	int log_level = SYSLOG_LEVEL_INFO;
 
@@ -2663,7 +2631,7 @@ main(int argc, char **argv)
 			gen_all_hostkeys = 1;
 			break;
 		case 'b':
-			bits = (u_int32_t)strtonum(optarg, 1, UINT32_MAX,
+			bits = strtonum(optarg, 1, UINT32_MAX,
 			    &errstr);
 			if (errstr)
 				fatal("Bits has bad value %s (%s)",
@@ -3015,14 +2983,19 @@ main(int argc, char **argv)
 	if (key_type_name == NULL)
 		key_type_name = default_key_type_name();
 
-	type = sshkey_type_from_shortname(key_type_name);
-	type_bits_valid(type, key_type_name, &bits);
+{	const struct sshkey_impl *impl;
+	impl = sshkey_impl_from_shortname(key_type_name);
+	if (impl == NULL)
+		fatal("unknown key type %s", key_type_name);
+
+	type_bits_valid(impl, key_type_name, &bits);
 
 	if (!quiet)
 		printf("Generating public/private %s key pair.\n",
 		    key_type_name);
-	if ((r = sshkey_generate(type, bits, &private)) != 0)
+	if ((r = sshkey_generate(impl->id.type, bits, &private)) != 0)
 		fatal_r(r, "sshkey_generate failed");
+}
 	if ((r = sshkey_from_private(private, &public)) != 0)
 		fatal_r(r, "sshkey_from_private");
 
