@@ -285,17 +285,13 @@ keyprint_one(const char *host, con *c, struct sshkey *key)
 		fatal("host_hash failed");
 	known_host = hash_hosts ? hashed : hostport;
 
-	if (fprintf(stdout, "%s ", known_host) < 0)
-		goto out;
-	/* sshkey_write prints X.509 certificate in blob format :-( */
-	if (sshkey_is_x509(key)) {
-		if (!Xkey_write_subject(c->c_keyname, key, stdout))
-			goto out;
-	} else {
-		if (sshkey_write(key, stdout) != 0)
-			goto out;
+{	struct sshbuf *b = sshbuf_new();
+	if (b == NULL) goto out;
+	if (Xkey_printb(c->c_keyname, key, b) == 0) {
+		printf("%s %s\n", known_host, sshbuf_ptr(b));
 	}
-	(void)fputs("\n", stdout);
+	sshbuf_free(b);
+}
 
 out:
 	free(hashed);
