@@ -97,18 +97,17 @@ ssh_ocsp_validate(X509 *cert, X509_STORE *x509store) {
 }
 
 #ifdef LDAP_ENABLED
-#ifdef USE_LDAP_STORE	 /* OpenSSL >= 1.1.1 */
+# ifdef USE_LDAP_STORE	 /* OpenSSL >= 1.1.1 */
+/* TODO: LDAP provider */
 void ENGINE_load_ldap(void);
 void ENGINE_load_ldap(void) {}
-#endif /*def USE_LDAP_STORE*/
-
-#ifndef USE_X509_LOOKUP_STORE /* OpenSSL < 3.0 */
+# else
 X509_LOOKUP_METHOD* X509_LOOKUP_ldap(void);
 X509_LOOKUP_METHOD*
 X509_LOOKUP_ldap(void) {
 	return NULL;
 }
-#endif /*ndef USE_X509_LOOKUP_STORE*/
+# endif /*def USE_LDAP_STORE*/
 #endif /*def LDAP_ENABLED*/
 
 #ifdef USE_X509_LOOKUP_MYSTORE /* OpenSSL 1.1.1* */
