@@ -49,6 +49,11 @@
 #include "sshxkey.h"
 #include "log.h"
 
+#define SSH_RSA_MINIMUM_MODULUS_SIZE	1024
+#ifndef OPENSSL_RSA_MAX_MODULUS_BITS
+# define OPENSSL_RSA_MAX_MODULUS_BITS	16384
+#endif
+
 
 struct ssh_rsa_alg_st {
 	const char *name;
@@ -210,6 +215,70 @@ RSA_set0_factors(RSA *rsa, BIGNUM *p, BIGNUM *q) {
 }
 #endif /* ndef HAVE_RSA_GET0_KEY */
 
+
+void
+ssh_EVP_PKEY_RSA_get1_key(EVP_PKEY *pkey,
+    BIGNUM **np, BIGNUM **ep, BIGNUM **dp
+) {
+#ifdef ENABLE_KEY_PROVIDERS
+	if (EVP_PKEY_get0_provider(pkey) != NULL) {
+		if (np != NULL)
+			(void)EVP_PKEY_get_bn_param(pkey,
+			    OSSL_PKEY_PARAM_RSA_N, np);
+		if (ep != NULL)
+			(void)EVP_PKEY_get_bn_param(pkey,
+			    OSSL_PKEY_PARAM_RSA_E, ep);
+		if (dp != NULL)
+			(void)EVP_PKEY_get_bn_param(pkey,
+			    OSSL_PKEY_PARAM_RSA_D, dp);
+		return;
+	}
+#endif
+{	const BIGNUM *n = NULL, *e = NULL, *d = NULL;
+	RSA *rsa = EVP_PKEY_get1_RSA(pkey);
+
+	if (rsa == NULL) return;
+
+	RSA_get0_key(rsa, &n, &e, &d);
+
+	if (np != NULL) *np = BN_dup(n);
+	if (ep != NULL) *ep = BN_dup(e);
+	if (dp != NULL) *dp = BN_dup(d);
+
+	RSA_free(rsa);
+}
+}
+
+void
+ssh_EVP_PKEY_RSA_get1_factors(EVP_PKEY *pkey,
+    BIGNUM **pp, BIGNUM **qp
+) {
+#ifdef ENABLE_KEY_PROVIDERS
+	if (EVP_PKEY_get0_provider(pkey) != NULL) {
+		if (pp != NULL)
+			(void)EVP_PKEY_get_bn_param(pkey,
+			    OSSL_PKEY_PARAM_RSA_FACTOR1, pp);
+		if (qp != NULL)
+			(void)EVP_PKEY_get_bn_param(pkey,
+			    OSSL_PKEY_PARAM_RSA_FACTOR2, qp);
+		return;
+	}
+#endif
+{	const BIGNUM *p = NULL, *q = NULL;
+	RSA *rsa = EVP_PKEY_get1_RSA(pkey);
+
+	if (rsa == NULL) return;
+
+	RSA_get0_factors(rsa, &p, &q);
+
+	if (pp != NULL) *pp = BN_dup(p);
+	if (qp != NULL) *qp = BN_dup(q);
+
+	RSA_free(rsa);
+}
+}
+
+
 #ifndef HAVE_EVP_PKEY_CMP	/* OpenSSL < 0.9.8 */
 extern int /* see sshkey-crypto.c */
 ssh_EVP_PKEY_cmp_rsa(const EVP_PKEY *ka, const EVP_PKEY *kb);
@@ -363,31 +432,6 @@ err:
 }
 
 
-extern int /* see sshkey-crypto.c */
-sshkey_from_pkey_rsa(EVP_PKEY *pk, struct sshkey **keyp);
-
-int
-sshkey_from_pkey_rsa(EVP_PKEY *pk, struct sshkey **keyp) {
-	int r;
-	struct sshkey* key;
-
-	r = ssh_EVP_PKEY_complete_pub_rsa(pk);
-	if (r != 0) return r;
-
-	key = sshkey_new(KEY_UNSPEC);
-	if (key == NULL)
-		return SSH_ERR_ALLOC_FAIL;
-
-	key->type = KEY_RSA;
-	key->pk = pk;
-
-	/* success */
-	SSHKEY_DUMP(key);
-	*keyp = key;
-	return 0;
-}
-
-
 int ssh_rsa_public_to_fp_traditional(struct sshkey *key, FILE *fp);
 
 int
@@ -720,7 +764,7 @@ sshbuf_write_priv_rsa(struct sshbuf *buf, const struct sshkey *key) {
 }
 
 
-extern int /* method used localy only in ssh-keygen.c */
+extern int /* method used locally only in ssh-keygen.c */
 sshbuf_read_custom_rsa(struct sshbuf *buf, struct sshkey *key);
 
 int
@@ -812,6 +856,38 @@ err:
 #endif /* def WITH_OPENSSL_4_0_API */
 
 
+/* pkey implementation */
+
+static int
+ssh_rsa_sshkey_from_pkey(EVP_PKEY *pk, struct sshkey **keyp) {
+	int r;
+	struct sshkey* key;
+
+	r = ssh_EVP_PKEY_complete_pub_rsa(pk);
+	if (r != 0) return r;
+
+	key = sshkey_new(KEY_UNSPEC);
+	if (key == NULL)
+		return SSH_ERR_ALLOC_FAIL;
+
+	key->type = KEY_RSA;
+	key->pk = pk;
+
+	/* success */
+	SSHKEY_DUMP(key);
+	*keyp = key;
+	return 0;
+}
+
+static const struct ssh_pkey_impl_funcs
+ssh_pkey_rsa_funcs = {
+	ssh_rsa_sshkey_from_pkey,
+	ssh_pkey_validate_public_rsa
+};
+
+#define SSH_PKEY_RSA_FUNCS	&ssh_pkey_rsa_funcs
+
+
 /* key implementation */
 
 static u_int
@@ -833,21 +909,20 @@ ssh_rsa_equal(const struct sshkey *a, const struct sshkey *b)
 }
 
 static int
-ssh_rsa_serialize_public(const struct sshkey *key, struct sshbuf *buf,
-    enum sshkey_serialize_rep opts)
+ssh_rsa_serialize_public(const struct sshkey_impl *impl,
+    const struct sshkey *key, struct sshbuf *buf)
 {
-	UNUSED(opts);
+	UNUSED(impl);
 	return sshbuf_write_pub_rsa(buf, key);
 }
 
 static int
-ssh_rsa_serialize_private(const struct sshkey *key, struct sshbuf *buf,
-    enum sshkey_serialize_rep opts)
+ssh_rsa_serialize_private(const struct sshkey_impl *impl,
+    const struct sshkey *key, struct sshbuf *buf)
 {
 	int r;
 
-	UNUSED(opts);
-	if (!sshkey_is_cert(key)) {
+	if (!impl->cert) {
 		if ((r = sshbuf_write_pub_rsa_priv(buf, key)) != 0)
 			return r;
 	}
@@ -978,6 +1053,23 @@ ssh_rsa_generate(struct sshkey *key, int bits) {
 	return r;
 }
 
+static u_int
+ssh_rsa_default_bits(const char* name) {
+	UNUSED(name);
+	/* NOTE: sinc with keygen manual page */
+	return 3072;
+}
+
+static void
+ssh_rsa_valid_bits(int bits) {
+	if (sshrsa_verify_length(bits) != 0)
+		fatal("Invalid RSA key length: minimum is %d bits",
+		    required_rsa_size);
+	else if (bits > OPENSSL_RSA_MAX_MODULUS_BITS)
+		fatal("Invalid RSA key length: maximum is %d bits",
+		    OPENSSL_RSA_MAX_MODULUS_BITS);
+}
+
 static void
 ssh_rsa_move_public(struct sshkey *from, struct sshkey *to) {
 	sshkey_move_pk(from, to);
@@ -1018,21 +1110,20 @@ err:
 }
 
 static int
-ssh_rsa_deserialize_public(const char *pkalg, struct sshbuf *buf,
-    struct sshkey *key)
+ssh_rsa_deserialize_public(const struct sshkey_impl *impl,
+    struct sshbuf *buf, struct sshkey *key)
 {
-	UNUSED(pkalg);
+	UNUSED(impl);
 	return sshbuf_read_pub_rsa(buf, key);
 }
 
 static int
-ssh_rsa_deserialize_private(const char *pkalg, struct sshbuf *buf,
-    struct sshkey *key)
+ssh_rsa_deserialize_private(const struct sshkey_impl *impl,
+    struct sshbuf *buf, struct sshkey *key)
 {
 	int r;
 
-	UNUSED(pkalg);
-	if (!sshkey_is_cert(key)) {
+	if (!impl->cert) {
 		if ((r = sshbuf_read_pub_rsa_priv(buf, key)) != 0)
 			return r;
 	}
@@ -1216,6 +1307,8 @@ static const struct sshkey_impl_funcs sshkey_rsa_funcs = {
 	/* .serialize_private = */	ssh_rsa_serialize_private,
 	/* .deserialize_private = */	ssh_rsa_deserialize_private,
 	/* .generate = */	ssh_rsa_generate,
+	/* .default_bits = */	ssh_rsa_default_bits,
+	/* .valid_bits = */	ssh_rsa_valid_bits,
 	/* .move_public = */	ssh_rsa_move_public,
 	/* .copy_public = */	ssh_rsa_copy_public,
 	/* .sign = */		ssh_rsa_sign,
@@ -1237,13 +1330,14 @@ static int sshkey_rsa_sha256_enabled(void) {
 static int sshkey_rsa_enabled(void) { return 0; }
 static int sshkey_rsa_sha256_enabled(void) { return 0; }
 #define SSHKEY_RSA_FUNCS	NULL
+#define SSH_PKEY_RSA_FUNCS	NULL
 
 #endif /* WITH_OPENSSL */
 
-static char *pkey_rsa_typename[] = { "RSA", NULL };
+static const char *pkey_rsa_typename[] = { "RSA", NULL };
 
 const struct sshkey_impl sshkey_rsa_impl = {
-	/* .id = */		{ KEY_RSA, 0,
+	/* .id = */		{ KEY_RSA, -1,
 				  EVP_PKEY_RSA, pkey_rsa_typename },
 	/* .name = */		"ssh-rsa",
 	/* .shortname = */	"RSA",
@@ -1256,7 +1350,7 @@ const struct sshkey_impl sshkey_rsa_impl = {
 };
 
 const struct sshkey_impl sshkey_rsa_cert_impl = {
-	/* .id = */		{ KEY_RSA_CERT, 0,
+	/* .id = */		{ KEY_RSA_CERT, -1,
 				  EVP_PKEY_RSA, pkey_rsa_typename },
 	/* .name = */		"ssh-rsa-cert-v01@openssh.com",
 	/* .shortname = */	"RSA-CERT",
@@ -1269,7 +1363,7 @@ const struct sshkey_impl sshkey_rsa_cert_impl = {
 };
 
 const struct sshkey_impl sshkey_rsa_sha256_impl = {
-	/* .id = */		{ KEY_RSA, 0,
+	/* .id = */		{ KEY_RSA, -1,
 				  EVP_PKEY_RSA, pkey_rsa_typename },
 	/* .name = */		"rsa-sha2-256",
 	/* .shortname = */	"RSA",
@@ -1282,7 +1376,7 @@ const struct sshkey_impl sshkey_rsa_sha256_impl = {
 };
 
 const struct sshkey_impl sshkey_rsa_sha512_impl = {
-	/* .id = */		{ KEY_RSA, 0,
+	/* .id = */		{ KEY_RSA, -1,
 				  EVP_PKEY_RSA, pkey_rsa_typename },
 	/* .name = */		"rsa-sha2-512",
 	/* .shortname = */	"RSA",
@@ -1295,7 +1389,7 @@ const struct sshkey_impl sshkey_rsa_sha512_impl = {
 };
 
 const struct sshkey_impl sshkey_rsa_sha256_cert_impl = {
-	/* .id = */		{ KEY_RSA_CERT, 0,
+	/* .id = */		{ KEY_RSA_CERT, -1,
 				  EVP_PKEY_RSA, pkey_rsa_typename },
 	/* .name = */		"rsa-sha2-256-cert-v01@openssh.com",
 	/* .shortname = */	"RSA-CERT",
@@ -1308,7 +1402,7 @@ const struct sshkey_impl sshkey_rsa_sha256_cert_impl = {
 };
 
 const struct sshkey_impl sshkey_rsa_sha512_cert_impl = {
-	/* .id = */		{ KEY_RSA_CERT, 0,
+	/* .id = */		{ KEY_RSA_CERT, -1,
 				  EVP_PKEY_RSA, pkey_rsa_typename },
 	/* .name = */		"rsa-sha2-512-cert-v01@openssh.com",
 	/* .shortname = */	"RSA-CERT",
@@ -1319,3 +1413,9 @@ const struct sshkey_impl sshkey_rsa_sha512_cert_impl = {
 	/* .enabled = */	sshkey_rsa_sha256_enabled,
 	/* .funcs = */		SSHKEY_RSA_FUNCS
 };
+
+const struct ssh_pkey_impl ssh_pkey_rsa_impl = {
+	pkey_rsa_typename,
+	EVP_PKEY_RSA,
+	SSH_PKEY_RSA_FUNCS
+};
