@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002-2024 Roumen Petrov.  All rights reserved.
+ * Copyright (c) 2002-2025 Roumen Petrov.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -175,6 +175,7 @@ done:
 
 static inline int
 ssh_x509_support_plain_type(int k_type) {
+#if 0
 	return (
 	    (k_type == KEY_RSA) ||
 #ifdef OPENSSL_HAS_ECC
@@ -188,6 +189,24 @@ ssh_x509_support_plain_type(int k_type) {
 #endif
 	    0
 	) ? 1 : 0;
+#else
+/* Let check list as it depends on enabled at run time algorithms.
+ * Note that algorithms like ED25519 could be supported by
+ * cryptographic library but disabled at run time by FIPS or
+ * other rules.
+ */
+	int loc;
+	const SSHX509KeyAlgs *xkalg;
+	for (
+	    loc = ssh_xkalg_ind(&xkalg, -1);
+	    loc >= 0;
+	    loc = ssh_xkalg_ind(&xkalg, loc)
+	) {
+		if (k_type == xkalg->base_id->type)
+			return 1;
+	}
+	return 0;
+#endif
 }
 
 
@@ -594,7 +613,7 @@ X509key_from_subject(const char *pkalg, const char *cp, char **ep) {
 	if (ssh_xkalg_nameind(pkalg, &p, -1) < 0)
 		return NULL;
 
-	ret = x509key_from_subject(p->basetype, cp);
+	ret = x509key_from_subject(p->base_id->type, cp);
 }
 
 	if (ret != NULL && ep != NULL) {
@@ -770,10 +789,11 @@ if ((SSHX_RFC6187_MISSING_KEY_IDENTIFIER & xcompat) == 0) {
 	}
 
 	if (xkalg != NULL) {
+		const sshkey_id *base_id = xkalg->base_id;
 		/* check if key match algorithm */
 		if (
-		    (key->type != xkalg->basetype) ||
-		    (key->ecdsa_nid != xkalg->subtype)
+		    (key->type != base_id->type) ||
+		    (key->ecdsa_nid != base_id->subtype)
 		) {
 			r = SSH_ERR_KEY_TYPE_MISMATCH;
 			goto err;
@@ -1107,7 +1127,7 @@ x509key_demote(const struct sshkey *k, struct sshkey *pk) {
 }
 
 
-int
+static int
 X509key_to_buf(const struct sshkey *key, struct sshbuf *b) {
 	X509 *x;
 
@@ -1189,7 +1209,6 @@ x509key_subject(const struct sshkey *key) {
 
 	if (!X509KEY_CHECK(key)) return NULL;
 
-	/* match format used in Xkey_write_subject */
 	dn = X509_get_subject_name(key->x509_data->cert);
 	return ssh_X509_NAME_oneline(dn);
 }
@@ -1234,24 +1253,60 @@ Xkey_write_subject_bio(const char *pkalg, const struct sshkey *key, BIO *out) {
 }
 
 
-int
-Xkey_write_subject(const char *pkalg, const struct sshkey *key, FILE *f) {
-	int ret;
-	BIO  *out;
+static int
+Xkey_write_subject_buf(const char *pkalg, const struct sshkey *key, struct sshbuf *b) {
+	BIO *mbio;
+	int r;
 
-	out = BIO_new_fp(f, BIO_NOCLOSE);
-	if (out == NULL) return 0;
-#ifdef VMS
-	{
-		BIO *tmpbio = BIO_new(BIO_f_linebuffer());
-		out = BIO_push(tmpbio, out);
+	mbio = BIO_new(BIO_s_mem());
+	if (mbio == NULL) return SSH_ERR_ALLOC_FAIL;
+
+	if (!Xkey_write_subject_bio(pkalg, key, mbio)) {
+		r = SSH_ERR_INTERNAL_ERROR;
+		goto out;
 	}
-#endif
-	ret = Xkey_write_subject_bio(pkalg, key, out);
 
-	(void)BIO_flush(out);
-	BIO_free_all(out);
-	return ret;
+{	char *data;
+	long len;
+	len = BIO_get_mem_data(mbio, &data);
+	r  = sshbuf_put(b, data, len);
+}
+
+out:
+	BIO_free_all(mbio);
+	return r;
+}
+
+
+int
+Xkey_printb(const char *pkalg, const struct sshkey *key, struct sshbuf *b) {
+	/* x509key_write prints X.509 certificate in blob format :-( */
+	return sshkey_is_x509(key)
+	    ? Xkey_write_subject_buf(pkalg, key, b)
+	    : sshkey_format_text(key, b);
+}
+
+
+int
+Xkey_printf(const char *pkalg, const struct sshkey *key, FILE *f) {
+	struct sshbuf *b;
+	int r;
+
+	b = sshbuf_new();
+	if (b == NULL) return SSH_ERR_ALLOC_FAIL;
+
+	r = Xkey_printb(pkalg, key, b);
+	if (r != 0) goto out;
+
+	if (fwrite(sshbuf_ptr(b), sshbuf_len(b), 1, f) != 1) {
+		r = SSH_ERR_SYSTEM_ERROR;
+		if (feof(f))
+			errno = EPIPE;
+	}
+
+out:
+	sshbuf_free(b);
+	return r;
 }
 
 
@@ -1527,7 +1582,7 @@ ssh_x509key_type(const char *name) {
 	if (ssh_xkalg_nameind(name, &p, -1) < 0)
 		return KEY_UNSPEC;
 
-	return p->basetype;
+	return p->base_id->type;
 }
 
 
@@ -1619,30 +1674,6 @@ plain_alg:
 }
 
 
-static int
-ssh_x509_validate_public(const struct sshkey *key) {
-	EVP_PKEY *pk = key->pk;
-
-	if (pk == NULL) return SSH_ERR_INVALID_ARGUMENT;
-
-{	int evp_id = EVP_PKEY_base_id(pk);
-	switch (evp_id) {
-	case EVP_PKEY_RSA:	return ssh_pkey_validate_public_rsa(pk);
-#ifdef WITH_DSA
-	case EVP_PKEY_DSA:	return ssh_pkey_validate_public_dsa(pk);
-#endif
-#ifdef OPENSSL_HAS_ECC
-	case EVP_PKEY_EC:	return ssh_pkey_validate_public_ecdsa(pk);
-#endif
-#ifdef USE_PKEY_ED25519
-	case EVP_PKEY_ED25519:	return 0 /* TODO? */;
-#endif
-	}
-}
-	return SSH_ERR_KEY_TYPE_UNKNOWN;
-}
-
-
 static int
 ssh_x509_sign(
 	const SSHX509KeyAlgs *xkalg, ssh_sign_ctx *ctx,
@@ -1659,7 +1690,7 @@ ssh_x509_sign(
 	debug3_f("compatibility: { 0x%08x, 0x%08x }",
 	    ctx->compat->datafellows, ctx->compat->extra);
 
-	r = ssh_x509_validate_public(key);
+	r = sshkey_validate_public(key);
 	if (r != 0) return r;
 
 	/* compute signature */
@@ -1687,6 +1718,7 @@ ssh_x509_sign(
 #ifdef HAVE_EVP_DIGESTSIGNINIT /* OpenSSL >= 1.0 */
 {	size_t slen;
 	if (ssh_pkey_sign(&dgst, key->pk, NULL, &slen, data, datalen) <= 0) {
+		do_log_crypto_errors(SYSLOG_LEVEL_ERROR);
 		r = SSH_ERR_LIBCRYPTO_ERROR;
 		goto done;
 	}
@@ -1746,7 +1778,7 @@ ssh_x509_verify(
 		return SSH_ERR_INVALID_ARGUMENT;
 	}
 
-	r = ssh_x509_validate_public(key);
+	r = sshkey_validate_public(key);
 	if (r != 0) return r;
 
 	pubkey = X509_get_pubkey(key->x509_data->cert);
@@ -2088,43 +2120,9 @@ xkey_to_blob(const char *pkalg, const struct sshkey *key, u_char **blobp, u_int
 
 
 int
-X509key_from_buf(struct sshbuf *b, struct sshkey **keyp) {
-	int r;
-	struct sshkey *key;
-
-	if (b == NULL) return SSH_ERR_INVALID_ARGUMENT;
-
-{	const u_char *blob = sshbuf_ptr(b);
-	size_t blen = sshbuf_len(b);
-
-	r = X509key_from_blob(blob, blen, &key);
-	if (r != SSH_ERR_SUCCESS) goto done;
-}
-{	/* rewind buffer */
-	size_t blen = i2d_X509(key->x509_data->cert, NULL);
-
-	r = sshbuf_consume(b, blen);
-	if (r != SSH_ERR_SUCCESS) goto done;
-}
-
-	if (keyp != NULL) {
-		*keyp = key;
-		key = NULL;
-	}
-
-done:
-	sshkey_free(key);
-	return r;
-}
-
-
-int
-Akey_puts_opts(
-	const struct sshkey *key, struct sshbuf *b,
-	enum sshkey_serialize_rep opts
-) {
+Akey_puts(const struct sshkey *key, struct sshbuf *b) {
 	if (!sshkey_is_x509(key))
-		return sshkey_puts_opts(key, b, opts);
+		return sshkey_puts(key, b);
 
 {	X509 *x;
 	SSH_X509 *xd;
