@@ -1,4 +1,4 @@
-/* $OpenBSD: ssh.c,v 1.619 2025/09/25 07:05:11 djm Exp $ */
+/* $OpenBSD: ssh.c,v 1.622 2025/12/22 01:17:31 djm Exp $ */
 /*
  * Author: Tatu Ylonen <ylo@cs.hut.fi>
  * Copyright (c) 1995 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
@@ -552,7 +552,7 @@ resolve_canonicalize(char **hostp, int port)
 }
 
 /*
- * Check the result of indentity loading, ignoring some errors and either
+ * Check the result of identity loading, ignoring some errors and either
  * discarding the key or fatal()ing for others.
  */
 static void
@@ -694,6 +694,8 @@ valid_ruser(const char *s)
 	if (*s == '-')
 		return 0;
 	for (i = 0; s[i] != 0; i++) {
+		if (iscntrl((u_char)s[i]))
+			return 0;
 		if (strchr("'`\";&<>|(){}", s[i]) != NULL)
 			return 0;
 		/* Disallow '-' after whitespace */
@@ -883,6 +885,12 @@ main(int ac, char **av)
 				muxclient_command = SSHMUX_COMMAND_CANCEL_FWD;
 			else if (strcmp(optarg, "proxy") == 0)
 				muxclient_command = SSHMUX_COMMAND_PROXY;
+			else if (strcmp(optarg, "conninfo") == 0)
+				muxclient_command = SSHMUX_COMMAND_CONNINFO;
+			else if (strcmp(optarg, "chaninfo") == 0)
+				muxclient_command = SSHMUX_COMMAND_CHANINFO;
+			else if (strcmp(optarg, "channels") == 0)
+				muxclient_command = SSHMUX_COMMAND_CHANINFO;
 			else
 				fatal("Invalid multiplex command.");
 			break;
