@@ -1,4 +1,4 @@
-/* $OpenBSD: ssh_api.c,v 1.32 2024/10/18 05:14:51 djm Exp $ */
+/* $OpenBSD: ssh_api.c,v 1.33 2025/11/13 10:35:14 dtucker Exp $ */
 /*
  * Copyright (c) 2012 Markus Friedl.  All rights reserved.
  * Copyright (c) 2014-2025 Roumen Petrov.  All rights reserved.
@@ -526,10 +526,18 @@ _ssh_order_hostkeyalgs(struct ssh *ssh)
 	*replace = '\0';
 	while ((alg = strsep(&avail, ",")) && *alg != '\0') {
 		int ktype, nid;
+		const struct sshkey_impl *impl;
 
-		ktype = sshkey_type_from_name(alg);
-		if (ktype == KEY_UNSPEC) continue;
-		nid = sshkey_ecdsa_nid_from_name(alg);
+		impl = sshkey_impl_from_name(alg);
+		if (impl == NULL) continue;
+
+		ktype = impl->id.type;
+		nid = impl->id.subtype;
+		/* The code below is not logical - it ignores the algoritms
+		 * instead of sorting them according to the comment.
+		 * Note that function is used only by unit tests, so keep as
+		 * is instead to make it working.
+		 */
 
 		TAILQ_FOREACH(k, &ssh->public_keys, next) {
 			if (k->key->type != ktype &&
