@@ -1656,41 +1656,38 @@ ssh_login(struct ssh *ssh, Sensitive *sensitive, const char *orighost,
 }
 
 /* print all known host keys for a given host, but skip keys of given type */
+extern const struct sshkey_impl sshkey_ed25519_impl;
+extern const struct sshkey_impl sshkey_ecdsa_nistp256_impl;
+extern const struct sshkey_impl sshkey_ecdsa_nistp384_impl;
+extern const struct sshkey_impl sshkey_ecdsa_nistp521_impl;
+extern const struct sshkey_impl sshkey_rsa_impl;
+extern const struct sshkey_impl sshkey_dss_impl;
+
+/* TODO add list function */
+static const struct sshkey_impl * const keyimpls[] = {
+	&sshkey_ed25519_impl,
+	&sshkey_ecdsa_nistp256_impl,
+	&sshkey_ecdsa_nistp384_impl,
+	&sshkey_ecdsa_nistp521_impl,
+	&sshkey_rsa_impl,
+	&sshkey_dss_impl
+};
+
 static int
 show_other_keys(struct hostkeys *hostkeys, struct sshkey *key)
 {
-	int type[] = {
-		KEY_RSA,
-	#ifdef WITH_DSA
-		KEY_DSA,
-	#endif
-		KEY_ECDSA,
-		KEY_ED25519,
-		-1
-	};
-	int subtype[] = {
-#ifdef OPENSSL_HAS_ECC
-		NID_X9_62_prime256v1,
-		NID_secp384r1,
-# ifdef OPENSSL_HAS_NISTP521
-		NID_secp521r1,
-# endif
-#endif /* OPENSSL_HAS_ECC */
-		-1 /* non-ECC keys has to be processed once */,
-		-1
-	};
-	int i, ret = 0;
+	int ret = 0;
+	size_t k;
 	char *fp, *ra;
 	const struct hostkey_entry *found;
 
-	for (i = 0; type[i] != -1; i++) {
-	int k;
-	k = (type[i] == KEY_ECDSA) ? 0 : (sizeof(subtype) / sizeof(*subtype) - 2);
-	do {
-		if (type[i] == key->type && subtype[k] == key->ecdsa_nid)
-			goto nextsubtype;
-		if (!lookup_key_in_hostkeys_by_types(hostkeys, type[i], subtype[k], &found))
-			goto nextsubtype;
+	for (k = 0; k < sizeof(keyimpls)/sizeof(keyimpls[0]); k++) {
+		const struct sshkey_impl *impl = keyimpls[k];
+
+		if (sshkey_match_impl(key, impl))
+			continue;
+		if (!lookup_key_in_hostkeys_by_impl(hostkeys, impl, &found))
+			continue;
 		fp = sshkey_fingerprint(found->key,
 		    options.fingerprint_hash, SSH_FP_DEFAULT);
 		ra = sshkey_fingerprint(found->key,
@@ -1713,9 +1710,6 @@ show_other_keys(struct hostkeys *hostkeys, struct sshkey *key)
 		free(ra);
 		free(fp);
 		ret = 1;
-nextsubtype:
-		k++;
-	} while(subtype[k] != -1);
 	}
 	return ret;
 }
