@@ -109,7 +109,6 @@ order_hostkeyalgs(char *host, struct sockaddr *hostaddr, u_short port,
 	char *oavail, *avail, *first, *last, *alg, *hostname, *ret;
 	size_t maxlen;
 	struct hostkeys *hostkeys;
-	int ktype;
 	u_int i;
 
 	UNUSED(cinfo);
@@ -139,21 +138,22 @@ order_hostkeyalgs(char *host, struct sockaddr *hostaddr, u_short port,
 
 {	int has_ca_mark = lookup_marker_in_hostkeys(hostkeys, MRK_CA);
 	while ((alg = strsep(&avail, ",")) && *alg != '\0') {
-		int subtype;
-		sshkey_types_from_name(alg, &ktype, &subtype);
-		if (ktype == KEY_UNSPEC)
+		const struct sshkey_impl *impl;
+
+		impl = sshkey_impl_from_name(alg);
+		if (impl == NULL)
 			fatal_f("unknown alg %s", alg);
 		/*
 		 * If we have a @cert-authority marker in known_hosts then
 		 * prefer all certificate algorithms.
 		 */
-		if (has_ca_mark && sshkey_type_is_cert(ktype)) {
+		if (has_ca_mark && impl->cert) {
 			ALG_APPEND(first, alg);
 			continue;
 		}
 		/* If the key appears in known_hosts then prefer it */
 		if (lookup_key_in_hostkeys_by_types(hostkeys,
-		    sshkey_type_plain(ktype), subtype, NULL)) {
+		    sshkey_type_plain(impl->id.type), impl->id.subtype, NULL)) {
 			ALG_APPEND(first, alg);
 			continue;
 		}
@@ -1653,7 +1653,7 @@ try_identity(struct ssh *ssh, Identity *id)
 	 */
 	const char *pkalg0;
 
-	if ((key->type == KEY_RSA_CERT) &&
+	if (sshkey_is_rsa_cert(key) &&
 	    !ssh_compat_fellows(ssh, SSH_BUG_SIGTYPE)) {
 		pkalg = "rsa-sha2-256-cert-v01@openssh.com";
 		pkalg0 = "rsa-sha2-256";
