@@ -104,22 +104,14 @@ list_hostkey_types(void)
 		key = sensitive_data.host_certificates[i];
 		if (key == NULL)
 			continue;
-		switch (key->type) {
-		case KEY_RSA_CERT:
+		if (sshkey_is_rsa_cert(key)) {
 			/* for RSA we also support SHA2 signatures */
 			append_hostkey_type(b,
 			    "rsa-sha2-256-cert-v01@openssh.com");
 			append_hostkey_type(b,
 			    "rsa-sha2-512-cert-v01@openssh.com");
-			/* FALLTHROUGH */
-	#ifdef WITH_DSA
-		case KEY_DSA_CERT:
-	#endif
-		case KEY_ECDSA_CERT:
-		case KEY_ED25519_CERT:
-			append_hostkey_type(b, sshkey_ssh_name(key));
-			break;
 		}
+		append_hostkey_type(b, sshkey_ssh_name(key));
 	}
 	if ((ret = sshbuf_dup_string(b)) == NULL)
 		fatal_f("sshbuf_dup_string failed");
