@@ -1,4 +1,4 @@
-/* $OpenBSD: sshd-session.c,v 1.16 2025/09/25 06:45:50 djm Exp $ */
+/* $OpenBSD: sshd-session.c,v 1.18 2025/12/16 08:32:50 dtucker Exp $ */
 /*
  * SSH2 implementation:
  * Privilege Separation:
@@ -336,31 +336,23 @@ privsep_postauth(struct ssh *ssh, Authctxt *authctxt)
 }
 
 static struct sshkey *
-get_hostkey_by_type(int type, int subtype, int need_private, struct ssh *ssh)
+get_hostkey_by_impl(const struct sshkey_impl *impl, int need_private, struct ssh *ssh)
 {
 	u_int i;
 	struct sshkey *key;
 
 	UNUSED(ssh);
 	for (i = 0; i < options.num_host_key_files; i++) {
-		switch (type) {
-		case KEY_RSA_CERT:
-	#ifdef WITH_DSA
-		case KEY_DSA_CERT:
-	#endif
-		case KEY_ECDSA_CERT:
-		case KEY_ED25519_CERT:
+		if (impl->cert) {
 			key = sensitive_data.host_certificates[i];
-			break;
-		default:
+		} else {
 			key = sensitive_data.host_keys[i];
 			if (key == NULL && !need_private)
 				key = sensitive_data.host_pubkeys[i];
-			break;
 		}
-		if (key != NULL && key->type == type &&
-		    (subtype == -1 || subtype == key->ecdsa_nid)
-		)
+		if (key == NULL) continue;
+
+		if (sshkey_match_impl(key, impl))
 			return need_private ?
 			    sensitive_data.host_keys[i] : key;
 	}
@@ -398,28 +390,32 @@ struct sshkey *
 get_hostkey_public_by_alg(const char* alg, struct ssh *ssh)
 {
 	struct sshkey *key;
+	const struct sshkey_impl *impl;
 
 	key = get_hostkey_by_alg(alg, 0, ssh);
 	if (key != NULL) return key;
 
-{	int keytype, subtype;
-	sshkey_types_from_name(alg, &keytype, &subtype);
-	return get_hostkey_by_type(keytype, subtype, 0, ssh);
-}
+	impl = sshkey_impl_from_name(alg);
+	if (impl == NULL)
+		fatal_f("unknown alg %s", alg);
+
+	return get_hostkey_by_impl(impl, 0, ssh);
 }
 
 struct sshkey *
 get_hostkey_private_by_alg(const char* alg, struct ssh *ssh)
 {
 	struct sshkey *key;
+	const struct sshkey_impl *impl;
 
 	key = get_hostkey_by_alg(alg, 1, ssh);
 	if (key != NULL) return key;
 
-{	int keytype, subtype;
-	sshkey_types_from_name(alg, &keytype, &subtype);
-	return get_hostkey_by_type(keytype, subtype, 1, ssh);
-}
+	impl = sshkey_impl_from_name(alg);
+	if (impl == NULL)
+		fatal_f("unknown alg %s", alg);
+
+	return get_hostkey_by_impl(impl, 1, ssh);
 }
 
 struct sshkey *
@@ -701,7 +697,7 @@ Xsshd_hostkey_sign(
 		} else {
 			ssh_sign_ctx a_ctx = { ctx->alg, NULL, ctx->compat,
 			    ctx->provider, ctx->pin };
-			/* mimic logic in priviledged mode */
+			/* mimic logic in privileged mode */
 			if (sshkey_is_cert(pubkey)) {
 				int nxd = get_hostkey_index(pubkey, 0, ssh);
 				if (nxd < 0)
