@@ -1045,14 +1045,13 @@ main(int ac, char **av)
 	int r, opt, on = 1, do_dump_cfg = 0, already_daemon, remote_port;
 	int sock_in = -1, sock_out = -1, newsock = -1;
 	const char *remote_ip, *rdomain;
-	char *fp, *line, *laddr, *logfile = NULL;
+	char *line, *laddr, *logfile = NULL;
 	int config_s[2] = { -1 , -1 };
 	u_int i, j;
 	u_int64_t ibytes, obytes;
 	mode_t new_umask;
 	struct sshkey *key;
 	struct sshkey *pubkey;
-	int keytype;
 	Authctxt *authctxt;
 	struct connection_info *connection_info = NULL;
 
@@ -1438,9 +1437,7 @@ main(int ac, char **av)
 		if (key == NULL && pubkey != NULL && have_agent) {
 			debug("will rely on agent for hostkey %s",
 			    options.host_key_files[i]);
-			keytype = pubkey->type;
 		} else if (key != NULL) {
-			keytype = key->type;
 			accumulate_host_timing_secret(cfg, key);
 		} else {
 			do_log2(ll, "Unable to load host key: %s",
@@ -1450,24 +1447,26 @@ main(int ac, char **av)
 			continue;
 		}
 
-		switch (keytype) {
-		case KEY_RSA:
-	#ifdef WITH_DSA
-		case KEY_DSA:
-	#endif
-		case KEY_ECDSA:
-		case KEY_ED25519:
+	{	const struct sshkey_impl *impl;
+
+		impl = sshkey_impl_from_key(pubkey);
+		if (impl == NULL)
+			fatal_r(SSH_ERR_INTERNAL_ERROR,
+			    "Fail to process host key: %s",
+			    options.host_key_files[i]);
+		if (!impl->cert) /* TODO? */
 			if (have_agent || key != NULL)
 				sensitive_data.have_ssh2_key = 1;
-			break;
-		}
+	{	char *fp;
 		if ((fp = sshkey_fingerprint(pubkey, options.fingerprint_hash,
 		    SSH_FP_DEFAULT)) == NULL)
 			fatal("sshkey_fingerprint failed");
 		debug("%s host key #%d: %s %s",
-		    key ? "private" : "agent", i, sshkey_ssh_name(pubkey), fp);
+		    key ? "private" : "agent", i, impl->name, fp);
 		free(fp);
 	}
+	} /*impl*/
+	} /*loop*/
 	accumulate_host_timing_secret(cfg, NULL);
 	if (!sensitive_data.have_ssh2_key) {
 		logit("sshd: no hostkeys available -- exiting.");
