@@ -40,7 +40,7 @@
 #include "log.h"
 
 
-#ifdef HAVE_EVP_KEM_FETCH
+#ifdef HAVE_EVP_KEYMGMT_GET0_PROVIDER	/* OpenSSL >= 3.0.0 */
 /* TODO: move to cryptographic library initialisation */
 OSSL_LIB_CTX *pkixssh_libctx = NULL;
 char *pkixssh_propq = NULL;
@@ -248,105 +248,33 @@ err:
 #endif /*def USE_EVP_PKEY_KEYGEN*/
 
 
-extern int sshkey_from_pkey_rsa(EVP_PKEY *pk, struct sshkey **keyp);
-#ifdef WITH_DSA
-extern int sshkey_from_pkey_dsa(EVP_PKEY *pk, struct sshkey **keyp);
-#endif /*def WITH_DSA*/
-#ifdef OPENSSL_HAS_ECC
-extern int sshkey_from_pkey_ecdsa(EVP_PKEY *pk, struct sshkey **keyp);
-#endif /* OPENSSL_HAS_ECC */
-
-#ifdef USE_PKEY_ED25519
-static int
-sshkey_from_pkey_ed25519(EVP_PKEY *pk, struct sshkey **keyp) {
-	int r;
-	struct sshkey* key;
-	char *raw_pk = NULL, *raw_sk = NULL;
-	size_t len;
-
-	key = sshkey_new(KEY_UNSPEC);
-	if (key == NULL)
-		return SSH_ERR_ALLOC_FAIL;
-
-	if ((raw_pk = calloc(1, ED25519_PK_SZ)) == NULL ||
-	    (raw_sk = calloc(1, ED25519_SK_SZ)) == NULL
-	) {
-		r = SSH_ERR_ALLOC_FAIL;
-		goto err;
-	}
-
-	len = ED25519_PK_SZ;
-	if (!EVP_PKEY_get_raw_public_key(pk, raw_pk, &len)) {
-		r = SSH_ERR_LIBCRYPTO_ERROR;
-		goto err;
-	}
-	if (len != ED25519_PK_SZ) {
-		r = SSH_ERR_INVALID_FORMAT;
-		goto err;
-	}
-
-	/* private part is not required */
-	len = ED25519_SK_SZ - ED25519_PK_SZ;
-	if (!EVP_PKEY_get_raw_private_key(pk, raw_sk, &len))
-		goto skip_private;
-	if (len != (ED25519_SK_SZ - ED25519_PK_SZ)) {
-		r = SSH_ERR_INVALID_FORMAT;
-		goto err;
-	}
-	/* append the public key to private to match internal format */
-	memcpy(raw_sk + len, raw_pk, ED25519_PK_SZ);
-skip_private:
-
-	key->type = KEY_ED25519;
-	key->pk = pk;
-
-	/* success */
-	SSHKEY_DUMP(key);
-	*keyp = key;
-	/* free raw values; TODO */
-	key->ed25519_pk = raw_pk;
-	key->ed25519_sk = raw_sk;
-	return 0;
-
-err:
-	free(raw_pk);
-	free(raw_sk);
-	sshkey_free(key);
-	return r;
-}
-#endif /*def USE_PKEY_ED25519*/
-
 int
 sshkey_from_pkey(EVP_PKEY *pk, struct sshkey **keyp) {
-	int r, evp_id;
+	const struct ssh_pkey_impl *impl;
 
 	/* NOTE do not set flags |= SSHKEY_FLAG_EXT !!! */
-	evp_id = EVP_PKEY_base_id(pk);
-	switch (evp_id) {
-	case EVP_PKEY_RSA:
-		r = sshkey_from_pkey_rsa(pk, keyp);
-		break;
-#ifdef WITH_DSA
-	case EVP_PKEY_DSA:
-		r = sshkey_from_pkey_dsa(pk, keyp);
-		break;
-#endif
-#ifdef OPENSSL_HAS_ECC
-	case EVP_PKEY_EC:
-		r = sshkey_from_pkey_ecdsa(pk, keyp);
-		break;
-#endif /*def OPENSSL_HAS_ECC*/
-#ifdef USE_PKEY_ED25519
-	case EVP_PKEY_ED25519:
-		r = sshkey_from_pkey_ed25519(pk, keyp);
-		break;
-#endif /*def USE_PKEY_ED25519*/
-	default:
-		error_f("unsupported pkey type %d", evp_id);
-		r = SSH_ERR_KEY_TYPE_UNKNOWN;
-	}
+#ifdef ENABLE_KEY_PROVIDERS
+{	const char *s = EVP_PKEY_get0_type_name(pk);
+	if (s == NULL) goto try_base_id;
 
-	return r;
+	impl = ssh_pkey_impl_from_type_name(s);
+	if (impl == NULL || impl->funcs == NULL) {
+		error_f("unsupported pkey name '%s'", s);
+		return SSH_ERR_KEY_TYPE_UNKNOWN;
+	}
+	return impl->funcs->sshkey_from_pkey(pk, keyp);
+}
+try_base_id:
+#endif /*def ENABLE_KEY_PROVIDERS*/
+{	int evp_id = EVP_PKEY_base_id(pk);
+	impl = ssh_pkey_impl_from_evp_id(evp_id);
+	if (impl == NULL || impl->funcs == NULL) {
+		error_f("unsupported pkey type %d", evp_id);
+		return SSH_ERR_KEY_TYPE_UNKNOWN;
+	}
+}
+
+	return impl->funcs->sshkey_from_pkey(pk, keyp);
 }
 
 
@@ -452,7 +380,7 @@ ssh_EVP_PKEY_eq(const EVP_PKEY *a, const EVP_PKEY *b) {
  * Note that some providers fail to export public part.
  * Another point is that public public part depends on key type i.e.,
  * it is quite complex to write "compare" for all key types.
- * The code bellow relies on provider key-management "match.
+ * The code below relies on provider key-management "match.
  */
 {	int r;
 	int len;
@@ -574,7 +502,7 @@ sshkey_private_to_bio(struct sshkey *key, BIO *bio,
 }
 
 
-/* methods used localy only in ssh-keygen.c */
+/* methods used locally only in ssh-keygen.c */
 extern int
 sshkey_public_to_fp(struct sshkey *key, FILE *fp, int format);
 
@@ -1166,7 +1094,10 @@ ssh_kem_allowed(const char *algorithm) {
 	int ret;
 	EVP_KEM *kem;
 
+	ERR_set_mark();
 	kem = EVP_KEM_fetch(pkixssh_libctx, algorithm, pkixssh_propq);
+	/* clear needless errors for unsupported algorithms */
+	ERR_pop_to_mark();
 	ret = (kem != NULL) ? 1 : 0;
 	EVP_KEM_free(kem);
 
@@ -1174,6 +1105,34 @@ ssh_kem_allowed(const char *algorithm) {
 }
 #endif
 
+#ifdef HAVE_EVP_KEYMGMT_FETCH
+int
+ssh_keytype_allowed(const char *algorithm) {
+	int ret;
+	EVP_KEYMGMT *km;
+
+	ERR_set_mark();
+	km = EVP_KEYMGMT_fetch(pkixssh_libctx, algorithm, pkixssh_propq);
+	/* clear needless errors for unsupported algorithms */
+	ERR_pop_to_mark();
+	ret = (km != NULL) ? 1 : 0;
+	EVP_KEYMGMT_free(km);
+
+	return ret;
+}
+#endif
+
+int
+ssh_key_allowed(const char *algorithm, int base_id) {
+#ifdef HAVE_EVP_KEYMGMT_FETCH	/* OpenSSL >= 3.0 */
+	UNUSED(base_id);
+	return ssh_keytype_allowed(algorithm);
+#else
+	UNUSED(algorithm);
+	return ssh_pkey_allowed(base_id);
+#endif
+}
+
 int
 ssh_pkey_sign(
 	const ssh_evp_md *dgst, EVP_PKEY *privkey,
@@ -1337,6 +1296,46 @@ ssh_pkey_verify_r(
 		: SSH_ERR_LIBCRYPTO_ERROR;
 }
 
+
+/* supported pkey types */
+extern const struct ssh_pkey_impl ssh_pkey_ed25519_impl;
+extern const struct ssh_pkey_impl ssh_pkey_ec_impl;
+extern const struct ssh_pkey_impl ssh_pkey_rsa_impl;
+extern const struct ssh_pkey_impl ssh_pkey_dsa_impl;
+
+static const struct ssh_pkey_impl * const pkey_impls[] = {
+	&ssh_pkey_ed25519_impl,
+	&ssh_pkey_ec_impl,
+	&ssh_pkey_rsa_impl,
+	&ssh_pkey_dsa_impl,
+	NULL
+};
+
+const struct ssh_pkey_impl*
+ssh_pkey_impl_from_type_name(const char *type_name) {
+	const struct ssh_pkey_impl *const* p;
+
+	for (p = pkey_impls; *p != NULL; p++) {
+		const char *const *q = (*p)->type_name;
+		for (; *q != NULL; q++) {
+			if (strcmp(*q, type_name) == 0)
+				return *p;
+		}
+	}
+	return NULL;
+}
+
+const struct ssh_pkey_impl *
+ssh_pkey_impl_from_evp_id(int evp_id) {
+	const struct ssh_pkey_impl *const* p;
+
+	for (p = pkey_impls; *p != NULL; p++) {
+		if ((*p)->evp_id == evp_id)
+			return *p;
+	}
+	return NULL;
+}
+
 #else
 
 typedef int sshkey_crypto_empty_translation_unit;
