@@ -1,4 +1,4 @@
-/* $OpenBSD: sshkey.c,v 1.154 2025/09/05 10:23:55 dtucker Exp $ */
+/* $OpenBSD: sshkey.c,v 1.156 2025/11/07 04:11:59 djm Exp $ */
 /*
  * Copyright (c) 2000, 2001 Markus Friedl.  All rights reserved.
  * Copyright (c) 2008 Alexander von Gernler.  All rights reserved.
@@ -135,7 +135,11 @@ sshkey_impl_from_type(int type)
 	return NULL;
 }
 
-static const struct sshkey_impl *
+/*static - TODO temporary, see hostfile.c*/
+extern const struct sshkey_impl *
+sshkey_impl_from_type_nid(int type, int nid);
+
+const struct sshkey_impl *
 sshkey_impl_from_type_nid(int type, int nid)
 {
 	const struct sshkey_impl *const* p;
@@ -149,7 +153,7 @@ sshkey_impl_from_type_nid(int type, int nid)
 	return NULL;
 }
 
-static const struct sshkey_impl *
+const struct sshkey_impl *
 sshkey_impl_from_key(const struct sshkey *k)
 {
 	if (k == NULL) return NULL;
@@ -157,6 +161,44 @@ sshkey_impl_from_key(const struct sshkey *k)
 	return sshkey_impl_from_type_nid(k->type, k->ecdsa_nid);
 }
 
+static const struct sshkey_impl*
+impl_from_name(const char *name, int allow_short)
+{
+{	const SSHX509KeyAlgs *p;
+	const sshkey_id *id;
+	if (ssh_xkalg_nameind(name, &p, -1) < 0) goto plain;
+
+	id = p->base_id;
+	return sshkey_impl_from_type_nid(id->type, id->subtype);
+}
+plain:
+{	const struct sshkey_impl *const* p;
+	for (p = keyimpls; *p != NULL; p++) {
+		if (!(*p)->enabled()) continue;
+		if (strcmp((*p)->name, name) == 0)
+			return *p;
+		/* only plain key type could match shortname */
+		if (allow_short && !(*p)->cert &&
+		    strcasecmp((*p)->shortname, name) == 0)
+			return *p;
+	}
+}
+	return NULL;
+}
+
+const struct sshkey_impl*
+sshkey_impl_from_name(const char *name)
+{
+	return impl_from_name(name, 0);
+}
+
+const struct sshkey_impl*
+sshkey_impl_from_shortname(const char *name)
+{
+	return impl_from_name(name, 1);
+}
+
+
 const char *
 sshkey_type(const struct sshkey *k)
 {
@@ -245,12 +287,6 @@ sshkey_type_from_name(const char *name)
 	return type_from_name(name, 0);
 }
 
-int
-sshkey_type_from_shortname(const char *name)
-{
-	return type_from_name(name, 1);
-}
-
 static int
 key_type_is_ecdsa_variant(int type)
 {
@@ -262,43 +298,6 @@ key_type_is_ecdsa_variant(int type)
 	return 0;
 }
 
-void
-sshkey_types_from_name(const char *name, int *type, int *subtype) {
-
-	if (type == NULL || subtype == NULL) {
-		fatal_f("NULL arguments");
-		return; /* unreachible code */
-	}
-
-	*type = sshkey_type_from_name((char*)name);
-
-	if (key_type_is_ecdsa_variant(*type))
-		*subtype = sshkey_ecdsa_nid_from_name(name);
-	else
-		*subtype = -1;
-}
-
-int
-sshkey_ecdsa_nid_from_name(const char *name)
-{
-	if (strncmp(name, "x509v3-ecdsa-", 13) == 0) {
-		return sshkey_ecdsa_nid_from_name(name+7);
-	}
-
-{
-	const struct sshkey_impl *const* p;
-
-	for (p = keyimpls; *p != NULL; p++) {
-		if (!(*p)->enabled()) continue;
-		if (!key_type_is_ecdsa_variant((*p)->id.type))
-			continue;
-		if (strcmp((*p)->name, name) == 0)
-			return (*p)->id.subtype;
-	}
-}
-	return -1;
-}
-
 char *
 sshkey_alg_list(int certs_only, int plain_only, int include_sigonly, char sep)
 {
@@ -496,101 +495,6 @@ sshkey_type_certified(int type)
 	}
 }
 
-#ifdef WITH_OPENSSL
-#ifdef OPENSSL_HAS_ECC
-/* XXX: these are really begging for a table-driven approach */
-int
-sshkey_curve_name_to_nid(const char *name)
-{
-	if (strcmp(name, "nistp256") == 0)
-		return NID_X9_62_prime256v1;
-	else if (strcmp(name, "nistp384") == 0)
-		return NID_secp384r1;
-# ifdef OPENSSL_HAS_NISTP521
-	else if (strcmp(name, "nistp521") == 0)
-		return NID_secp521r1;
-# endif /* OPENSSL_HAS_NISTP521 */
-	else
-		return -1;
-}
-
-u_int
-sshkey_curve_nid_to_bits(int nid)
-{
-	switch (nid) {
-	case NID_X9_62_prime256v1:
-		return 256;
-	case NID_secp384r1:
-		return 384;
-# ifdef OPENSSL_HAS_NISTP521
-	case NID_secp521r1:
-		return 521;
-# endif /* OPENSSL_HAS_NISTP521 */
-	default:
-		return 0;
-	}
-}
-
-int
-sshkey_ecdsa_bits_to_nid(int bits)
-{
-	switch (bits) {
-	case 256:
-		return NID_X9_62_prime256v1;
-	case 384:
-		return NID_secp384r1;
-# ifdef OPENSSL_HAS_NISTP521
-	case 521:
-		return NID_secp521r1;
-# endif /* OPENSSL_HAS_NISTP521 */
-	default:
-		return -1;
-	}
-}
-
-const char *
-sshkey_curve_nid_to_name(int nid)
-{
-	switch (nid) {
-	case NID_X9_62_prime256v1:
-		return "nistp256";
-	case NID_secp384r1:
-		return "nistp384";
-# ifdef OPENSSL_HAS_NISTP521
-	case NID_secp521r1:
-		return "nistp521";
-# endif /* OPENSSL_HAS_NISTP521 */
-	default:
-		return NULL;
-	}
-}
-
-int
-sshkey_ec_nid_to_hash_alg(int nid)
-{
-	int kbits = sshkey_curve_nid_to_bits(nid);
-
-	if (kbits <= 0)
-		return -1;
-
-	/* RFC5656 section 6.2.1 */
-	if (kbits <= 256)
-		return SSH_DIGEST_SHA256;
-	else if (kbits <= 384)
-		return SSH_DIGEST_SHA384;
-	else
-		return SSH_DIGEST_SHA512;
-}
-
-#else /*ndef OPENSSL_HAS_ECC*/
-
-int   sshkey_curve_name_to_nid(const char *name)	{ (void)name;	return -1; }
-u_int sshkey_curve_nid_to_bits(int nid)			{ (void)nid;	return  0; }
-int   sshkey_ecdsa_bits_to_nid(int bits)		{ (void)bits;	return -1; }
-
-#endif /*ndef OPENSSL_HAS_ECC*/
-#endif /* WITH_OPENSSL */
-
 static void
 cert_free(struct sshkey_cert *cert)
 {
@@ -793,21 +697,21 @@ sshkey_equal(const struct sshkey *a, const struct sshkey *b)
 
 int/*bool*/
 sshkey_match_pkalg(struct sshkey *key, const char* pkalg) {
-	int keytype, subtype;
+	const struct sshkey_impl *impl;
 
 	if (key == NULL) return 0;
 	if (pkalg == NULL) return 0;
 
-	sshkey_types_from_name(pkalg, &keytype, &subtype);
+	impl = sshkey_impl_from_name(pkalg);
+	if (impl == NULL) return 0;
 	return (
-	    (keytype == key->type) &&
-	    (subtype == key->ecdsa_nid)
+	    (impl->id.type == key->type) &&
+	    (impl->id.subtype == key->ecdsa_nid)
 	);
 }
 
 static int
-to_blob_buf(const struct sshkey *key, struct sshbuf *b, int force_plain,
-  enum sshkey_serialize_rep opts)
+to_blob_buf(const struct sshkey *key, struct sshbuf *b, int force_plain)
 {
 	int type;
 	const struct sshkey_impl *impl;
@@ -820,7 +724,7 @@ to_blob_buf(const struct sshkey *key, struct sshbuf *b, int force_plain,
 	impl = sshkey_impl_from_type_nid(type, key->ecdsa_nid);
 	if (impl == NULL) return SSH_ERR_KEY_TYPE_UNKNOWN;
 
-	if (sshkey_type_is_cert(type)) {
+	if (impl->cert) {
 		if (key->cert == NULL)
 			return SSH_ERR_EXPECTED_CERT;
 		if (sshbuf_len(key->cert->certblob) == 0)
@@ -832,25 +736,24 @@ to_blob_buf(const struct sshkey *key, struct sshbuf *b, int force_plain,
 {	int r = sshbuf_put_cstring(b, impl->name);
 	if (r != 0) return r;
 }
-	return impl->funcs->serialize_public(key, b, opts);
+	return impl->funcs->serialize_public(impl, key, b);
 }
 
 int
 sshkey_putb(const struct sshkey *key, struct sshbuf *b)
 {
-	return to_blob_buf(key, b, 0, SSHKEY_SERIALIZE_DEFAULT);
+	return to_blob_buf(key, b, 0);
 }
 
 int
-sshkey_puts_opts(const struct sshkey *key, struct sshbuf *b,
-    enum sshkey_serialize_rep opts)
+sshkey_puts(const struct sshkey *key, struct sshbuf *b)
 {
 	struct sshbuf *tmp;
 	int r;
 
 	if ((tmp = sshbuf_new()) == NULL)
 		return SSH_ERR_ALLOC_FAIL;
-	r = to_blob_buf(key, tmp, 0, opts);
+	r = to_blob_buf(key, tmp, 0);
 	if (r == 0)
 		r = sshbuf_put_stringb(b, tmp);
 	sshbuf_free(tmp);
@@ -858,22 +761,15 @@ sshkey_puts_opts(const struct sshkey *key, struct sshbuf *b,
 }
 
 #if 1 /* used in unit tests only */
-int
-sshkey_puts(const struct sshkey *key, struct sshbuf *b)
-{
-	return sshkey_puts_opts(key, b, SSHKEY_SERIALIZE_DEFAULT);
-}
-
 int
 sshkey_putb_plain(const struct sshkey *key, struct sshbuf *b)
 {
-	return to_blob_buf(key, b, 1, SSHKEY_SERIALIZE_DEFAULT);
+	return to_blob_buf(key, b, 1);
 }
 #endif
 
 static int
-to_blob(const struct sshkey *key, u_char **blobp, size_t *lenp, int force_plain,
-    enum sshkey_serialize_rep opts)
+to_blob(const struct sshkey *key, u_char **blobp, size_t *lenp, int force_plain)
 {
 	int ret = SSH_ERR_INTERNAL_ERROR;
 	size_t len;
@@ -885,7 +781,7 @@ to_blob(const struct sshkey *key, u_char **blobp, size_t *lenp, int force_plain,
 		*blobp = NULL;
 	if ((b = sshbuf_new()) == NULL)
 		return SSH_ERR_ALLOC_FAIL;
-	if ((ret = to_blob_buf(key, b, force_plain, opts)) != 0)
+	if ((ret = to_blob_buf(key, b, force_plain)) != 0)
 		goto out;
 	len = sshbuf_len(b);
 	if (lenp != NULL)
@@ -906,14 +802,14 @@ to_blob(const struct sshkey *key, u_char **blobp, size_t *lenp, int force_plain,
 int
 sshkey_to_blob(const struct sshkey *key, u_char **blobp, size_t *lenp)
 {
-	return to_blob(key, blobp, lenp, 0, SSHKEY_SERIALIZE_DEFAULT);
+	return to_blob(key, blobp, lenp, 0);
 }
 
 #if 0 /* unused */
 int
 sshkey_plain_to_blob(const struct sshkey *key, u_char **blobp, size_t *lenp)
 {
-	return to_blob(key, blobp, lenp, 1, SSHKEY_SERIALIZE_DEFAULT);
+	return to_blob(key, blobp, lenp, 1);
 }
 #endif
 
@@ -938,7 +834,7 @@ sshkey_fingerprint_raw(const struct sshkey *k, int dgst_alg,
 		r = SSH_ERR_INVALID_ARGUMENT;
 		goto out;
 	}
-	if ((r = to_blob(k, &blob, &blob_len, 1, SSHKEY_SERIALIZE_DEFAULT))
+	if ((r = to_blob(k, &blob, &blob_len, 1))
 	    != 0)
 		goto out;
 	if ((ret = calloc(1, SSH_DIGEST_MAX_LENGTH)) == NULL) {
@@ -1245,10 +1141,10 @@ sshkey_read_pkalg(struct sshkey *ret, char **cpp, char **pkalg)
 {
 	struct sshkey *k;
 	char pkname[64];
-	char *cp, *blobcopy;
+	char *cp;
 	size_t space;
-	int r, type, curve_nid = -1;
-	struct sshbuf *blob;
+	const struct sshkey_impl *impl;
+	int r;
 
 	if (ret == NULL)
 		return SSH_ERR_INVALID_ARGUMENT;
@@ -1267,10 +1163,10 @@ sshkey_read_pkalg(struct sshkey *ret, char **cpp, char **pkalg)
 	strncpy(pkname, cp, space);
 	pkname[space] = '\0';
 
-	sshkey_types_from_name(pkname, &type, &curve_nid);
-	if (type == KEY_UNSPEC)
+	impl = sshkey_impl_from_name(pkname);
+	if (impl == NULL)
 		return SSH_ERR_INVALID_FORMAT;
-	if (ret->type != KEY_UNSPEC && ret->type != type)
+	if (ret->type != KEY_UNSPEC && ret->type != impl->id.type)
 		return SSH_ERR_KEY_TYPE_MISMATCH;
 
 	/* skip whitespace */
@@ -1287,12 +1183,14 @@ sshkey_read_pkalg(struct sshkey *ret, char **cpp, char **pkalg)
 	}
 }
 
-	if ((blob = sshbuf_new()) == NULL)
+{	struct sshbuf *blob = sshbuf_new();
+	if (blob == NULL)
 		return SSH_ERR_ALLOC_FAIL;
 
 	/* find end of keyblob and decode */
 	space = strcspn(cp, " \t");
-	if ((blobcopy = strndup(cp, space)) == NULL) {
+{	char *blobcopy = strndup(cp, space);
+	if (blobcopy == NULL) {
 		sshbuf_free(blob);
 		return SSH_ERR_ALLOC_FAIL;
 	}
@@ -1302,19 +1200,22 @@ sshkey_read_pkalg(struct sshkey *ret, char **cpp, char **pkalg)
 		return r;
 	}
 	free(blobcopy);
+}
 	r = parse_key_from_blob(sshbuf_ptr(blob), sshbuf_len(blob), &k, NULL);
 	if (r != 0) {
 		sshbuf_free(blob);
 		return r;
 	}
 	sshbuf_free(blob);
+}
 
 	/* ensure type of blob matches type at start of line */
-	if (k->type != type) {
+	if (k->type != impl->id.type) {
 		sshkey_free(k);
 		return SSH_ERR_KEY_TYPE_MISMATCH;
 	}
-	if (sshkey_type_plain(type) == KEY_ECDSA && curve_nid != k->ecdsa_nid) {
+	if (key_type_is_ecdsa_variant(impl->id.type) &&
+	    impl->id.subtype != k->ecdsa_nid) {
 		sshkey_free(k);
 		return SSH_ERR_EC_CURVE_MISMATCH;
 	}
@@ -1325,7 +1226,7 @@ noblob:
 	cp += strspn(cp, " \t");;
 
 	/* Fill in ret from parsed key */
-	ret->type = type;
+	ret->type = impl->id.type;
 	if (sshkey_is_cert(ret)) {
 		if (!sshkey_is_cert(k)) {
 			sshkey_free(k);
@@ -1336,14 +1237,9 @@ noblob:
 		ret->cert = k->cert;
 		k->cert = NULL;
 	}
-{	const struct sshkey_impl *impl = sshkey_impl_from_type(
-	    sshkey_type_plain(ret->type));
-	if (impl == NULL) {
-		sshkey_free(k);
-		return SSH_ERR_INTERNAL_ERROR;
-	}
+
 	impl->funcs->move_public(k, ret);
-}
+
 	/* Test with loaded key, as for X.509 certificates
 	   we consider keys as equivalent if base type match
 	 */
@@ -1952,11 +1848,31 @@ sshkey_check_length(const struct sshkey *k)
 }
 
 
+int
+sshkey_validate_public(const struct sshkey *k) {
+	const char *const*type_name;
+
+{	const struct sshkey_impl *impl;
+	impl = sshkey_impl_from_key(k);
+	if (impl == NULL)
+		return SSH_ERR_INVALID_ARGUMENT;
+	type_name = impl->id.type_name;
+}
+{	const struct ssh_pkey_impl *impl;
+	impl = ssh_pkey_impl_from_type_name(type_name[0]);
+	if (impl == NULL || impl->funcs == NULL)
+		return SSH_ERR_INVALID_ARGUMENT;
+	return impl->funcs->validate_public(k->pk);
+}
+}
+
+
 static int
 sshkey_from_blob_internal(struct sshbuf *b, struct sshkey **keyp,
     int allow_cert)
 {
-	int type, ret = SSH_ERR_INTERNAL_ERROR;
+	const struct sshkey_impl *impl;
+	int ret = SSH_ERR_INTERNAL_ERROR;
 	char *ktype = NULL;
 	struct sshkey *key = NULL;
 	struct sshbuf *copy;
@@ -1975,35 +1891,37 @@ sshkey_from_blob_internal(struct sshbuf *b, struct sshkey **keyp,
 		ret = SSH_ERR_INVALID_FORMAT;
 		goto out;
 	}
-
-	type = sshkey_type_from_name(ktype);
 	debug3_f("ktype=%.100s", ktype);
-	if (!allow_cert && sshkey_type_is_cert(type)) {
+
+	impl = sshkey_impl_from_name(ktype);
+	if (impl == NULL) {
+		ret = SSH_ERR_KEY_TYPE_UNKNOWN;
+		goto out;
+	}
+
+	if (!allow_cert && impl->cert) {
 		ret = SSH_ERR_KEY_CERT_INVALID_SIGN_KEY;
 		goto out;
 	}
-	if ((key = sshkey_new(type)) == NULL) {
+	if ((key = sshkey_new(impl->id.type)) == NULL) {
 		ret = SSH_ERR_ALLOC_FAIL;
 		goto out;
 	}
 	/* skip nonce that precedes all custom certificates */
-	if (sshkey_type_is_cert(type)) {
+	if (impl->cert) {
 		if (sshbuf_get_string_direct(b, NULL, NULL) != 0) {
 			ret = SSH_ERR_INVALID_FORMAT;
 			goto out;
 		}
 	}
-{	const struct sshkey_impl *impl = sshkey_impl_from_type(type);
-	if (impl == NULL) {
-		ret = SSH_ERR_KEY_TYPE_UNKNOWN;
-		goto out;
-	}
-	ret = impl->funcs->deserialize_public(ktype, b, key);
+	ret = impl->funcs->deserialize_public(impl, b, key);
 	if (ret != SSH_ERR_SUCCESS) goto out;
-}
+
 	/* parse custom certificate portion */
-	if (sshkey_is_cert(key) && (ret = cert_parse(b, key, copy)) != 0)
-		goto out;
+	if (impl->cert) {
+		ret = cert_parse(b, key, copy);
+		if (ret != 0) goto out;
+	}
 
 	if (key != NULL && sshbuf_len(b) != 0) {
 		ret = SSH_ERR_INVALID_FORMAT;
@@ -2316,8 +2234,7 @@ sshkey_certify_custom(struct sshkey *k, struct sshkey *ca, const char *alg,
 		goto out;
 
 	/* Public key next */
-	if ((ret = impl->funcs->serialize_public(k, cert,
-	    SSHKEY_SERIALIZE_DEFAULT)) != 0)
+	if ((ret = impl->funcs->serialize_public(impl, k, cert)) != 0)
 		goto out;
 
 	/* Then remaining cert fields */
@@ -2518,9 +2435,8 @@ sshkey_format_cert_validity(const struct sshkey_cert *cert, char *s, size_t l)
 	return strlcpy(s, ret, l);
 }
 
-static int
-sshkey_private_serialize_opt(struct sshkey *key, struct sshbuf *buf,
-    enum sshkey_serialize_rep opts)
+int
+sshkey_private_serialize(struct sshkey *key, struct sshbuf *buf)
 {
 	const struct sshkey_impl *impl;
 	const char *pkalg = NULL;
@@ -2531,7 +2447,7 @@ sshkey_private_serialize_opt(struct sshkey *key, struct sshbuf *buf,
 	impl = sshkey_impl_from_key(key);
 	if (impl == NULL) return SSH_ERR_INVALID_ARGUMENT;
 
-	if (sshkey_type_is_cert(key->type)) {
+	if (impl->cert) {
 		if (key->cert == NULL ||
 		    sshbuf_len(key->cert->certblob) == 0)
 			return SSH_ERR_INVALID_ARGUMENT;
@@ -2542,15 +2458,15 @@ sshkey_private_serialize_opt(struct sshkey *key, struct sshbuf *buf,
 	was_shielded = sshkey_is_shielded(key);
 	if ((r = sshkey_unshield_private(key)) != 0)
 		goto out;
-	pkalg = sshkey_ssh_name(key);
+	pkalg = sshkey_ssh_name(key); /* NOTE: X.509 identities */
 	if ((r = sshbuf_put_cstring(b, pkalg)) != 0)
 		goto out;
-	if (sshkey_type_is_cert(key->type)) {
+	if (impl->cert) {
 		r = sshbuf_put_stringb(b, key->cert->certblob);
 		if (r != 0) goto out;
 	}
 
-	r = impl->funcs->serialize_private(key, b, opts);
+	r = impl->funcs->serialize_private(impl, key, b);
 	if (r != SSH_ERR_SUCCESS) goto out;
 
 	r = X509key_encode_identity(pkalg, key, b);
@@ -2573,26 +2489,24 @@ sshkey_private_serialize_opt(struct sshkey *key, struct sshbuf *buf,
 	return r;
 }
 
-int
-sshkey_private_serialize(struct sshkey *key, struct sshbuf *b)
-{
-	return sshkey_private_serialize_opt(key, b,
-	    SSHKEY_SERIALIZE_DEFAULT);
-}
-
 int
 sshkey_private_deserialize(struct sshbuf *buf, struct sshkey **kp)
 {
+	const struct sshkey_impl *impl;
 	char *tname = NULL;
 	struct sshkey *k = NULL;
-	int type, r = SSH_ERR_INTERNAL_ERROR;
+	int r = SSH_ERR_INTERNAL_ERROR;
 
 	if (kp != NULL)
 		*kp = NULL;
 	if ((r = sshbuf_get_cstring(buf, &tname, NULL)) != 0)
 		goto out;
-	type = sshkey_type_from_name(tname);
-	if (sshkey_type_is_cert(type)) {
+	impl = sshkey_impl_from_name(tname);
+	if (impl == NULL) {
+		r = SSH_ERR_KEY_TYPE_UNKNOWN;
+		goto out;
+	}
+	if (impl->cert) {
 		/*
 		 * Certificate key private keys begin with the certificate
 		 * itself. Make sure this matches the type of the enclosing
@@ -2600,30 +2514,20 @@ sshkey_private_deserialize(struct sshbuf *buf, struct sshkey **kp)
 		 */
 		if ((r = sshkey_froms(buf, &k)) != 0)
 			goto out;
-		if (k->type != type) {
-			r = SSH_ERR_KEY_CERT_MISMATCH;
-			goto out;
-		}
-		/* For ECDSA keys, the group must match too */
-		if (k->type == KEY_ECDSA &&
-		    k->ecdsa_nid != sshkey_ecdsa_nid_from_name(tname)) {
+		if (!sshkey_match_impl(k, impl)) {
 			r = SSH_ERR_KEY_CERT_MISMATCH;
 			goto out;
 		}
 	} else {
-		if ((k = sshkey_new(type)) == NULL) {
+		if ((k = sshkey_new(impl->id.type)) == NULL) {
 			r = SSH_ERR_ALLOC_FAIL;
 			goto out;
 		}
 	}
-{	const struct sshkey_impl *impl = sshkey_impl_from_type(type);
-	if (impl == NULL) {
-		r = SSH_ERR_KEY_TYPE_UNKNOWN;
-		goto out;
-	}
-	r = impl->funcs->deserialize_private(tname, buf, k);
+
+	r = impl->funcs->deserialize_private(impl, buf, k);
 	if (r != SSH_ERR_SUCCESS) goto out;
-}
+
 	r = X509key_decode_identity(tname, buf, k);
 	if (r != SSH_ERR_SUCCESS) goto out;
 	/* success */
