@@ -1,4 +1,4 @@
-/* $OpenBSD: umac.c,v 1.26 2025/05/24 02:33:33 dtucker Exp $ */
+/* $OpenBSD: umac.c,v 1.27 2025/09/05 10:34:35 dtucker Exp $ */
 /* -----------------------------------------------------------------------
  *
  * umac.c -- C Implementation UMAC Message Authentication
@@ -157,11 +157,11 @@ typedef unsigned int	UWORD;  /* Register */
 #if defined(WITH_OPENSSL) && defined(HAVE_AES_ENCRYPT)
 /* Note low level AES API is deprecated in OpenSSL 3.0.
  * We will not use it for builds with OpenSSL < 3.0
- * unless build-in rijndael is requested globaly.
+ * unless built-in rijndael is requested globally.
  */
 #  if !defined(USE_BUILTIN_RIJNDAEL) && \
       defined(HAVE_ERR_GET_ERROR_ALL)	/* new is OpenSSL 3.0 */
-#    define USE_BUILTIN_RIJNDAEL	/* force build-in rijndael */
+#    define USE_BUILTIN_RIJNDAEL	/* force built-in rijndael */
 #  endif
 #else
 #  ifndef USE_BUILTIN_RIJNDAEL
@@ -178,7 +178,7 @@ typedef AES_KEY aes_int_key[1];
 #define aes_key_setup(key,int_key)                      \
   AES_set_encrypt_key((const u_char *)(key),UMAC_KEY_LEN*8,int_key)
 #else
-/* build-in rijndael */
+/* built-in rijndael */
 #include "rijndael.h"
 #define AES_ROUNDS ((UMAC_KEY_LEN / 4) + 6)
 typedef UINT8 aes_int_key[AES_ROUNDS+1][4][4];	/* AES internal */
