@@ -735,14 +735,16 @@ ssh_x509store_addldapurl(const char *ldap_url, const char *ldap_ver) {
 		X509_LOOKUP_METHOD* lookup_method;
 		X509_LOOKUP *lookup;
 
-#ifdef USE_X509_LOOKUP_STORE
+#ifdef USE_LDAP_STORE
+# ifdef USE_X509_LOOKUP_STORE
 		#define SSH_X509_LOOKUP_ADD	X509_LOOKUP_add_store
 		lookup_method = X509_LOOKUP_store();
 		load_ldap_engine();
-#elif defined(USE_X509_LOOKUP_MYSTORE)
+# else
 		#define SSH_X509_LOOKUP_ADD	X509_LOOKUP_add_mystore
 		lookup_method = X509_LOOKUP_mystore();
 		load_ldap_engine();
+# endif
 #else
 		#define SSH_X509_LOOKUP_ADD	X509_LOOKUP_add_ldap
 		lookup_method = X509_LOOKUP_ldap();
